#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include "admin_pqlx.h"
#include "admin_externs.h"
GOptionContext *context;
static GOptionEntry entries[] = 
{
{ "dbName", 'd', 0, G_OPTION_ARG_STRING, NULL, "PQLX Database Name - optional", "[SERVER:]pqlxDB" },
{ NULL }
};
static SRVRDB *startDB;
GSList *ntw, *ntwPDF; 
static void printHelp()
{
int argc=2;
char **argv;
GError *error = NULL;
argv = calloc(2, sizeof(char*));
argv[0] = g_strdup("pqlx-admin");
argv[1] = g_strdup("--help");
g_option_context_parse(context, &argc, &argv, &error);
free(argv[0]);
free(argv[1]);
free(argv);
}
static gboolean parseArgs(int *argc, char **argv)
{
gboolean ret=TRUE;
GError   *error = NULL;
char  *dbName = NULL;
entries[0].arg_data = &dbName;
context = g_option_context_new("- Execute pqlx-admin");
g_option_context_add_main_entries(context, entries, NULL);
if (!g_option_context_parse(context, argc, &argv, &error))
{
ret = FALSE;
if (error)
fprintf(stdout, "parse error: %s\n", error->message);
printHelp();
}
if (dbName)
{
startDB = calloc(1, sizeof(SRVRDB));
if (strrchr(dbName, ':'))
{
startDB->name[SRVRDEFAULT] = g_strdup(strtok(dbName, ":"));
startDB->name[DBDEFAULT] = g_strdup(strtok(NULL, ":"));
}
else
{
startDB->name[SRVRDEFAULT] = g_strdup("localhost");
startDB->name[DBDEFAULT] = g_strdup(dbName);
}
}
free(dbName);
return ret;
}
void setSrvrCFGDefaults(SRVREXECCFG *srvrCFG)
{
IFFREE(srvrCFG->identFile);
memset(srvrCFG, 0, sizeof(SRVREXECCFG));
srvrCFG->numCPU = admin.settings.maxCPU;
srvrCFG->dirScan = TRUE;
srvrCFG->refreshSysPNG = FALSE;
srvrCFG->timeRestricted.set = FALSE;
}
int main(int argc, char **argv)
{
int  i;
char execTime[50];
if (!parseArgs(&argc, argv))
{
exit(-1);
}
TIMESTR(execTime);
if (!loggingInit("admin", NULL, execTime, FALSE, FALSE))
{
fprintf(stderr, "Unable to Initialize Logging, Necessarily Quitting...\n");
exit(-1);
}
fprintf(stdout, "pqlx-admin: started\n");
admin.settings.maxCPU = get_ncpu_pqlx();
setSrvrCFGDefaults(&admin.settings.srvrCFG);
gtk_init(&argc, &argv);
make_adminScreens();
gtk_accelerator_set_default_mod_mask(GDK_CONTROL_MASK|GDK_SHIFT_MASK|GDK_MOD1_MASK);
gtk_widget_show_all(topWindow);
for(i=0;i<TTLDATAMODELS;i++)
{ 
admin.sel[i] = NONE;
}
for(i=1;i<TTLPNGPLOTS;i++)
{
gtk_widget_hide(pngPlot[i]);
}
for(i=1;i<TTLADMINTABS;i++)
{
gtk_widget_hide(sideBar[i]);
}
setDispState(DBNOCONN);
readPDFsrvrs();
admin.tab.main = A_INFO;
admin.tab.sub = INFO_DESC;
gtk_window_resize(GTK_WINDOW(topWindow), 1, 1);
gtk_window_set_position(GTK_WINDOW(topWindow), GTK_WIN_POS_CENTER);
if (startDB)
{
g_idle_add((GSourceFunc) initDBConn, startDB);
}
gtk_main();
fprintf(stdout, "pqlx-admin: exited\n");
logging(LOG_SHUTDOWN);
exit(0);
}
