#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#ifndef MAC
#include <values.h>
#endif
#include <sys/stat.h>
#include "pqlx_defs.h"
#include "pql_defines.h"
#include "pqlx_externs.h"
#include "pqlx_stn.h"
#include "gtk_stn_externs.h"
#include "pqlx_pdf.h"
#include "gtk_pdf_externs.h"
#include "util_externs.h"
static int threadCnt;
static GStaticMutex myMutex = G_STATIC_MUTEX_INIT;
static gboolean arrivalsRet(ARRIVALS *arrivals)
{
threadCnt--;
if (arrivals)
stnStates.event.arrivals = g_slist_prepend(stnStates.event.arrivals, arrivals);
if (!threadCnt)
{
if (stnStates.event.arrivals)
setSensitive(STNSYSTEM, STNNOEVENTSENS, STNANALYSIS, TRUE);
else
setSensitive(STNSYSTEM, STNNOEVENTSENS, STNANALYSIS, FALSE);
stnStates.pmap.dirty[STNANALYSIS][0] = TRUE;
makeSTNDrawing(STNANALYSIS);
g_static_mutex_unlock(&myMutex);
}
return FALSE;
}
void clearArrivals(GSList *arrivalsList)
{ 
GSList *iter, *iter2, *iter3;
ARRIVALS *arrivals;
TTIME *phase;
INTERPTT *interpTT;
if (!arrivalsList)
return;
for(iter = arrivalsList;
iter;
iter = g_slist_next(iter))
{
arrivals = iter->data;
free(arrivals->keys.stn);
for(iter2 = arrivals->data.phases;
iter2;
iter2 = g_slist_next(iter2))
{
for(iter3 = iter2->data;
iter3;
iter3 = g_slist_next(iter3))
{
phase = iter3->data;
free(phase->phaseName);
free(phase);
}
g_slist_free(iter2->data);
}
g_slist_free(arrivals->data.phases);
for(iter2 = arrivals->data.arrivals;
iter2;
iter2 = g_slist_next(iter2))
{
interpTT = iter2->data;
free(interpTT->phaseName);
free(interpTT);
}
g_slist_free(arrivals->data.arrivals);
free(arrivals);
}
g_slist_free(arrivalsList);
}
static float latLonDist(float lat1, float lon1, float lat2, float lon2)
{
double a, c, dlon, dlat;
dlon = TORAD(lon2 - lon1);
dlat = TORAD(lat2 - lat1);
a = (pow(sin(dlat/2),2) + cos(TORAD(lat1)) * cos(TORAD(lat2)) * pow(sin(dlon/2),2));
c = 2 * atan2(sqrt(a), sqrt(1-a));
return (float) c*180./G_PI;
}
float backAZ(float lat1, float lon1, float lat2, float lon2)
{
float dLon = lon2 - lon1;
float y, x, BA;
y = sin(TORAD(dLon)) * cos(TORAD(lat2));
x = cos(TORAD(lat1)) * sin(TORAD(lat2)) - 
sin(TORAD(lat1)) * cos(TORAD(lat2)) * cos(TORAD(dLon));
BA = atan2(y, x);
BA = TODEG(BA + 2 * G_PI);
BA = ((int)BA) % 360 + (float) (BA - (int) BA);
return BA;
}
enum {
INTERPDEPTH,
INTERPDIST
};
enum {
D1D1,
D1D2,
D2D1,
D2D2,
TTLDDS
};
#define INTERPOLATETT(a, b, c, d) (double) ((a->ttime - (((c==INTERPDIST ? a->distance : a->depth) - (c==INTERPDIST ? d->keys.distance : d->keys.depth)) / ((c==INTERPDIST ? a->distance : a->depth) - (c==INTERPDIST ? b->distance : b->depth))) * (a->ttime - b->ttime)))
static gint sortArrivals(INTERPTT *int1, INTERPTT *int2, gpointer nil)
{
return(int1->ttime*1000 - int2->ttime*1000);
}
static int doArrivals(ANADATA *anaData)
{
GSList *iter, *phaseList;
SQLDI *sqldi;
CHNMETA *chnMETA;
ARRIVALS *arrivals;
TTIME *phase[TTLDDS];
EVENTINFO *event = stnStates.event.event;
INTERPTT *interpTT;
int  i, j, errNO;
double ttime, firstTtime;
sqldi = calloc(1, sizeof(SQLDI));
chnMETA = anaData->chnMETA->data;
arrivals = calloc(1, sizeof(ARRIVALS));
arrivals->keys.modelKey = stnStates.event.model->key;
arrivals->keys.stn = g_strdup(anaData->trc->data.head.station);
arrivals->keys.depth = event->loc[STNEVTDEP];
arrivals->keys.distance = latLonDist(event->loc[STNEVTLAT], event->loc[STNEVTLON], 
chnMETA->coords[LAT], chnMETA->coords[LON]);
sqldi->args.sqr = anaData->trcNum; 
sqldi->args.dataEvent = GET_DATA;
sqldi->descr.tableData = TTMODELTBL;
sqldi->data = arrivals;
db_request(sqldi);
errNO = sqldi->args.call.errorNO;
free(sqldi);
if (errNO)
{
free(arrivals->keys.stn);
free(arrivals);
g_idle_add((GSourceFunc) arrivalsRet, NULL);
return 0;
}
for(iter = arrivals->data.phases, firstTtime=NONE, i=0;
iter;
iter = g_slist_next(iter), i++)
{
phaseList = iter->data;
if (g_slist_length(phaseList) != 4)
{ 
i--;
continue; 
}
for(j=0;
phaseList;
phaseList = g_slist_next(phaseList), j++)
{ 
phase[j] = phaseList->data;
}
ttime = 0;
if (phase[D1D1]->distance != phase[D1D2]->distance &&
phase[D1D1]->depth != phase[D2D1]->depth)
{ 
ttime = INTERPOLATETT(phase[D1D1], phase[D1D2], INTERPDIST, arrivals);
ttime += INTERPOLATETT(phase[D2D1], phase[D2D2], INTERPDIST, arrivals);
ttime += INTERPOLATETT(phase[D1D1], phase[D2D1], INTERPDEPTH, arrivals);
ttime += INTERPOLATETT(phase[D1D2], phase[D2D2], INTERPDEPTH, arrivals);
ttime /= 4.;
}
else
{ 
if (phase[D1D1]->distance == phase[D1D2]->distance &&
phase[D1D1]->depth == phase[D2D1]->depth)
{ 
ttime = phase[D1D1]->ttime;
}
else
{
if (phase[D1D1]->distance == phase[D1D2]->distance)
{ 
ttime = INTERPOLATETT(phase[D1D1], phase[D2D1], INTERPDEPTH, arrivals);
}
if (phase[D1D1]->depth == phase[D2D1]->depth)
{ 
ttime = INTERPOLATETT(phase[D1D1], phase[D1D2], INTERPDIST, arrivals);
}
}
}
interpTT = calloc(1, sizeof(INTERPTT));
interpTT->phaseName = g_strdup(phase[0]->phaseName);
interpTT->ttime = ttime;
arrivals->data.arrivals = g_slist_prepend(arrivals->data.arrivals, interpTT);
if (!firstTtime ||
ttime < firstTtime)
{
firstTtime = ttime;
}
}
arrivals->data.arrivals = g_slist_sort_with_data(arrivals->data.arrivals, 
(GCompareDataFunc) sortArrivals, NULL); 
g_idle_add((GSourceFunc) arrivalsRet, (gpointer) arrivals);
return 0;
}
static gint getStn(char *stn1, char *stn2)
{
return(strcmp(stn1, stn2));
}
void calcArrivals(gboolean new)
{
GSList *iter, *iter2, *stations=NULL;
ANADATA *anaData;
gboolean threadMade=FALSE;
g_static_mutex_lock(&myMutex); 
switch(new)
{
case TRUE:
clearArrivals(stnStates.event.arrivals);
stnStates.event.arrivals = NULL;
if (!stnStates.event.event)
break;
for(iter = stnStates.analysis.traces;
iter;
iter = g_slist_next(iter))
{
anaData = ((PQLDATA *) (iter->data))->anaData;
if (!anaData->chnMETA)
continue;   
iter2 = g_slist_find_custom(stations, anaData->trc->data.head.station, 
(GCompareFunc) getStn);
if (iter2)
continue;   
stations = g_slist_prepend(stations, anaData->trc->data.head.station);
threadCnt++;
makeThread(ASYNC, (GThreadFunc) doArrivals, anaData, NULL);
threadMade = TRUE;
}
break;
case FALSE:
if (!stnStates.event.event)
break;
for(iter = stnStates.analysis.traces;
iter;
iter = g_slist_next(iter))
{
anaData = ((PQLDATA *) (iter->data))->anaData;
if (!anaData)
continue;
if (!anaData->chnMETA)
continue;   
iter2 = g_slist_find_custom(stnStates.event.arrivals, 
anaData->trc->data.head.station, (GCompareFunc) getStnArrivals);
if (iter2)
continue;   
iter2 = g_slist_find_custom(stations, anaData->trc->data.head.station, 
(GCompareFunc) getStn);
if (iter2)
continue;   
stations = g_slist_prepend(stations, anaData->trc->data.head.station);
threadCnt++;
makeThread(ASYNC, (GThreadFunc) doArrivals, anaData, NULL);
threadMade = TRUE;
}
break;
}
if (stations)
g_slist_free(stations);
if (!threadMade)
g_static_mutex_unlock(&myMutex);
}
