#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <gdk/gdkkeysyms.h>

#include "pqlx_defs.h"
#include "pql_defines.h"
#include "pqlx_externs.h"
#include "pqlx_stn.h"
#include "gtk_stn_externs.h"
#include "util_externs.h"

enum {
STNNOEVENT,  
STNTRCSEL,
STNTRCTFORMDISP,
STNVERTLINE,
STNUSERPICK,
STNPOINT,
STNEVTARRIVALS
};

static void getXY(GtkWidget *wi, int trcNum, int *x, int *y)
{
int trcH = (wi->allocation.height - (ANATOPM + ANABOTM))/
g_slist_length(stnStates.analysis.traces);
gtk_window_get_position(GTK_WINDOW(topWindow), x, y);
(*x) += wi->allocation.x + STNLFTM + 7;
(*y) += wi->allocation.y + ANATOPM + (trcNum)*trcH + trcH/2;
}

static void destroyEvtPhInfo(GtkButton *button, void *nothing)
{
EvtPhInfo = NULL;
return;
}

void evtPhDisp(GtkWidget *widget, gint x1, gint y1)
{
static GtkWidget *view;
static GtkTextBuffer *buffer;
GtkTextIter iter, endIter;
gchar *str;
EVENTINFO *event = stnStates.event.event;
GSList *arrivalsIter, *phaseIter;
ARRIVALS  *arrivals;
INTERPTT *interpTT;

if (!EvtPhInfo)
{
GtkWidget *frame, *phSW;
EvtPhInfo = gtk_window_new(GTK_WINDOW_TOPLEVEL);
g_signal_connect(EvtPhInfo, "destroy", G_CALLBACK(destroyEvtPhInfo), NULL);
frame = gtk_frame_new(NULL);
gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
view = gtk_text_view_new();
gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view), FALSE);
buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
gtk_text_buffer_create_tag(buffer, "heading", "justification", GTK_JUSTIFY_CENTER, 
"family", "monospace", 
NULL);
gtk_text_buffer_create_tag(buffer, "text", "family", "monospace", 
NULL);
phSW = gtk_scrolled_window_new(NULL, NULL);
gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(phSW), GTK_SHADOW_ETCHED_IN);
gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(phSW),
GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
gtk_container_add(GTK_CONTAINER(EvtPhInfo), frame);
gtk_container_add(GTK_CONTAINER(frame), phSW);
gtk_container_add(GTK_CONTAINER(phSW), view);
gtk_widget_show_all(EvtPhInfo);
gtk_widget_hide(EvtPhInfo);
}
else
{
gtk_window_get_position(GTK_WINDOW(EvtPhInfo), &stnStates.event.evtPhInfoX, &stnStates.event.evtPhInfoY);
}

gtk_text_buffer_get_iter_at_offset(buffer, &endIter, -1);
gtk_text_buffer_get_iter_at_offset(buffer, &iter, 0);
if (gtk_text_iter_get_offset(&endIter) > 0)
gtk_text_buffer_delete(buffer, &iter, &endIter);

str = g_strdup_printf("Predicted Arrivals\n");
gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, str, -1, "heading", NULL); free(str);

for(arrivalsIter = stnStates.event.arrivals;
arrivalsIter;
arrivalsIter = g_slist_next(arrivalsIter))
{
arrivals = arrivalsIter->data;
str = g_strdup_printf("\n %s:\t%.3f Km / %.2f deg\n", arrivals->keys.stn, 
(float) (arrivals->keys.distance*111.195), 
arrivals->keys.distance);
gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, str, -1, "text", NULL); free(str);
for(phaseIter = arrivals->data.arrivals;
phaseIter;
phaseIter = g_slist_next(phaseIter))
{
PTIME arrTime;
double tt;
interpTT = phaseIter->data;
tt = event->eventTE + interpTT->ttime;
passcal_etoh(&arrTime, tt);
str = g_strdup_printf("%6s: %04d-%02d-%02d %02d:%02d:%02d.%03d \n", 
interpTT->phaseName, arrTime.yr, arrTime.mo,
arrTime.day, arrTime.hr, arrTime.mn,
(int) arrTime.sec, 
(int) ((arrTime.sec - (int) arrTime.sec)*1000+.5));
gtk_text_buffer_insert_with_tags_by_name(buffer, &iter, str, -1, "text", NULL); free(str);
}
}

if (stnStates.event.evtPhInfoX ||
stnStates.event.evtPhInfoY)
gtk_window_move(GTK_WINDOW(EvtPhInfo), stnStates.event.evtPhInfoX, stnStates.event.evtPhInfoY);
gtk_window_resize(GTK_WINDOW(EvtPhInfo), 300, 400);
gtk_widget_show(EvtPhInfo);
}

static void analysesDisp(GtkWidget *widget, int trcNum)
{
static GtkWidget *view;
static GtkTextBuffer *buffer;
GtkTextIter iter, endIter;
gint x, y;
gchar *str;
ANADATA *anaData;
PQLDATA *pqlData;

pqlData = g_slist_nth_data(stnStates.analysis.traces, trcNum);
anaData = pqlData->anaData;
if (!anaData)
return;  

if (!AnalysesString)
{
GtkWidget  *frame;
AnalysesString = gtk_window_new(GTK_WINDOW_POPUP);
frame = gtk_frame_new(NULL);
gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
view = gtk_text_view_new();
buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW (view));
gtk_container_add(GTK_CONTAINER(AnalysesString), frame);
gtk_container_add(GTK_CONTAINER(frame), view);
gtk_widget_show_all(AnalysesString);
gtk_widget_hide(AnalysesString);
}

gtk_text_buffer_get_iter_at_offset(buffer, &endIter, -1);
gtk_text_buffer_get_iter_at_offset(buffer, &iter, 0);
if (gtk_text_iter_get_offset(&endIter) > 0)
gtk_text_buffer_delete(buffer, &iter, &endIter);
str = g_strdup_printf(" %s ", anaData->tForm.dispStr);
gtk_text_buffer_insert(buffer, &iter, str, -1); free(str);

getXY(widget, trcNum, &x, &y);
gtk_window_move(GTK_WINDOW(AnalysesString), x, y);
gtk_window_resize(GTK_WINDOW(AnalysesString), 1, 1);
gtk_widget_show(AnalysesString);
}

void stnComboSel(GtkComboBox *combo, gpointer which)
{
int i = gtk_combo_box_get_active(combo), 
scr = stnStates.screen, 
comboBox = GPOINTER_TO_INT(which);

switch(comboBox)
{
case STNXSCALE:
switch(stnSettings.display.xScale)
{
case RELATIVE:
case ABSOLUTE:
switch(i)
{
case RELATIVE:
case ABSOLUTE:
stnStates.analysis.plot.limits.selection.user.left =
stnStates.analysis.plot.limits.selection.user.right = 0;
break;
}
break;
}
stnSettings.display.xScale = i;
stnStates.pmap.dirty[scr][0] = TRUE;
break;

case STNZANCHOR:
stnSettings.analysis.anchor = i;
break;
}

makeSTNDrawing(scr);
}

void stnTFormSel(GtkComboBox *combo, gpointer o)
{ 
gboolean original = (gboolean) GPOINTER_TO_INT(o);
switch(original)
{
case TRUE:  
{
GSList *selIter;

stnStates.analysis.tForm.transform = ANAORIG;
stnStates.analysis.plot.limits.selection.user.left = 0;
stnStates.analysis.plot.limits.selection.user.right = 0;
stnStates.analysis.tForm.list = FALSE;
if (stnStates.analysis.tForm.tForms)
{
g_hash_table_destroy(stnStates.analysis.tForm.tForms); 
stnStates.analysis.tForm.tForms = NULL;
}
stnStates.analysis.tForm.common = TRUE;
IFFREE(stnStates.analysis.tForm.tFormStr);
for(selIter = stnStates.analysis.selections;
selIter;
selIter = g_slist_next(selIter))
{ 
PQLDATA *pqlData = selIter->data;
ANADATA *anaData = pqlData->anaData;
anaData->display.selected = FALSE; 
}
makeSTNSelectionList(STNANALYSIS);
}
break;

case FALSE:
stnStates.analysis.tForm.transform = gtk_combo_box_get_active(combo);

g_signal_handler_block(combo, stnSignals2[STNANATFORMSGNL][STNANALYSIS]);
gtk_combo_box_set_active(combo, 0);
g_signal_handler_unblock(combo, stnSignals2[STNANATFORMSGNL][STNANALYSIS]);
break;
}

switch(stnStates.analysis.tForm.transform)
{
case ANASPECTRA:
specPsdDo(SPECTRA_WIN);
break;
case ANAPSD:
specPsdDo(PSD_WIN);
break;
case ANAPARTMOTION:
particleMotion(PM_WIN);
break;
default:
stnStates.pmap.dirty[STNANALYSIS][0] = TRUE;
makeSTNDrawing(STNANALYSIS);
break;
}
}

void cutANA(GtkButton *button, gpointer nil)
{
PQLDATA *pqlOld, *pqlNew;
ANADATA *anaData;
GSList *trcIter, *traces2;
void *chnMETA;
double startE=0, endE=0;
traceInfo *trcOld;
gint  hashKey;

if (stnStates.analysis.plot.limits.selection.pix.left &&
!stnStates.analysis.plot.limits.selection.pix.right)
ANAreMag(NULL, GINT_TO_POINTER(SNAP));   
else if (stnStates.analysis.plot.limits.selection.pix.left &&
stnStates.analysis.plot.limits.selection.pix.right)
ANAreMag(NULL, GINT_TO_POINTER(MAG));   

for(trcIter=stnStates.analysis.selections 
? stnStates.analysis.selections 
: stnStates.analysis.traces,
traces2=(trcIter == stnStates.analysis.selections)
? stnStates.analysis.traces
: stnStates.analysis.selections;
trcIter;
trcIter=g_slist_next(trcIter))
{
pqlOld = trcIter->data;
trcOld = pqlOld->trc;
anaData = (ANADATA *) pqlOld->anaData;

switch(stnSettings.display.xScale)
{
case ABSOLUTE:
startE = stnStates.analysis.plot.limits.times[STARTABS] + 
stnStates.analysis.plot.limits.selection.user.left;
endE = stnStates.analysis.plot.limits.times[STARTABS] + 
stnStates.analysis.plot.limits.selection.user.right;
break;
case RELATIVE:
startE = trcOld->data.head.epochs + 
stnStates.analysis.plot.limits.selection.user.left;
endE = trcOld->data.head.epochs + 
stnStates.analysis.plot.limits.selection.user.right;
break;
}

if (startE > (trcOld->data.head.epochs + trcOld->data.head.length) ||
endE < trcOld->data.head.epochs)
{ 
continue;
}

hashKey = pqlOld->misc.hashKey;   
chnMETA = anaData->chnMETA;    
anaData->chnMETA = NULL;    
freeMyData(ANAINFO, anaData);
free(pqlOld->anaData);
pqlOld->anaData = NULL;

pqlNew = cutTrc(pqlOld, startE, endE);
pqlNew->misc.hashKey = hashKey;   
pqlNew->misc.chnMETA = chnMETA;

freeMyData(TRCINFO, trcOld);   
free(trcOld);
if (traces2)
{
GSList *trc2Iter;
trc2Iter = g_slist_find(traces2, pqlOld); 
if (trc2Iter)
{
trc2Iter->data = pqlNew;
}
}
trcIter->data = pqlNew;   
}

stnStates.analysis.plot.limits.selection.pix.left = 0;  
stnStates.analysis.plot.limits.selection.pix.right = 0;  
stnStates.analysis.plot.limits.selection.user.left = 0;  
stnStates.analysis.plot.limits.selection.user.right = 0; 
stnStates.pmap.dirty[STNANALYSIS][0] = TRUE;
makeSTNDrawing(STNANALYSIS);
}

void STNdispMsg(int action, char *msg)
{
static guint msgID;
static guint msgContext;

if(!msgContext)
msgContext = gtk_statusbar_get_context_id((GtkStatusbar *) statusBarSTN, "DATASCREENS");

if (msgID)
gtk_statusbar_remove((GtkStatusbar *) statusBarSTN, msgContext, msgID);

switch(action)
{
case DISPLAY:
msgID = gtk_statusbar_push((GtkStatusbar *) statusBarSTN, msgContext, msg);
break;

case UNDISPLAY:
gtk_statusbar_pop((GtkStatusbar *) statusBarSTN, msgContext);
break;
}

return;
}

static void dispPointVals(pointID *pointPtr, int pointNum)
{
pointID pointDiff;
char string[255], strsv[255];
static char *str1=NULL, *str2=NULL, *dispStr=NULL, *diffStr=NULL;
int  xVal, yVal, xpos;
static GdkPixmap *pixmap;
GtkWidget *da=STNDA[STNANALYSIS];
pixmap = gdk_pixmap_new(da->window, da->allocation.width, da->allocation.height, -1);
gdk_draw_drawable(pixmap, da->style->fg_gc[GTK_STATE_NORMAL],
stnStates.pmap.analysis[PMAPBASE], 0, 0, 0, 0, -1, -1);

xpos = pointNum ? 5 : 10;
xVal = pointPtr[pointNum].xVal;
yVal = pointPtr[pointNum].yVal;
gdk_draw_line(pixmap, stnSettings.drawing.ctxts[stnStates.screen][STNFGBG][INVGC], 
xVal - xpos, yVal - 10,
xVal + xpos, yVal + 10);
gdk_draw_line(pixmap, stnSettings.drawing.ctxts[stnStates.screen][STNFGBG][INVGC], 
xVal + xpos, yVal -10,
xVal - xpos, yVal + 10);
gdk_draw_drawable(da->window,
da->style->fg_gc[GTK_WIDGET_STATE (da)],
pixmap,
0, 0, 0, 0, -1, -1);
if (pointNum == POINT2)
{
xpos = 10;
xVal = pointPtr[POINT1].xVal;
yVal = pointPtr[POINT1].yVal;
gdk_draw_line(pixmap, stnSettings.drawing.ctxts[stnStates.screen][STNFGBG][INVGC], 
xVal - xpos, yVal - 10,
xVal + xpos, yVal + 10);
gdk_draw_line(pixmap, stnSettings.drawing.ctxts[stnStates.screen][STNFGBG][INVGC], 
xVal + xpos, yVal -10,
xVal - xpos, yVal + 10);
gdk_draw_drawable(da->window,
da->style->fg_gc[GTK_WIDGET_STATE (da)],
pixmap,
0, 0, 0, 0, -1, -1);
}
g_object_unref(pixmap);

sprintf(string, "[ #%d: Time = %4d %03d:%02d:%02d:%06.3f / ", 
pointNum+1, pointPtr[pointNum].timeVal.yr,
julian(&pointPtr[pointNum].timeVal), pointPtr[pointNum].timeVal.hr, 
pointPtr[pointNum].timeVal.mn, pointPtr[pointNum].timeVal.sec);
strcpy(strsv, string);

if (pointPtr[pointNum].minAmp == pointPtr[pointNum].maxAmp)
{
if (stnSettings.display.units[STNANALYSIS] == STNCOUNTS)
sprintf(string, "Amp = %d ", (int) pointPtr[pointNum].minAmp);
else
sprintf(string, "Amp = %6.8f ", pointPtr[pointNum].minAmp);
}
else
{
if (stnSettings.display.units[STNANALYSIS] == STNCOUNTS)
sprintf(string, "Amp Range = < %d ~ %d > ",
(int) pointPtr[pointNum].minAmp, (int) pointPtr[pointNum].maxAmp);
else
sprintf(string, "Amp Range = < %6.8f ~ %6.8f > ",
pointPtr[pointNum].minAmp, pointPtr[pointNum].maxAmp);
}
strcat(string, "] ");
strcat(strsv, string);

switch(pointNum)
{
case POINT1:
if (str1)
free(str1);
if (str2)
{
free(str2);  str2 = NULL;
free(diffStr); diffStr = NULL;
}
dispStr = str1 = g_strdup(strsv);
break;
case POINT2:
str2 = g_strdup(strsv);

memset(&pointDiff, 0, sizeof(pointDiff));
pointDiff.epoch = (pointPtr[POINT2].epoch - pointPtr[POINT1].epoch);
pointDiff.maxAmp = (pointPtr[POINT2].maxAmp - pointPtr[POINT1].maxAmp);
pointDiff.minAmp = (pointPtr[POINT2].minAmp - pointPtr[POINT1].minAmp);

sprintf(string, "[ DELTA: Time = %6.3f secs / ", pointDiff.epoch);
strcpy(strsv, string);

if (pointDiff.minAmp == pointDiff.maxAmp)
{
sprintf(string, "Amp = %d ] ", (int) pointDiff.minAmp);
}
else
{
sprintf(string, "Amp Range = < %d ~ %d > ] ", 
(int) pointDiff.minAmp, 
(int) pointDiff.maxAmp);
}
strcat(strsv, string);

strcat(strsv, str2);

strcat(strsv, str1);
diffStr = g_strdup(strsv);
dispStr = diffStr;
break;
}
STNdispMsg(DISPLAY, dispStr);

return;
}

static void setANABounds(int type)
{
double diff, ratio, middle;
graph *localGR = &stnStates.analysis.plot.gr[LOCALGR];
graph *mainGR = &stnStates.analysis.plot.gr[MAINGR];

switch(type)
{
case ZOOMX:
ratio = (float) (stnStates.analysis.plot.limits.selection.pix.right -    
stnStates.analysis.plot.limits.selection.pix.left) /
(float) (localGR->pix_xmax - localGR->pix_xmin);
ratio = 1. - ratio;
diff = PIX2USERX(localGR, stnStates.analysis.plot.limits.selection.pix.right) -
PIX2USERX(localGR, stnStates.analysis.plot.limits.selection.pix.left);
middle = PIX2USERX(localGR, stnStates.analysis.plot.limits.selection.pix.right) - diff/2.;

diff = (stnStates.analysis.plot.limits.selection.user.right -
stnStates.analysis.plot.limits.selection.user.left);
diff += ratio*diff;
stnStates.analysis.plot.limits.selection.user.right += middle + diff/2.;
stnStates.analysis.plot.limits.selection.user.left += middle - diff/2.;
break;
}

if (stnStates.analysis.plot.limits.selection.user.left < mainGR->user_xmin)
{
stnStates.analysis.plot.limits.selection.user.left = mainGR->user_xmin;
}
if (stnStates.analysis.plot.limits.selection.user.right > mainGR->user_xmax)
{
stnStates.analysis.plot.limits.selection.user.right = mainGR->user_xmax;
}
}

gboolean ctrlKeyEventANA(GtkWidget *widget, GdkEventKey *event, gpointer s)
{
guint  modifiers;
int   ctrlKey;
gboolean retVal = TRUE;

modifiers = gtk_accelerator_get_default_mod_mask();
ctrlKey = ((event->state & modifiers) == GDK_CONTROL_MASK) ? CTRL : NONE;

switch (ctrlKey)
{
case CTRL:
if (event->keyval == GDK_z &&
event->type == GDK_KEY_RELEASE)
{
stnTFormSel(NULL, GINT_TO_POINTER(TRUE));
retVal = FALSE;   
}
break;
}

return retVal;
}

gboolean mouseEventANA(GtkWidget *widget, GdkEventButton *event, gpointer s)
{
int   height=widget->allocation.height, width=widget->allocation.width;
int   trcNum=0, trcCnt, trcH=0, mouseEvent = STNNOEVENT, vertPos=0;
int   screen = GPOINTER_TO_INT(s);
gboolean selected=FALSE;
traceInfo *trc=NULL;
PQLDATA  *pqlData;
GdkPixmap *pmap=NULL;
guint  modifiers;
GtkWidget *da=STNDA[screen];

if (!g_slist_length(stnStates.analysis.traces))
return TRUE;

if (stnStates.user.keyActive != DKEY &&
stnStates.user.keyActive != ZKEY &&
stnStates.user.keyActive != XKEY )
{
modifiers = gtk_accelerator_get_default_mod_mask();
stnStates.user.keyActive = ((event->state & modifiers) == GDK_CONTROL_MASK) ? CTRL : NONE;
}

if (stnStates.event.event &&
event->y < ANATOPMSG &&
screen == STNANALYSIS)
mouseEvent = STNEVTARRIVALS;
#if 0
else
if (EvtPhInfo)
gtk_widget_hide(EvtPhInfo);
#endif

if (event->x < ANALFTM &&
event->y > ANATOPM &&
event->y < height - ANABOTM)
mouseEvent = STNTRCSEL;

if (event->x > ANALFTM &&
event->x < width - ANARGTM &&
event->y > ANATOPM &&
event->y < height - ANABOTM &&
stnStates.user.keyActive == NONE)
{
if (stnStates.event.curPick == NONE)
mouseEvent = STNVERTLINE;
else
mouseEvent = STNUSERPICK;
}

if (event->x > ANALFTM &&
event->x < width - ANARGTM &&
event->y > ANATOPM &&
event->y < height - ANABOTM &&
stnStates.user.keyActive == CTRL)
mouseEvent = STNPOINT;

if (!mouseEvent)
return TRUE;   

switch(mouseEvent)
{
case STNTRCSEL:
case STNVERTLINE:
case STNUSERPICK:
case STNEVTARRIVALS:
if (event->type != GDK_BUTTON_RELEASE)
return TRUE;   
break;
case STNPOINT:
stnStates.user.mouseDown = (event->type == GDK_BUTTON_RELEASE ? FALSE : TRUE);
if (!stnStates.user.mouseDown)
return TRUE;   
break;   
}

switch(mouseEvent)
{
case STNEVTARRIVALS: 
evtPhDisp(widget, event->x, event->y);
break;

case STNTRCSEL:  
{
ANADATA  *anaData=NULL;
switch(screen)
{ 
case STNANALYSIS:
{
trcCnt = g_slist_length(stnStates.analysis.traces);
trcH = (height-(ANATOPM+ANABOTM))/trcCnt;
trcNum = (event->y-ANATOPM)/trcH;
pqlData = g_slist_nth_data(stnStates.analysis.traces, trcNum);
anaData = pqlData->anaData;
trc = anaData->trc;
pmap = stnStates.pmap.analysis[PMAPVISIBLE];
selected = anaData->display.selected = !anaData->display.selected;
vertPos = ANATOPM + trcNum*trcH;
}
break;
}

if (!stnSettings.display.overlay[stnStates.screen])
{ 
printSTNLabel(pmap, trc, trcNum, screen, selected, stnSettings.display.overlay[screen], NULL);
gtk_widget_queue_draw_area(widget, 0, vertPos, STNLFTM, trcH);
makeSTNSelectionList(screen);
}
else
{ 
if (trc->magnify.begin != -1)
{
stn_mag_plot(pmap, &trc->magnify.GR[ANAPLOT], trc, anaData, 
trc->magnify.plot.increment, trcNum, STNANALYSIS);
gtk_widget_queue_draw_area(widget, 0, 0, width, height);
}
}
}
break;

case STNVERTLINE:
{
static GdkPixmap *pixmap;
pixmap = gdk_pixmap_new(da->window, width, height, -1);
gdk_draw_drawable(pixmap, da->style->fg_gc[GTK_STATE_NORMAL],
stnStates.pmap.analysis[PMAPBASE], 0, 0, 0, 0, -1, -1);
if (stnStates.analysis.plot.limits.selection.pix.right ||
!stnStates.analysis.plot.limits.selection.pix.left)
{
gtk_widget_hide(STNMag);
gtk_widget_show(STNSnap);
stnStates.analysis.plot.limits.selection.pix.left = event->x;
stnStates.analysis.plot.limits.selection.pix.right = 0;
}
else
{
gtk_widget_hide(STNSnap);
gtk_widget_show(STNMag);
stnStates.analysis.plot.limits.selection.pix.right = event->x;
}

if (stnStates.analysis.plot.limits.selection.pix.left)
gdk_draw_line(pixmap, stnSettings.drawing.ctxts[stnStates.screen][STNFGBG][INVGC],
stnStates.analysis.plot.limits.selection.pix.left, ANATOPM-1, 
stnStates.analysis.plot.limits.selection.pix.left, height-ANABOTM);
if (stnStates.analysis.plot.limits.selection.pix.right)
gdk_draw_line(pixmap, stnSettings.drawing.ctxts[stnStates.screen][STNFGBG][INVGC],
stnStates.analysis.plot.limits.selection.pix.right, ANATOPM-1, 
stnStates.analysis.plot.limits.selection.pix.right, height-ANABOTM);

gdk_draw_drawable(widget->window,
widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
pixmap,
0, 0, 0, 0, -1, -1);
g_object_unref(pixmap);
if (stnStates.analysis.plot.limits.selection.pix.right &&
stnStates.analysis.plot.limits.selection.pix.left >
stnStates.analysis.plot.limits.selection.pix.right)
{ 
int tmp = stnStates.analysis.plot.limits.selection.pix.left;
stnStates.analysis.plot.limits.selection.pix.left = 
stnStates.analysis.plot.limits.selection.pix.right;
stnStates.analysis.plot.limits.selection.pix.right = tmp;
}
}
break;

case STNUSERPICK:
{
int trcCnt = g_slist_length(stnStates.analysis.traces), 
trcH = (height - (ANATOPM + ANABOTM))/trcCnt,
trcNum;
PQLDATA  *pqlData;
ANADATA  *anaData;
USERPICK *userPick, *userP1;
GSList  *iter, *trcIter;
graph  *gr;
char  *stnNm;
double  time;

dispMsgSTN(UNDISPLAY, NULL);
trcNum = (event->y - ANATOPM) / trcH;
pqlData = g_slist_nth_data(stnStates.analysis.traces, trcNum);
anaData = pqlData->anaData;
trc = anaData->trc;
gr = &trc->magnify.GR[ANAPLOT];
stnNm = trc->data.head.station;
time = PIX2USERX(gr, (int) event->x);

for(trcIter = stnStates.analysis.traces;
trcIter;
trcIter = g_slist_next(trcIter))
{
pqlData = trcIter->data;
anaData = pqlData->anaData;
trc = anaData->trc;
if (strcmp(trc->data.head.station, stnNm))
continue;

userPick = calloc(1, sizeof(USERPICK));
userPick->pickNum = stnStates.event.curPick;
userPick->time = time;
for (iter = anaData->userPicks;
iter;
iter = g_slist_next(iter))
{ 
userP1 = iter->data;
if (userP1->pickNum == userPick->pickNum)
{
free(userP1);
anaData->userPicks = g_slist_remove(anaData->userPicks, userP1);
break;
}
}
anaData->userPicks = g_slist_prepend(anaData->userPicks, userPick);
}
stnStates.event.curPick = NONE;
stnStates.pmap.dirty[stnStates.screen][0] = TRUE;
makeSTNDrawing(stnStates.screen);
}
break;

case STNPOINT:
{
int trcCnt = g_slist_length(stnStates.analysis.traces), 
trcH = (height - (ANATOPM + ANABOTM))/trcCnt,
trcNum;
PQLDATA *pqlData;

trcNum = (event->y - ANATOPM) / trcH;
pqlData = g_slist_nth_data(stnStates.analysis.traces, trcNum);
if (pqlData)
{
pointID *pointPtr;
int  pointNum=POINT1;
if (stnStates.analysis.plot.extra.user.point[POINT1].xVal && 
stnStates.analysis.plot.extra.user.point[POINT2].xVal)
{ 
memset(&stnStates.analysis.plot.extra.user.point[POINT1], 0, sizeof(pointID));
memset(&stnStates.analysis.plot.extra.user.point[POINT2], 0, sizeof(pointID));
pointNum = POINT1;
}
else if (stnStates.analysis.plot.extra.user.point[POINT1].xVal)
{ 
pointNum = POINT2;
}
pointPtr = &stnStates.analysis.plot.extra.user.point[pointNum];
ANAPoint(pqlData, (int) event->x, pointPtr);
dispPointVals(&stnStates.analysis.plot.extra.user.point[POINT1], pointNum);
}
}
}

if (event->type == GDK_BUTTON_RELEASE)
{
switch(stnStates.display.user.keyActive)
{
case ZKEY:
if (stnStates.analysis.plot.limits.selection.pix.right)
{
ANAreMag((GtkWidget *)TRUE, GINT_TO_POINTER(MAG));
}
break;
case XKEY:
if (stnStates.analysis.plot.limits.selection.pix.right)
{
setANABounds(ZOOMX);
ANAreMag((GtkWidget *)TRUE, GINT_TO_POINTER(KEY));
}
break;
}
}

return TRUE;
}

gboolean dragEventANA(GtkWidget *widget, GdkEventMotion *event, gpointer s)
{
static GdkWindow *gdkWindow;
GdkModifierType  state;
gint  x, y;
gint  dragEvent=STNNOEVENT, height=widget->allocation.height, 
width=widget->allocation.width;
int   trcNum=0, trcCnt, trcH, screen = GPOINTER_TO_INT(s);

if (!gdkWindow)
gdkWindow = gdk_get_default_root_window();

trcCnt = g_slist_length(stnStates.analysis.traces);
if (!trcCnt)
{
gdk_window_get_pointer(gdkWindow, &x, &y, &state);
return TRUE;
}

#if 0
if (stnStates.event.event &&
event->y < ANATOPMSG &&
*scr == STNANALYSIS)
dragEvent = STNEVTARRIVALS;
else
if (EvtPhInfo)
gtk_widget_hide(EvtPhInfo);
#endif

if (event->x < STNLFTM &&
(event->y > ANATOPM &&
event->y < (height - ANABOTM)) &&
screen == STNANALYSIS &&
!stnStates.analysis.tForm.common)
dragEvent = STNTRCTFORMDISP;

if (event->x > ANALFTM &&
event->x < width - ANARGTM &&
event->y > ANATOPM &&
event->y < height - ANABOTM &&
stnStates.user.keyActive == CTRL &&
stnStates.user.mouseDown)
dragEvent = STNPOINT;

if (!dragEvent)
{
gdk_window_get_pointer(gdkWindow, &x, &y, &state);
return TRUE;   
}

switch(screen)
{
case STNANALYSIS:
switch(dragEvent)
{
case STNTRCTFORMDISP:
case STNPOINT:
trcH = (height-(ANATOPM+ANABOTM))/trcCnt;
trcNum = (event->y-ANATOPM)/trcH;
if (trcNum >= trcCnt)
{
dragEvent=STNNOEVENT;
break;
}
break;
}
break;
}

switch(dragEvent)
{
#if 0
case STNEVTARRIVALS: 
evtPhDisp(widget, event->x, event->y);
break;
#endif

case STNTRCTFORMDISP: 
if (stnStates.analysis.tForm.common)
break;
analysesDisp(widget, trcNum);
break;

case STNPOINT:
{
PQLDATA *pqlData;
pqlData = g_slist_nth_data(stnStates.analysis.traces, trcNum);
if (pqlData)
{
pointID *pointPtr;
int  pointNum= stnStates.analysis.plot.extra.user.point[POINT2].xVal ? POINT2 : POINT1;
pointPtr = &stnStates.analysis.plot.extra.user.point[pointNum];
ANAPoint(pqlData, (int) event->x, pointPtr);
dispPointVals(&stnStates.analysis.plot.extra.user.point[POINT1], pointNum);
}
}
break;

case STNNOEVENT:
if (AnalysesString)
gtk_widget_hide(AnalysesString);
break;
}

gdk_window_get_pointer(gdkWindow, &x, &y, &state);
return TRUE;
}

void printSTNLabel(GdkPixmap *pmap, traceInfo *trc, int rec, int scr, 
gboolean selected, gboolean overlay, ANAPOPUP *specPsd)
{
gchar *label;
int  fontGC=0, trcH, trcCnt;
int  labelW, vertPos=0;
static int labelH;
int  width, height;

gdk_drawable_get_size(pmap, &width, &height);
switch(scr)
{
case STNANALYSIS:
trcCnt = g_slist_length(stnStates.analysis.traces);
trcH = (height - (ANATOPM + ANABOTM))/trcCnt;
vertPos = ANATOPM + rec*trcH + trcH/2;
fontGC = STNFLBLA;
break;
case STNSPECTRA:
trcCnt = g_slist_length(specPsd->data);
trcH = (height - (SPECTOPM + ANABOTM))/trcCnt;
vertPos = SPECTOPM + rec*trcH + trcH/2;
fontGC = STNFLBLS;
break;
case STNPSD:
trcCnt = g_slist_length(stnStates.analysis.traces);
trcH = (height - (ANATOPM + ANABOTM))/trcCnt;
vertPos = ANATOPM + rec*trcH + trcH/2;
fontGC = STNFLBLP;
break;
}

switch (overlay)
{
case TRUE:
fontGC = rec % 30;
fontGC += NUMFONTGCS;
break;
}

label = g_strdup_printf("%s.%s", trc->data.head.network, trc->data.head.station);
labelW = fontSIZEP(label, STNLBLF, WIDTH);
if (!labelH)
labelH = fontSIZEP(label, STNLBLF, HEIGHT);
switch(selected)
{
case FALSE:
fontDrawP(pmap, label, STNLFTM/2 - labelW/2, vertPos - labelH, STNLBLF, fontGC);
break;
case TRUE:
fontDrawPRev(pmap, label, STNLFTM/2 - labelW/2, vertPos - labelH, STNLBLF, STNFLBLA, fontGC);
break;
}
g_free(label);

switch(strlen(trc->data.head.location))
{
case 0:
label = g_strdup_printf("--.%s", trc->data.head.channel);
break;
default:
label = g_strdup_printf("%s.%s", trc->data.head.location, trc->data.head.channel);
break;
}
labelW = fontSIZEP(label, STNLBLF, WIDTH);
switch(selected)
{
case FALSE:
fontDrawP(pmap, label, STNLFTM/2 - labelW/2, vertPos + 1, STNLBLF, fontGC);
break;
case TRUE:
fontDrawPRev(pmap, label, STNLFTM/2 - labelW/2, vertPos + 1, STNLBLF, STNFLBLA, fontGC);
break;
}
g_free(label);
gtk_widget_queue_draw_area(STNDA[STNANALYSIS], 0, vertPos-labelH-1, STNLFTM, labelH*2); 
}

void makeSTNSelectionList(int scr)
{
GSList *trcIter, **traces=NULL, **selections=NULL;
gboolean selected;
PQLDATA  *pqlData;
ANADATA  *anaData;
CHNDATA  *chnData;

switch(scr)
{
case STNMAIN:
traces = &stnStates.main.chnList;
selections = &stnStates.main.selections;
break;
case STNANALYSIS:
traces = &stnStates.analysis.traces;
selections = &stnStates.analysis.selections;
break;
}

g_slist_free(*selections);
*selections = NULL;
for(trcIter = *traces, selected = FALSE;
trcIter;
trcIter = g_slist_next(trcIter), selected = FALSE) 
{
switch(scr)
{
case STNMAIN:
chnData = trcIter->data;
if (chnData &&
chnData->display.selected)
selected = TRUE;
break;
case STNANALYSIS:
pqlData = trcIter->data;
anaData = pqlData->anaData;
if (anaData &&
anaData->display.selected)
selected = TRUE;
break;
}
if (selected)
{
*selections = g_slist_append(*selections, trcIter->data);
}
}
}

gboolean selectSTNT(GtkButton *button, gpointer c)
{
GSList *trcIter, **traces, **selections, *list=NULL;
gboolean selected=FALSE;
GdkPixmap *pmap=NULL;
PQLDATA  *pqlData;
int   trcNum, command = GPOINTER_TO_INT(c);

switch(stnStates.screen)
{
case STNANALYSIS:
traces = &stnStates.analysis.traces;
selections = &stnStates.analysis.selections;
list = (command == REMOVE) ? *selections : *traces;
pmap = stnStates.pmap.analysis[PMAPVISIBLE];
break;
}

switch(command)
{
case SELECT:
selected = TRUE;
break;
case DESELECT:
selected = FALSE;
break;
}
for(trcIter = list, trcNum = 0;
trcIter;
trcIter = g_slist_next(trcIter), trcNum++) 
{
pqlData = trcIter->data;
switch(stnStates.screen)
{
case STNANALYSIS:
switch(command)
{
case REMOVE:
freeMyData(TRCINFO, pqlData->trc);
freeMyData(ANAINFO, pqlData->anaData);
free(pqlData->anaData);
break;
case SELECT:
case DESELECT:
((ANADATA *) (pqlData->anaData))->display.selected = selected;
break;
}
break;
}
switch (command)
{
case REMOVE:
*traces = g_slist_remove(*traces, pqlData);
break;
default:
{ 
GtkWidget *da = STNDA[stnStates.screen];
int  trcH, trcCnt;
printSTNLabel(pmap, pqlData->trc, trcNum, stnStates.screen, selected, 
stnSettings.display.overlay[stnStates.screen], NULL);
trcCnt = g_slist_length(stnStates.analysis.traces);
trcH = (da->allocation.height - (ANATOPM + ANABOTM))/trcCnt;
gtk_widget_queue_draw_area(da, 0, ANATOPM+trcH*trcNum, STNLFTM+10, trcH);
}
break;
}
}
makeSTNSelectionList(stnStates.screen);

if (command == REMOVE)
{
stnStates.pmap.dirty[STNANALYSIS][0] = TRUE;
makeSTNDrawing(STNANALYSIS);
}
return TRUE;
}

void ANAreMag(GtkWidget *widget, gpointer a)
{
int  leftFact, rightFact, action = GPOINTER_TO_INT(a);
gboolean replot=FALSE;
graph *localGR = &stnStates.analysis.plot.gr[LOCALGR];
graph *mainGR = &stnStates.analysis.plot.gr[MAINGR];

if (!stnStates.analysis.plot.limits.selection.pix.left &&
widget)
{ 
gdk_beep();
return;
}

if (widget)   
replot = TRUE; 

gdk_window_set_cursor(topLevel, hourCursor);
while (g_main_context_iteration(NULL, FALSE));

switch(action)
{
case SNAP:
{
double userL = PIX2USERX(localGR, stnStates.analysis.plot.limits.selection.pix.left);
switch(stnSettings.analysis.anchor)
{
case LEFT:
leftFact = 0;
rightFact = 2;
break;

case RIGHT:
leftFact = 2;
rightFact = 0;
break;

case MIDDLE:
case MOUSE:
leftFact = 1;
rightFact = 1;
break;
}
switch(stnSettings.analysis.anchor)
{
case LEFT:
case RIGHT:
stnStates.analysis.plot.limits.selection.user.left += 
userL - leftFact * (localGR->user_xdif/2);
if (stnStates.analysis.plot.limits.selection.user.left < mainGR->user_xmin)
{  
stnStates.analysis.plot.limits.selection.user.left = 0;
stnStates.analysis.plot.limits.selection.user.right = (2.0/leftFact) * userL;
}
else
{  
stnStates.analysis.plot.limits.selection.user.right = 
stnStates.analysis.plot.limits.selection.user.left + 
rightFact * (localGR->user_xdif/2);
}
if (stnStates.analysis.plot.limits.selection.user.right > mainGR->user_xmax)
{ 
stnStates.analysis.plot.limits.selection.user.right = mainGR->user_xmax;
}
break;
case MIDDLE:
case MOUSE:
stnStates.analysis.plot.limits.selection.user.left +=
userL - localGR->user_xdif/2;
if (stnStates.analysis.plot.limits.selection.user.left < mainGR->user_xmin)
{  
stnStates.analysis.plot.limits.selection.user.left = 0;
stnStates.analysis.plot.limits.selection.user.right = userL*2;
}
else
{  
stnStates.analysis.plot.limits.selection.user.right =
stnStates.analysis.plot.limits.selection.user.left + localGR->user_xdif;
}
if (stnStates.analysis.plot.limits.selection.user.right > mainGR->user_xmax)
{ 
stnStates.analysis.plot.limits.selection.user.right = mainGR->user_xmax;
stnStates.analysis.plot.limits.selection.user.left = 
mainGR->user_xmax - (mainGR->user_xmax - userL)*2;
}
break;
}
}
break;

case MAG: 
{
double userL = PIX2USERX(localGR, stnStates.analysis.plot.limits.selection.pix.left);
double userR = PIX2USERX(localGR, stnStates.analysis.plot.limits.selection.pix.right);
stnStates.analysis.plot.limits.selection.user.left += userL;
stnStates.analysis.plot.limits.selection.user.right = 
stnStates.analysis.plot.limits.selection.user.left + (userR - userL);
}
break;

case KEY: 
break;
}
stnStates.analysis.plot.limits.selection.pix.left = 0;  

if (replot)
{
stnStates.pmap.dirty[STNANALYSIS][0] = TRUE;
makeSTNDrawing(STNANALYSIS);
}
}

static int getMinDisp(GSList *traces)
{
int  min;
GSList *anaIter;
traceInfo *trcData;
gboolean displayed=FALSE;

for(anaIter = traces, min=100;
anaIter;
anaIter = g_slist_next(anaIter))
{
trcData = ((PQLDATA *) (anaIter->data))->trc;
if (trcData->magnify.begin == -1)
continue;  

displayed = TRUE;
if (trcData->magnify.num_points_shown < min)
min = trcData->magnify.num_points_shown;
}
if (displayed)
return min;
else
return 100;
}

static gboolean checkScrollBounds(double diffL, double diffR, double length, graph *gr)
{
gboolean ret=FALSE;
if (stnStates.analysis.plot.limits.selection.user.left + diffL < gr->user_xmin)
{
gdk_beep();
stnStates.analysis.plot.limits.selection.user.left = gr->user_xmin;
stnStates.analysis.plot.limits.selection.user.right =
stnStates.analysis.plot.limits.selection.user.left + length;
ret=TRUE;
}
if (stnStates.analysis.plot.limits.selection.user.right + diffR > gr->user_xmax)
{
gdk_beep();
stnStates.analysis.plot.limits.selection.user.right = gr->user_xmax;
stnStates.analysis.plot.limits.selection.user.left =
stnStates.analysis.plot.limits.selection.user.right - length;
ret=TRUE;
}

return ret;
}

gboolean anaScrollEvent(GtkWidget *widget, GdkEventScroll *event, gpointer nil)
{ 
int   zoomDir, scrollDir;
int   evt=NONE, min;
float  jumpU;
double  diff, diffL=0, diffR=0;
double  zAnchor;
static gboolean blocked=FALSE;
gboolean replot=FALSE;
graph  *localGR = &stnStates.analysis.plot.gr[LOCALGR];
graph  *mainGR = &stnStates.analysis.plot.gr[MAINGR];

if (blocked)
return FALSE;
blocked = TRUE;

switch(event->direction)
{
case GDK_SCROLL_UP:
min = getMinDisp(stnStates.analysis.traces);
if (min < 3)
{
gdk_beep();
blocked = FALSE;
zAnchor = 0;
return TRUE;
}
zoomDir = 1;
scrollDir = -1;
break;
case GDK_SCROLL_DOWN:
zoomDir = -1;
scrollDir = 1;
break;
default:
blocked = FALSE;
zAnchor = 0;
return TRUE;
break;
}

diff = (stnStates.analysis.plot.limits.selection.user.right - 
stnStates.analysis.plot.limits.selection.user.left);
if (event->y < ANATOPM &&
event->y > ANATOPMSG)
{ 
jumpU = (float) (diff * settings.magnify.scrollFactor/100.);
diffL = diffR = jumpU * scrollDir;
evt = SCROLL;
}
else
{ 
evt = ZOOM;
switch(stnSettings.analysis.anchor)
{
case LEFT:
case RIGHT:
case MIDDLE:
diffL = (diff / (100/settings.magnify.zoomFactor)) * 
factors[stnSettings.analysis.anchor*2] * zoomDir;
diffR = (diff / (100/settings.magnify.zoomFactor)) * 
factors[stnSettings.analysis.anchor*2+1] * zoomDir * -1;
break;
case MOUSE:
zAnchor = stnStates.analysis.plot.limits.selection.user.left +
PIX2USERX(localGR, (int) event->x);
diffL = (diff / (100/settings.magnify.zoomFactor)) *
((zAnchor-stnStates.analysis.plot.limits.selection.user.left)/(diff/2.)) * 
zoomDir;
diffR = (diff / (100/settings.magnify.zoomFactor)) *
((stnStates.analysis.plot.limits.selection.user.right-zAnchor)/(diff/2.)) * 
zoomDir * -1;
break;
}
}

if (event->x < ANALFTM)
{ 
if (event->y < widget->allocation.height - ANABOTM && 
event->y > ANATOPM &&
stnSettings.display.overlay[stnStates.screen])
{
evt = REPLOT;  
}
}

switch(evt)
{
#if 0
case REPLOT:      
topTrace += scrollDir;    
topTrace = (topTrace >= states.trace.numSelected) 
? 0 
: (topTrace < 0)
? states.trace.numSelected - 1
: topTrace;
states.user.topRec.magnify = topTrace;
mag_plot_files(widget, DAPixmap[*scr], widget->allocation.width, 
widget->allocation.height, states.user.topRec.magnify);

gtk_widget_queue_draw_area(widget, 
states.magnify.limits.margin[states.currentScreen].left,
MAG_T_MARGIN, 
states.magnify.limits.margin[states.currentScreen].length,
widget->allocation.height - (MAG_T_MARGIN+MAG_B_MARGIN));
break;
#endif
case SCROLL:
if (checkScrollBounds(diffL, diffR, localGR->user_xdif, mainGR)==TRUE)
{ 
replot = TRUE;
break;
}
case ZOOM:
stnStates.analysis.plot.limits.selection.user.left += diffL;
stnStates.analysis.plot.limits.selection.user.right += diffR;
replot = TRUE;
if (stnStates.analysis.plot.limits.selection.user.left < mainGR->user_xmin)
{  
stnStates.analysis.plot.limits.selection.user.left = 0;
}
if (stnStates.analysis.plot.limits.selection.user.right > mainGR->user_xmax)
{  
stnStates.analysis.plot.limits.selection.user.right = mainGR->user_xmax;
}
break;
}

if (replot)
{
stnStates.analysis.plot.limits.selection.pix.left = 0;  
stnStates.pmap.dirty[STNANALYSIS][0] = TRUE;
makeSTNDrawing(STNANALYSIS);
}

blocked = FALSE;
return TRUE;
}

static void removeUPicks(int pickNum)
{
USERPICK *userP1;
PQLDATA *pqlData;
ANADATA *anaData;
traceInfo *trc=NULL;
GSList *selIter, *trcIter, *pickIter;
char *stnNm;

for(selIter = stnStates.analysis.selections;
selIter;
selIter = g_slist_next(selIter))
{ 
pqlData = selIter->data;
anaData = pqlData->anaData;
anaData->display.selected = FALSE; 
trc = anaData->trc;
stnNm = trc->data.head.station;

for(trcIter = stnStates.analysis.traces;
trcIter;
trcIter = g_slist_next(trcIter))
{ 
pqlData = trcIter->data;
anaData = pqlData->anaData;
trc = anaData->trc;
if (strcmp(trc->data.head.station, stnNm))
continue;

for (pickIter = anaData->userPicks;
pickIter;
pickIter = g_slist_next(pickIter))
{ 
userP1 = pickIter->data;
if (userP1->pickNum == pickNum)
{
free(userP1);
anaData->userPicks = g_slist_remove(anaData->userPicks, userP1);
break;
}
}
}
}
makeSTNSelectionList(STNANALYSIS);  
}

gboolean keyEventANA(GtkWidget *widget, GdkEventKey *event, gpointer nil)
{ 
int   x, y, min;
int   zoomDir, scrollDir;
float  jumpU;
double  diff, diffL=0, diffR=0;
double  zAnchor; 
static GdkWindow *gdkWindow;
GdkModifierType state;
static gboolean blocked=FALSE;
gboolean replot = FALSE;
graph  *localGR = &stnStates.analysis.plot.gr[LOCALGR];
graph  *mainGR = &stnStates.analysis.plot.gr[MAINGR];

if (blocked)
return TRUE;
blocked = TRUE;

if (!gdkWindow)
gdkWindow = gdk_get_default_root_window();

switch(event->keyval)
{
case GDK_Up:
case GDK_Down:
case GDK_Right:
case GDK_Left:
if (event->type == GDK_KEY_RELEASE)
{ 
blocked = FALSE;
return TRUE;
}
break;
case GDK_1:
case GDK_2:
case GDK_3:
case GDK_4:
case GDK_5:
case GDK_6:
case GDK_7:
case GDK_8:
case GDK_9:
case GDK_0:
{
char *msg=NULL;
int newPick = (event->keyval == GDK_0) ? 9 : (int) (event->keyval - GDK_0) - 1;
if (event->type == GDK_KEY_RELEASE)
{ 
blocked = FALSE;
return TRUE;
}
stnStates.display.user.keyActive = NONE;
if (stnStates.analysis.selections)
{ 
removeUPicks(newPick);
stnStates.event.curPick = NONE;
stnStates.pmap.dirty[stnStates.screen][0] = TRUE;
makeSTNDrawing(stnStates.screen);
}
else
{ 
if (stnStates.event.curPick == NONE ||
(stnStates.event.curPick != NONE &&
stnStates.event.curPick != newPick))
{ 
stnStates.event.curPick = newPick;
msg = g_strdup_printf("Selecting Pick %s", stnSettings.event.userP[stnStates.event.curPick]);
}
else
{ 
stnStates.event.curPick = NONE;
dispMsgSTN(UNDISPLAY, NULL);
}
}
if (msg)
dispMsgSTN(DISPLAY, msg);
free(msg);
blocked = FALSE;
return TRUE;
}
break;
}
if (event->type == GDK_KEY_RELEASE)
{ 
switch(stnStates.display.user.keyActive)
{
case ZKEY:
if (stnStates.analysis.plot.limits.selection.pix.right)
{
ANAreMag((GtkWidget *)TRUE, GINT_TO_POINTER(MAG));
}
break;
case XKEY:
if (stnStates.analysis.plot.limits.selection.pix.right)
{
setANABounds(ZOOMX);
ANAreMag((GtkWidget *)TRUE, GINT_TO_POINTER(KEY));
}
break;
}
stnStates.display.user.keyActive = NONE;
blocked = FALSE;
return TRUE;
}

switch(event->keyval)
{
case GDK_Up:
min = getMinDisp(stnStates.analysis.traces);
if (min < 3)
{ 
gdk_beep();
blocked = FALSE;
return TRUE;
}
case GDK_Right:
zoomDir = 1;
scrollDir = 1;
break;
case GDK_Down:
case GDK_Left:
zoomDir = -1;
scrollDir = -1;
break;

case GDK_z:
stnStates.display.user.keyActive = ZKEY;
blocked = FALSE;
return TRUE;
break; 
case GDK_x:
stnStates.display.user.keyActive = XKEY;
blocked = FALSE;
return TRUE;
break;

default:
stnStates.display.user.keyActive = NONE;
blocked = FALSE;
return TRUE;
break;
}

switch(event->keyval)
{
case GDK_Up:
case GDK_Down:
diff = (stnStates.analysis.plot.limits.selection.user.right - 
stnStates.analysis.plot.limits.selection.user.left);
switch(stnSettings.analysis.anchor)
{
case LEFT:
case RIGHT:
case MIDDLE:
diffL = (diff / (100/settings.magnify.zoomFactor)) * 
factors[stnSettings.analysis.anchor*2] * zoomDir;
diffR = (diff / (100/settings.magnify.zoomFactor)) * 
factors[stnSettings.analysis.anchor*2+1] * zoomDir * -1;
break;
case MOUSE:
gdk_window_get_pointer(gdkWindow, &x, &y, &state);
zAnchor = stnStates.analysis.plot.limits.selection.user.left +
PIX2USERX(localGR, (int) x);
diffL = (diff / (100/settings.magnify.zoomFactor)) *
((zAnchor-stnStates.analysis.plot.limits.selection.user.left)/
(diff/2.)) * zoomDir;
diffR = (diff / (100/settings.magnify.zoomFactor)) *
((stnStates.analysis.plot.limits.selection.user.right-zAnchor)/
(diff/2.)) * zoomDir * -1;
break;
}
break;
case GDK_Right:
case GDK_Left:
jumpU = (float) (localGR->user_xdif * settings.magnify.scrollFactor/100.);
diffL = diffR = jumpU * scrollDir;
break;
}

switch(event->keyval)
{
case GDK_Right:
case GDK_Left:
if (checkScrollBounds(diffL, diffR, localGR->user_xdif, mainGR))
{
replot=TRUE;
break;
}
case GDK_Up:
case GDK_Down:
stnStates.analysis.plot.limits.selection.user.left += diffL;
stnStates.analysis.plot.limits.selection.user.right += diffR;
replot = TRUE;
if (stnStates.analysis.plot.limits.selection.user.left < mainGR->user_xmin)
{  
stnStates.analysis.plot.limits.selection.user.left = 0;
}
if (stnStates.analysis.plot.limits.selection.user.right > mainGR->user_xmax)
{  
stnStates.analysis.plot.limits.selection.user.right = mainGR->user_xmax;
}
break;
}

if (replot)
{
stnStates.analysis.plot.limits.selection.pix.left = 0;  
stnStates.pmap.dirty[STNANALYSIS][0] = TRUE;
makeSTNDrawing(STNANALYSIS);
}

blocked = FALSE;
return TRUE;
}
