#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pqlx_defs.h"
#include "pql_defines.h"
#include "pqlx_externs.h"
#include "pqlx_stn.h"
#include "gtk_stn_externs.h"
#include "util_externs.h"
double GDate2Epoch(double gDate)
{
int   jday;
double  epoch, secs;
struct ptime date;
GDate  *day;
memset(&date, 0, sizeof(struct ptime));
jday = (int) gDate;
secs = (gDate - jday)*ONEDAYOFSECS;
day = g_date_new_julian(jday);
date.yr = g_date_get_year(day);
date.day = g_date_get_day_of_year(day);
epoch = passcal_htoe(&date, FALSE) + secs;
g_date_free(day);
return (epoch);
}
static void setNewHdr(traceInfo *trc)
{
struct ptime date;
trc->data.fileName = g_strdup(STNCOMPFN);
trc->data.head.longFN = trc->data.fileName;
trc->data.head.shortFN = g_strdup(trc->data.fileName);
trc->data.dataInfo.data = NULL;
trc->trace.numGaps = trc->trace.numOlaps = 0;
trc->trace.gaps = trc->trace.overlaps = NULL;
passcal_etoh(&date, trc->data.head.epochs);
trc->data.head.year = date.yr;
trc->data.head.day = date.jday;
trc->data.head.hour = date.hr;
trc->data.head.minute = date.mn;
trc->data.head.second = date.sec;
trc->data.head.m_secs = (trc->data.head.epochs - (int) trc->data.head.epochs)*1000.;
trc->data.head.descr1 = g_strdup_printf("%i %03i %02i:%02i" ,
trc->data.head.year, trc->data.head.day, 
trc->data.head.hour, trc->data.head.minute);
trc->data.head.descr2 = g_strdup_printf("%s.%s.%s.%s",
trc->data.head.network, trc->data.head.station,
trc->data.head.location, trc->data.head.channel);
trc->data.dataFilt = NULL;
trc->trace.olapData = NULL;
}
enum {
GAPIDENTIFY,
GAPADD,
GAPFINISH
};
static void manageGLaps(int action, traceInfo *trcIn, traceInfo *trcOut, int startSamp, int endSamp, int totSamp, GSList  **gapIter, GSList **olapIter)
{
int  i;
glaps *glapPtr;
switch(action)
{
case GAPIDENTIFY:
{
int realStart;
switch(startSamp)
{
case -1:
realStart = 0;
break;
default:
realStart = startSamp;
break;
}
for(i=0;i<trcIn->trace.numGaps*2;i+=2)
{
if (startSamp != -1 &&
trcIn->trace.gaps[i+1] < startSamp)
continue;        
if (endSamp != -1 && 
trcIn->trace.gaps[i] > endSamp)
break;         
glapPtr = calloc(1, sizeof(glaps));
glapPtr->bounds[GLAPBEG] = trcIn->trace.gaps[i] - realStart + totSamp;
if (endSamp != -1 && trcIn->trace.gaps[i+1] > endSamp)
{
glapPtr->bounds[GLAPEND] = endSamp - realStart + totSamp;
}
else
{
glapPtr->bounds[GLAPEND] = trcIn->trace.gaps[i+1] - realStart + totSamp;
}
*gapIter = g_slist_append(*gapIter, glapPtr);
}  
for(i=0;i<trcIn->trace.numOlaps*2;i+=2)
{
if (startSamp != -1 && 
trcIn->trace.overlaps[i] < startSamp)
continue;
if (endSamp != -1 && 
trcIn->trace.overlaps[i] > endSamp)
break;
glapPtr = calloc(1, sizeof(glaps));
glapPtr->bounds[GLAPBEG] = trcIn->trace.overlaps[i] - realStart + totSamp;
if (endSamp != -1 && trcIn->trace.overlaps[i+1] > endSamp)
{
glapPtr->bounds[GLAPEND] = endSamp - realStart + totSamp;
}
else
{
glapPtr->bounds[GLAPEND] = trcIn->trace.overlaps[i+1] - realStart + totSamp;
}
*olapIter = g_slist_append(*olapIter, glapPtr);
}
}
break;
case GAPADD:
{
glapPtr = calloc(1, sizeof(glaps));
glapPtr->bounds[GLAPBEG] = startSamp;
glapPtr->bounds[GLAPEND] = endSamp;
*gapIter = g_slist_append(*gapIter, glapPtr);
}
break;
case GAPFINISH:
{
trcOut->trace.numGaps = g_slist_length(*gapIter);
if (trcOut->trace.numGaps)
{
trcOut->trace.gaps = calloc(trcOut->trace.numGaps, sizeof(int)*2);
for(i=0;*gapIter;*gapIter=g_slist_next(*gapIter), i+=2)
{
glapPtr = (*gapIter)->data;
trcOut->trace.gaps[i] = glapPtr->bounds[GLAPBEG];
trcOut->trace.gaps[i+1] = glapPtr->bounds[GLAPEND];
free(glapPtr);
}
g_slist_free(*gapIter);
*gapIter = NULL;
}
trcOut->trace.numOlaps = g_slist_length(*olapIter);
if (trcOut->trace.numOlaps)
{
trcOut->trace.overlaps = calloc(trcOut->trace.numOlaps, sizeof(int)*2);
for(i=0;*olapIter;*olapIter=g_slist_next(*olapIter), i+=2)
{
glapPtr = (*olapIter)->data;
trcOut->trace.overlaps[i] = glapPtr->bounds[GLAPBEG];
trcOut->trace.overlaps[i+1] = glapPtr->bounds[GLAPEND];
free(glapPtr);
}
g_slist_free(*olapIter);
}
}
break;
}
}
static int checkIfGap(traceInfo *trc1, traceInfo *trc2)
{
double sampInterval;
unsigned int sampsPsec;
int  diffSamples=0;
double endSecs, begSecs, ONESAMPLE;
sampInterval = trc1->data.head.sampInt/1000000.;
sampsPsec=(int) (1./sampInterval + .5);
if (sampsPsec)
{
ONESAMPLE = (1000000./sampsPsec); 
}
else
{
ONESAMPLE = trc1->data.head.sampInt;
}  
endSecs = trc1->data.head.epochs + (trc1->data.dataInfo.head.numSamples * sampInterval);
begSecs = trc2->data.head.epochs;
if (endSecs <= begSecs)
{
if ((double) (((begSecs-endSecs)*1000000)-ONESAMPLE) > (double) (ONESAMPLE*2))
{ 
if (sampsPsec)
diffSamples = (int)((begSecs-endSecs)*sampsPsec);
else
diffSamples = (int)((begSecs-endSecs)/sampInterval);
trc1->data.dataInfo.data = realloc(trc1->data.dataInfo.data, 
(trc1->data.dataInfo.head.numSamples+diffSamples)*trc1->data.head.dataSize);
memset(&trc1->data.dataInfo.data[trc1->data.dataInfo.head.numSamples*trc1->data.head.dataSize], 0, 
sizeof(char)*diffSamples*trc1->data.head.dataSize);
trc1->data.dataInfo.head.numSamples += diffSamples;
}
}
else
{ 
}
return(diffSamples);
}
PQLDATA *cutTrc(PQLDATA *pqlIn, double startE, double endE)
{
traceInfo *trcOut, *trcIn;
PQLDATA  *pqlOut;
int   startSamp, endSamp, dataStart;
double  sps;
GSList  *gapIter=NULL, *olapIter=NULL;
trcOut = calloc(1, sizeof(traceInfo));
pqlOut = calloc(1, sizeof(PQLDATA));
if (!pqlIn->trc)
{
traceInfo *tmp;
pqlIn->trc = tmp = calloc(1, sizeof(traceInfo));
tmp->data.fileName = g_strdup(pqlIn->fileName);
readTrace(tmp);
}
trcIn = pqlIn->trc;
if(!trcIn->data.dataInfo.data)
return NULL;
sps = 1000000./trcIn->data.head.sampInt;
if (startE <= trcIn->data.head.epochs)
startSamp = 0; 
else
startSamp = (startE - trcIn->data.head.epochs) * sps;
if (endE >= (trcIn->data.head.epochs + trcIn->data.head.length))
endSamp = trcIn->data.dataInfo.head.numSamples;
else
endSamp = (endE - trcIn->data.head.epochs) * sps;
memcpy(trcOut, trcIn, sizeof(traceInfo));
trcOut->data.dataInfo.head.numSamples = endSamp - startSamp - 1;
trcOut->data.head.epochs = trcIn->data.head.epochs + (startSamp * 
(double) (trcIn->data.head.sampInt/1000000.));
setNewHdr(trcOut);
trcOut->data.dataInfo.data = malloc(trcOut->data.dataInfo.head.numSamples * 
trcOut->data.head.dataSize);
dataStart = (startSamp)*trcIn->data.head.dataSize;
memcpy(&trcOut->data.dataInfo.data[0], &trcIn->data.dataInfo.data[dataStart],
(trcOut->data.dataInfo.head.numSamples)*trcIn->data.head.dataSize);
trcOut->data.head.length = (trcOut->data.dataInfo.head.numSamples-1) * 
(trcOut->data.head.sampInt/1000000.);
trcOut->data.head.epochStop = trcOut->data.head.epochs + trcOut->data.head.length;
manageGLaps(GAPIDENTIFY, trcIn, trcOut, startSamp, endSamp, 0, &gapIter, &olapIter);
manageGLaps(GAPFINISH, trcIn, trcOut, startSamp, endSamp, 0, &gapIter, &olapIter);
get_maxmin_amp(trcOut);
pqlOut->trc = trcOut;
IFFREE(pqlIn->fileName);
free(pqlIn);
return pqlOut; 
}
PQLDATA *marryTrcs(GSList *trcsToMarry, double startE, double endE)
{
traceInfo *trcOut, *trcIn;
PQLDATA  *pqlOut, *pqlIn;
GSList  *trcsIter=trcsToMarry;
int   totSamp, startSamp, endSamp, fullTrcsCnt, trcCntr, dataStart, gapSamps;
double  sps;
gboolean readData;
GSList  *gapIter=NULL, *olapIter=NULL;
trcOut = calloc(1, sizeof(traceInfo));
pqlOut = calloc(1, sizeof(PQLDATA));
pqlIn = trcsIter->data;
if (pqlIn->trc)
{
readData = FALSE;
}
else
{
traceInfo *tmp;
pqlIn->trc = tmp = calloc(1, sizeof(traceInfo));
tmp->data.fileName = g_strdup(pqlIn->fileName);
readTrace(tmp);
readData = TRUE;
}
trcIn = pqlIn->trc;
if(!trcIn->data.dataInfo.data)
return NULL;
trcIn = pqlIn->trc;
memcpy(trcOut, trcIn, sizeof(traceInfo));
sps = 1000000./trcIn->data.head.sampInt;
if (startE <= trcIn->data.head.epochs)
startSamp = 0; 
else
startSamp = (startE - trcIn->data.head.epochs) * sps;
trcOut->data.dataInfo.head.numSamples = trcIn->data.dataInfo.head.numSamples - startSamp;
trcOut->data.head.epochs = trcIn->data.head.epochs + 
(startSamp * (double) (trcIn->data.head.sampInt/1000000.));
setNewHdr(trcOut);
manageGLaps(GAPIDENTIFY, trcIn, trcOut, startSamp, -1, 0, &gapIter, &olapIter);
fullTrcsCnt = g_slist_length(trcsToMarry) - 2;
for(trcCntr=0;trcCntr<fullTrcsCnt;trcCntr++)
{
trcsIter = g_slist_next(trcsIter);
pqlIn = trcsIter->data;
if (readData)
{
traceInfo *tmp;
pqlIn->trc = tmp = calloc(1, sizeof(traceInfo));
tmp->data.fileName = g_strdup(pqlIn->fileName);
readTrace(tmp);
}
if((gapSamps = checkIfGap(trcIn, pqlIn->trc)))
{
manageGLaps(GAPADD, NULL, NULL, trcOut->data.dataInfo.head.numSamples, 
trcOut->data.dataInfo.head.numSamples+gapSamps, 0, &gapIter, &olapIter);
trcOut->data.dataInfo.head.numSamples += gapSamps;
}
trcIn = pqlIn->trc;
manageGLaps(GAPIDENTIFY, trcIn, trcOut, -1, -1, trcOut->data.dataInfo.head.numSamples, &gapIter, &olapIter);
trcOut->data.dataInfo.head.numSamples += trcIn->data.dataInfo.head.numSamples;
}
trcsIter = g_slist_next(trcsIter);
pqlIn = trcsIter->data;
if (readData)
{
traceInfo *tmp;
pqlIn->trc = tmp = calloc(1, sizeof(traceInfo));
tmp->data.fileName = g_strdup(pqlIn->fileName);
readTrace(tmp);
}
if((gapSamps = checkIfGap(trcIn, pqlIn->trc)))
{
manageGLaps(GAPADD, NULL, NULL, trcOut->data.dataInfo.head.numSamples, 
trcOut->data.dataInfo.head.numSamples+gapSamps, 0, 
&gapIter, &olapIter);
trcOut->data.dataInfo.head.numSamples += gapSamps;
}
trcIn = pqlIn->trc;
if (endE >= (trcIn->data.head.epochs + trcIn->data.head.length))
endSamp = trcIn->data.dataInfo.head.numSamples;
else
endSamp = (endE - trcIn->data.head.epochs) * sps;
manageGLaps(GAPIDENTIFY, trcIn, trcOut, -1, endSamp, trcOut->data.dataInfo.head.numSamples, &gapIter, &olapIter);
manageGLaps(GAPFINISH, trcIn, trcOut, -1, -1, trcOut->data.dataInfo.head.numSamples, &gapIter, &olapIter);
trcOut->data.dataInfo.head.numSamples += endSamp;
trcOut->data.dataInfo.data = malloc(trcOut->data.dataInfo.head.numSamples * trcOut->data.head.dataSize);
trcsIter = trcsToMarry;
pqlIn = trcsIter->data;
trcIn = pqlIn->trc;
dataStart = (startSamp) * trcIn->data.head.dataSize;
totSamp = trcIn->data.dataInfo.head.numSamples - startSamp;
memcpy(&trcOut->data.dataInfo.data[0], &trcIn->data.dataInfo.data[dataStart],
(totSamp)*trcIn->data.head.dataSize);
if (pqlIn->fileName)
free(pqlIn->fileName);
free(pqlIn);
for(trcCntr=0;trcCntr<fullTrcsCnt;trcCntr++)
{ 
trcsIter = g_slist_next(trcsIter);
pqlIn = trcsIter->data;
trcIn = pqlIn->trc;
memcpy(&trcOut->data.dataInfo.data[totSamp*trcIn->data.head.dataSize], 
trcIn->data.dataInfo.data, 
trcIn->data.dataInfo.head.numSamples*trcIn->data.head.dataSize);
totSamp += trcIn->data.dataInfo.head.numSamples;
if (pqlIn->fileName)
free(pqlIn->fileName);
free(pqlIn);
}
trcsIter = g_slist_next(trcsIter);
pqlIn = trcsIter->data;
trcIn = pqlIn->trc;
memcpy(&trcOut->data.dataInfo.data[totSamp*trcIn->data.head.dataSize], 
trcIn->data.dataInfo.data, endSamp*trcIn->data.head.dataSize);
trcOut->data.head.length = (trcOut->data.dataInfo.head.numSamples-1) * 
(trcOut->data.head.sampInt/1000000.);
trcOut->data.head.epochStop = trcOut->data.head.epochs + trcOut->data.head.length;
if (pqlIn->fileName)
free(pqlIn->fileName);
free(pqlIn);
pqlOut->trc = trcOut;
return pqlOut; 
}
