#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include "pql_defines.h"
#include "pqlx_externs.h"
#include "pqlx_stn.h"
#include "gtk_stn_externs.h"
static GtkTooltips *tips;
static char  toolTip[1000];
void make_stnEventScreens();
enum {
EVTTIME,
EVTMAG,
EVTTYPE,
EVTLAT,
EVTLON,
EVTDEP,
EVTCAT,
EVTDESC,
TTLEVTCOLS,
EVTID=TTLEVTCOLS
};
static char *eventCols[TTLEVTCOLS] = { "Event Time","Mag","Type","Latitude",
"Longitude","Depth","Catalog","Description"};
static GtkListStore *eventStore;
static gint sortCol(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer col)
{
char *str1, *str2;
float val1, val2;
gint ret;
gint sortcol = GPOINTER_TO_INT(col);
switch(sortcol)
{
case EVTTIME:
case EVTTYPE:
case EVTCAT:
case EVTDESC:
gtk_tree_model_get(model, a, sortcol, &str1, -1);
gtk_tree_model_get(model, b, sortcol, &str2, -1);
ret = strcmp(str1, str2);
break;
case EVTMAG:
case EVTLAT:
case EVTLON:
case EVTDEP:
gtk_tree_model_get(model, a, sortcol, &str1, -1);
gtk_tree_model_get(model, b, sortcol, &str2, -1);
val1 = atof(str1);
val2 = atof(str2);
ret = val1 - val2;
break;
}
return ret;
}
static GtkWidget *makeEventHdrModel()
{
int   i;
GtkWidget *treeview;
GtkTreeSortable *sortModel;
eventStore = gtk_list_store_new(9, G_TYPE_STRING, G_TYPE_STRING, 
G_TYPE_STRING, G_TYPE_STRING, 
G_TYPE_STRING, G_TYPE_STRING,
G_TYPE_STRING, G_TYPE_STRING,
G_TYPE_INT);
sortModel = GTK_TREE_SORTABLE(eventStore);
for(i=0;i<TTLEVTCOLS;i++)
{ 
gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(sortModel), i, sortCol, GINT_TO_POINTER(i), NULL);
}
treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL (eventStore));
return (treeview);
}
static void add_event_hdr_columns(GtkTreeView *treeview)
{
int    i;
GtkCellRenderer *renderer;
GtkTreeViewColumn *col;
for(i=0;i<TTLEVTCOLS;i++)
{
renderer = gtk_cell_renderer_text_new();
col = gtk_tree_view_column_new_with_attributes(eventCols[i], renderer, "text", i, NULL);
gtk_tree_view_column_set_sort_column_id(col, i);
gtk_tree_view_insert_column(treeview, col, i);
}
}
static void add_events(GSList *events)
{
GtkTreeIter iter;
EVENTINFO *evtInfo;
GSList  *iter1;
int   i;
char  *loc[TTLSTNEVTLOCS];
if (eventStore)
gtk_list_store_clear(eventStore);
for(iter1 = events;
iter1;
iter1 = g_slist_next(iter1))
{
evtInfo = iter1->data;
gtk_list_store_append(eventStore, &iter);
for(i=0;i<TTLSTNEVTLOCS;i++)
{
switch(i)
{
case STNEVTLAT:
case STNEVTLON:
loc[i] = g_strdup_printf("%0.3f", evtInfo->loc[i]);
break;
case STNEVTMAG:
case STNEVTDEP:
loc[i] = g_strdup_printf("%0.1f", evtInfo->loc[i]);
break;
}
}
gtk_list_store_set(eventStore, &iter, 0, evtInfo->timeStamp,
1, loc[STNEVTMAG],
2, evtInfo->magType,
3, loc[STNEVTLAT],
4, loc[STNEVTLON],
5, loc[STNEVTDEP],
6, evtInfo->catalog,
7, evtInfo->description, 
8, evtInfo->originID, -1);
for(i=0;i<TTLSTNEVTLOCS;i++)
free(loc[i]);
}
}
void eventSelect(GtkButton *button, void *nothing)
{ 
if (!stnEventWindow)
{
make_stnEventScreens();
}
gtk_widget_show(stnEventWindow);
gtk_notebook_set_current_page((GtkNotebook *) stnEventScreens, EVTEVTCAT);
gtk_grab_add(stnEventWindow);
return;
}
static void getStnEventInfo(int which)
{ 
int i;
EVENTINFO *event;
switch(which)
{
case EVTEVTUSER:
event = &stnStates.event.user;
memset(event, 0, sizeof(EVENTINFO));
for(i=0;i<TTLSTNEVTTIMES;i++)
{
event->dtime[i] = atoi(gtk_entry_get_text(GTK_ENTRY(stnEvtUserTime[i])));
}
for(i=0;i<TTLSTNEVTLOCS;i++)
{
event->loc[i] = atof(gtk_entry_get_text(GTK_ENTRY(stnEvtLoc[i])));
}
event->timeStamp = g_strdup_printf("%d-%02d-%02d %02d:%02d:%02d.%04d",
event->dtime[STNEVTYR], event->dtime[STNEVTMO], event->dtime[STNEVTDY],
event->dtime[STNEVTHR], event->dtime[STNEVTMN], event->dtime[STNEVTSC], 
event->dtime[STNEVTMSC]);
event->magType = g_strdup("U");
event->catalog = g_strdup("USER");
stnStates.event.event = &stnStates.event.user;
event = stnStates.event.event;
break;
case EVTEVTCAT:
{
event = stnStates.event.event;
}
break;
}
if (!event)
return;
if (!event->eventTE)
{ 
PTIME eventT;
memset(&eventT, 0, sizeof(PTIME));
eventT.yr = event->dtime[STNEVTYR];
eventT.mo = event->dtime[STNEVTMO];
eventT.day = event->dtime[STNEVTDY];
eventT.hr = event->dtime[STNEVTHR];
eventT.mn = event->dtime[STNEVTMN];
eventT.sec = event->dtime[STNEVTSC];
event->eventTE = (double) passcal_htoe(&eventT, TRUE);
event->eventTE += (float) ((float)event->dtime[STNEVTMSC])/10000.;
}
}
static void xclStnEvents(GtkButton *button, gpointer nil)
{
if (stnEventWindow)
{ 
gtk_widget_hide(stnEventWindow);
gtk_grab_remove(stnEventWindow);
}
}
static void doStnEvents(GtkButton *button, gpointer w)
{
int  eventType = GPOINTER_TO_INT(w);
if (stnEventWindow)
{ 
gtk_widget_hide(stnEventWindow);
gtk_grab_remove(stnEventWindow);
}
getStnEventInfo(eventType);
calcArrivals(TRUE);
#if 0
if (stnStates.event.arrivals)
setSensitive(STNSYSTEM, STNNOEVENTSENS, STNANALYSIS, TRUE);
else
setSensitive(STNSYSTEM, STNNOEVENTSENS, STNANALYSIS, FALSE);
stnStates.pmap.dirty[STNANALYSIS][0] = TRUE;
makeSTNDrawing(STNANALYSIS);
#endif
}
static gint insertEventLst(EVENTINFO *one, EVENTINFO *two)
{
return(strcmp(one->timeStamp, two->timeStamp));
}
static void procEvents(GtkButton *button, gpointer a)
{
EVENTINFO *evtInfo;
GSList  *iter;
int   action = GPOINTER_TO_INT(a);
switch(action)
{
case EVTCLEAR:
{
if (eventStore)
gtk_list_store_clear(eventStore);
for(iter = stnStates.event.events;
iter;
iter = g_slist_next(iter))
{
evtInfo = iter->data;
free(evtInfo->catalog);
free(evtInfo->timeStamp);
free(evtInfo);
}
g_slist_free(stnStates.event.events);
stnStates.event.events = NULL;
stnStates.event.event = NULL;
clearArrivals(stnStates.event.arrivals);
stnStates.event.arrivals = NULL;
}
break;
case EVTGET:
{
SQLDI sqldi;
SEISEVENTS events;
EVENTINFO *evtInfo;
static GtkWidget *eventDate, *calendar;
if (!eventDate)
{
eventDate = gtk_dialog_new_with_buttons("Date of Events",
GTK_WINDOW(topWindow),
GTK_DIALOG_DESTROY_WITH_PARENT,
GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
GTK_STOCK_OK, GTK_RESPONSE_OK,
NULL);
calendar = gtk_calendar_new();
gtk_container_add(GTK_CONTAINER(GTK_DIALOG(eventDate)->vbox), calendar);
gtk_widget_show_all(eventDate);
}
else
{
gtk_widget_show(eventDate);
}
memset(&sqldi, 0, sizeof(SQLDI));
memset(&events, 0, sizeof(SEISEVENTS));
sqldi.args.dataEvent = GET_DATA;
sqldi.descr.tableData = EVENTINFOTBL;
sqldi.data = &events;
if (gtk_dialog_run (GTK_DIALOG (eventDate)) == GTK_RESPONSE_OK)
{
guint year, month, day;
gtk_calendar_get_date(GTK_CALENDAR(calendar), &year, &month, &day);
events.keys.day[STNEVTDY] = day;
events.keys.day[STNEVTMO] = month+1;
events.keys.day[STNEVTYR] = year;
}
gtk_widget_hide(eventDate);
if (!events.keys.day[STNEVTDY])
break;  
db_request(&sqldi);
if (events.data.events)
{
for(iter=events.data.events;
iter;
iter = g_slist_next(iter))
{ 
evtInfo = iter->data;
stnStates.event.events = g_slist_insert_sorted(stnStates.event.events, evtInfo, (GCompareFunc) insertEventLst);
}
add_events(stnStates.event.events);
}
else
{
char *msg = g_strdup_printf("\n\n\tNo Events found in DB for day %04d-%02d-%02d\t\n", 
events.keys.day[STNEVTYR],
events.keys.day[STNEVTMO],
events.keys.day[STNEVTDY]);
error_dialog("Retrieve EVENTs", msg);
free(msg);
}
}
break;
}
} 
#if 0
static void eventSel(GtkTreeView *treeview, gpointer nil)
{ 
char *rowS;
GtkTreePath *path;
gtk_tree_view_get_cursor (treeview, &path, NULL);
rowS = gtk_tree_path_to_string(path);
gtk_tree_path_free(path);
stnStates.event.event = g_slist_nth_data(stnStates.event.events, (guint) atoi(rowS));
g_free(rowS);
}
#endif
static gint getEvent(EVENTINFO *event, int *originID)
{
return(event->originID - *originID);
}
static void eventSel (GtkTreeSelection *selection, gpointer nil)
{ 
GtkTreeIter iter;
GtkTreeModel *model;
int  originID;
GSList *evt;
if (gtk_tree_selection_get_selected (selection, &model, &iter))
{
gtk_tree_model_get (model, &iter, EVTID, &originID, -1);
evt = g_slist_find_custom(stnStates.event.events, 
(gpointer) &originID, (GCompareFunc) getEvent);
stnStates.event.event = evt->data;
}
}
static GtkWidget *make_Event(int screen)
{
GtkWidget *controlBox, *control_box, *label, *eBox, *hbox, *hbox1, *hbox2;
GtkWidget *frame, *controlFrame, *actionB, *button, *entry, *vbox1, *vbox2, *vbox3;
GtkWidget *eventSW, *vbox;
GtkSizeGroup *size_group;
GtkTreeSelection *select;
control_box = gtk_event_box_new();
controlFrame = gtk_frame_new(NULL);
controlBox = gtk_vbox_new(FALSE, 0);
gtk_container_set_border_width(GTK_CONTAINER(controlBox), 5);
gtk_container_add(GTK_CONTAINER(controlFrame), controlBox);
gtk_container_add(GTK_CONTAINER(control_box), controlFrame);
switch(screen)
{
case EVTEVTCAT:
hbox = gtk_hbox_new(FALSE, 0);
eventSW = gtk_scrolled_window_new(NULL, NULL);
gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(eventSW),
GTK_SHADOW_ETCHED_IN);
gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(eventSW),
GTK_POLICY_AUTOMATIC,
GTK_POLICY_AUTOMATIC);
eventVIEW = makeEventHdrModel();
gtk_container_add(GTK_CONTAINER(eventSW), eventVIEW);
gtk_box_pack_start(GTK_BOX(hbox), eventSW, TRUE, TRUE, 2);
add_event_hdr_columns(GTK_TREE_VIEW(eventVIEW));
gtk_widget_set_size_request(eventSW, 900, 350);
gtk_box_pack_start(GTK_BOX(controlBox), hbox, TRUE, TRUE, 2);
select = gtk_tree_view_get_selection(GTK_TREE_VIEW(eventVIEW));
g_signal_connect(G_OBJECT(select), "changed", G_CALLBACK(eventSel), NULL);
actionB = gtk_hbutton_box_new();
gtk_button_box_set_layout(GTK_BUTTON_BOX(actionB), GTK_BUTTONBOX_SPREAD);
gtk_box_set_spacing(GTK_BOX(actionB), 2);
button = gtk_button_new_with_label("Continue");
gtk_container_add(GTK_CONTAINER(actionB), button);
g_signal_connect(button, "clicked", G_CALLBACK(doStnEvents), GINT_TO_POINTER(EVTEVTCAT));
strcpy(toolTip, "APPLY New Settings and RETURN");
gtk_tooltips_set_tip(GTK_TOOLTIPS(tips), button, toolTip, toolTip);
gtk_box_pack_start(GTK_BOX(controlBox), actionB, FALSE, FALSE, 2);
break;
case EVTEVTUSER:
size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
frame = gtk_frame_new("Event Information");
vbox = gtk_vbox_new(FALSE, 0);
hbox2 = gtk_hbox_new(FALSE, 0);
gtk_container_add(GTK_CONTAINER(frame), vbox);
gtk_container_add(GTK_CONTAINER(vbox), hbox2);
gtk_box_pack_start(GTK_BOX(controlBox), frame, FALSE, FALSE, 2);
vbox1 = gtk_vbox_new(FALSE, 0);
label = gtk_label_new("Time:");
gtk_size_group_add_widget(size_group, label);
gtk_box_pack_start(GTK_BOX(vbox1), label, FALSE, FALSE, 2);
label = gtk_label_new("Date:");
gtk_size_group_add_widget(size_group, label);
gtk_box_pack_start(GTK_BOX(vbox1), label, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox2), vbox1, FALSE, FALSE, 0);
hbox1 = gtk_hbox_new(FALSE, 0);
stnEvtUserTime[STNEVTHR] = entry = gtk_entry_new();
label = gtk_label_new(":");
gtk_entry_set_width_chars(GTK_ENTRY(entry), 2);
hbox = gtk_hbox_new(FALSE, 0);
gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox1), hbox, FALSE, FALSE, 0);
stnEvtUserTime[STNEVTMN] = entry = gtk_entry_new();
label = gtk_label_new(":");
gtk_entry_set_width_chars(GTK_ENTRY(entry), 2);
hbox = gtk_hbox_new(FALSE, 0);
gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox1), hbox, FALSE, FALSE, 0);
stnEvtUserTime[STNEVTSC] = entry = gtk_entry_new();
label = gtk_label_new(".");
gtk_entry_set_width_chars(GTK_ENTRY(entry), 2);
hbox = gtk_hbox_new(FALSE, 0);
gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox1), hbox, FALSE, FALSE, 0);
stnEvtUserTime[STNEVTMSC] = entry = gtk_entry_new();
gtk_entry_set_width_chars(GTK_ENTRY(entry), 3);
hbox = gtk_hbox_new(FALSE, 0);
gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox1), hbox, FALSE, FALSE, 0);
vbox2 = gtk_vbox_new(FALSE, 0);
gtk_box_pack_start(GTK_BOX(vbox2), hbox1, FALSE, FALSE, 0);
hbox1 = gtk_hbox_new(FALSE, 0);
stnEvtUserTime[STNEVTDY] = entry = gtk_entry_new();
label = gtk_label_new("/");
gtk_entry_set_width_chars(GTK_ENTRY(entry), 2);
hbox = gtk_hbox_new(FALSE, 0);
gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox1), hbox, FALSE, FALSE, 0);
stnEvtUserTime[STNEVTMO] = entry = gtk_entry_new();
label = gtk_label_new("/");
gtk_entry_set_width_chars(GTK_ENTRY(entry), 2);
hbox = gtk_hbox_new(FALSE, 0);
gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox1), hbox, FALSE, FALSE, 0);
stnEvtUserTime[STNEVTYR] = entry = gtk_entry_new();
gtk_entry_set_width_chars(GTK_ENTRY(entry), 4);
hbox = gtk_hbox_new(FALSE, 0);
gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox1), hbox, FALSE, FALSE, 0);
gtk_box_pack_start(GTK_BOX(vbox2), hbox1, FALSE, FALSE, 0);
gtk_box_pack_start(GTK_BOX(hbox2), vbox2, FALSE, FALSE, 0);
vbox3 = gtk_vbox_new(FALSE, 0);
label = gtk_label_new(" HH:MM:SS.mSec");
gtk_box_pack_start(GTK_BOX(vbox3), label, FALSE, FALSE, 2);
label = gtk_label_new(" DD/MM/YYYY");
gtk_box_pack_start(GTK_BOX(vbox3), label, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(hbox2), vbox3, FALSE, FALSE, 0);
hbox = gtk_hbox_new(FALSE, 0);
label = gtk_label_new("LAT:");
gtk_size_group_add_widget(size_group, label);
gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
stnEvtLoc[STNEVTLAT] = entry = gtk_entry_new();
gtk_entry_set_width_chars(GTK_ENTRY(entry), 10);
gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
hbox = gtk_hbox_new(FALSE, 0);
label = gtk_label_new("LON:");
gtk_size_group_add_widget(size_group, label);
gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
stnEvtLoc[STNEVTLON] = entry = gtk_entry_new();
gtk_entry_set_width_chars(GTK_ENTRY(entry), 10);
gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
hbox = gtk_hbox_new(FALSE, 0);
label = gtk_label_new("Depth:");
gtk_size_group_add_widget(size_group, label);
gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
stnEvtLoc[STNEVTDEP] = entry = gtk_entry_new();
gtk_entry_set_width_chars(GTK_ENTRY(entry), 10);
gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
hbox = gtk_hbox_new(FALSE, 0);
label = gtk_label_new("Mag:");
gtk_size_group_add_widget(size_group, label);
gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
stnEvtLoc[STNEVTMAG] = entry = gtk_entry_new();
gtk_entry_set_width_chars(GTK_ENTRY(entry), 10);
gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 2);
gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
actionB = gtk_hbutton_box_new();
gtk_button_box_set_layout(GTK_BUTTON_BOX(actionB), GTK_BUTTONBOX_SPREAD);
gtk_box_set_spacing(GTK_BOX(actionB), 2);
button = gtk_button_new_with_label("Continue");
gtk_container_add(GTK_CONTAINER(actionB), button);
g_signal_connect(button, "clicked", G_CALLBACK(doStnEvents), GINT_TO_POINTER(EVTEVTUSER));
strcpy(toolTip, "APPLY New Settings and RETURN");
gtk_tooltips_set_tip(GTK_TOOLTIPS(tips), button, toolTip, toolTip);
gtk_box_pack_start(GTK_BOX(controlBox), actionB, FALSE, FALSE, 2);
eBox = gtk_event_box_new();
gtk_widget_modify_bg(eBox, GTK_STATE_NORMAL, &stnSettings.drawing.colors[screen][STNSB]);
gtk_container_add(GTK_CONTAINER(controlBox), eBox);
break; 
}
return control_box;
}
static GtkWidget *make_EventButtons()
{
GtkWidget *widget, *button, *menu_box;
GtkWidget *actionB;
menu_box = gtk_event_box_new();
widget = gtk_vbox_new(FALSE, 0);
gtk_container_set_border_width(GTK_CONTAINER(widget), 5);
gtk_container_add(GTK_CONTAINER(menu_box), widget);
actionB = gtk_vbutton_box_new();
gtk_box_pack_start(GTK_BOX(widget), actionB, FALSE, FALSE, 0);
gtk_button_box_set_layout(GTK_BUTTON_BOX(actionB), GTK_BUTTONBOX_START);
gtk_box_set_spacing(GTK_BOX(actionB), 2);
button = gtk_button_new_with_label("Cancel");
gtk_container_add(GTK_CONTAINER(actionB), button);
g_signal_connect(button, "clicked", G_CALLBACK(xclStnEvents), NULL);
strcpy(toolTip, "Cancel this Request");
gtk_tooltips_set_tip(GTK_TOOLTIPS(tips), button, toolTip, toolTip);
button = gtk_button_new_with_label("Clear");
gtk_container_add(GTK_CONTAINER(actionB), button);
g_signal_connect(button, "clicked", G_CALLBACK(procEvents), GINT_TO_POINTER(EVTCLEAR));
strcpy(toolTip, "Clear the List of Events");
gtk_tooltips_set_tip(GTK_TOOLTIPS(tips), button, toolTip, toolTip);
button = gtk_button_new_with_label("Retrieve");
gtk_container_add(GTK_CONTAINER(actionB), button);
g_signal_connect(button, "clicked", G_CALLBACK(procEvents), GINT_TO_POINTER(EVTGET));
strcpy(toolTip, "Retrieve Events For a Specific Day");
gtk_tooltips_set_tip(GTK_TOOLTIPS(tips), button, toolTip, toolTip);
gtk_widget_modify_bg(menu_box, GTK_STATE_NORMAL, &settings.colors.sb[TRACESCR]);
return menu_box;
}
void make_stnEventScreens()
{
GtkWidget *ctrlBox, *vbox;
GtkWidget *frame, *container;
if (!stnEventWindow)
{
tips = gtk_tooltips_new();
stnEventWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
gtk_window_set_title(GTK_WINDOW(stnEventWindow), "STN System - Events"); 
gtk_container_set_border_width(GTK_CONTAINER(stnEventWindow), 2);
gtk_window_set_position(GTK_WINDOW(stnEventWindow), GTK_WIN_POS_CENTER);
ctrlBox = gtk_hbox_new (FALSE, 0);
gtk_container_add (GTK_CONTAINER(stnEventWindow), ctrlBox);
container = make_EventButtons ();
gtk_box_pack_start(GTK_BOX(ctrlBox), container, FALSE, FALSE, 0);
vbox = gtk_vbox_new(FALSE, 0);
stnEventScreens = gtk_notebook_new ();
gtk_box_pack_start(GTK_BOX(vbox), stnEventScreens, TRUE, TRUE, 0);
gtk_box_pack_start(GTK_BOX(ctrlBox), vbox, TRUE, TRUE, 0);
#if 0
statusBarCtrl = gtk_statusbar_new();
gtk_box_pack_start(GTK_BOX(vbox), statusBarCtrl, FALSE, FALSE, 0);
gtk_box_pack_start(GTK_BOX(ctrlBox), vbox, TRUE, TRUE, 0);
#endif
frame = gtk_frame_new(NULL);
gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
container = make_Event(EVTEVTCAT);
gtk_container_add(GTK_CONTAINER(frame), container);
gtk_notebook_append_page (GTK_NOTEBOOK(stnEventScreens), frame, gtk_label_new(" Catalog Events"));
frame = gtk_frame_new(NULL);
gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
container = make_Event(EVTEVTUSER);
gtk_container_add(GTK_CONTAINER(frame), container);
gtk_notebook_append_page (GTK_NOTEBOOK(stnEventScreens), frame, gtk_label_new(" User Event"));
gtk_window_set_transient_for((GtkWindow *) stnEventWindow, (GtkWindow *) topWindow); 
gtk_widget_show_all (stnEventWindow);
}
return;
}
