#!/bin/bash

if [ $# -ne 5 ] ; then
	echo "usage: $0 [HOST:]DBName NTW STN LOC CHN"
	echo "example:	$0 micros IU ANMO -- BHE"
	echo "	Verifies the integrity of the PSD tables for this channel"
	exit
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

HOST=`echo $1 | sed 's/:.*//'`
DBn=`echo $1 | sed 's/.*://'`
if [ "$HOST" = "$DBn" ] ; then
	HOST=localhost
fi

NTWi=$2
STNi=$3
LOCi=$4
CHNi=$5

# check that the database exists
check=`$PQLXBIN/_checkDB $HOST $DBn DE`
if [ "$check" = "FALSE" ] ; then
	exit
fi

PWORD=PqLXClnT
execCMD="${mysqlexec} --host=$HOST --database=$DBn --user=pqlxClnt --password=$PWORD --compress"

chniFK=`echo "SELECT chni_pk FROM  ${DBn}.nslc WHERE ntw = '$NTWi' and stn = '$STNi' and loc = '$LOCi' and chn = '$CHNi'" | ${execCMD} | grep -v chni_pk`
if [ "$chniFK" = "" ] ; then
	echo "No channel instance found for $NTWi.$STNi.$LOCi.$CHNi, please try again..."
	echo "Quitting without doing anything... Goodbye"
	exit
fi

echo "Verifying PSD Table Integrity for Channel ($NTWi $STNi $LOCi $CHNi)..."

dataDB="\`${DBn}DATA\`"
PSDidx="${dataDB}.\`psd${chniFK}idx\`"
PSDpsd="${dataDB}.\`psd${chniFK}\`"
tmpDB="\`${DBn}TMP\`"
tmpTBL="${tmpDB}.\`PSDCNT${chniFK}\`"

sql="DROP TABLE IF EXISTS ${tmpTBL}"
echo $sql | ${execCMD}
sql="CREATE TABLE ${tmpTBL} ENGINE=MEMORY SELECT P2.psd_pk FROM ${PSDidx} P2 WHERE NOT EXISTS (SELECT trc_pk FROM TRC T WHERE T.chni_fk = ${chniFK} AND P2.trc_fk = T.trc_pk)"
echo $sql | ${execCMD}

sql="SELECT COUNT(*) cnt FROM ${tmpTBL}"
cnt=`echo $sql | ${execCMD} | grep -v cnt`
if [ $cnt -eq 0 ] ; then
  sql="DROP TABLE ${tmpTBL}"
  echo $sql | ${execCMD}
  echo "Integrity Verified, No dangling PSD's found"
  echo
  exit
fi

echo "Dangling PSD's found, removing..."
sql="DELETE ${dataDB}.P1 FROM ${PSDpsd} P1, ${tmpTBL} P2 WHERE P1.psd_fk = P2.psd_pk"
echo $sql | ${execCMD}

sql="DELETE ${dataDB}.P1 FROM ${PSDidx} P1, ${tmpTBL} P2 WHERE P1.psd_pk = P2.psd_pk"
echo $sql | ${execCMD}

echo "PSD Tables Verified, ALL dangling PSD's removed."
echo "Re-run to confirm, if so desired..."
echo


