#!/bin/bash

if	[ "$1" = "" ] || \
	[ "$2" != "ADD" -a "$2" != "DISABLE" -a "$2" != "ENABLE" ] || \
	[ "$3" = "" ] ; then
	if !( [ "$2" == "LIST" ] && [ "$3" = "" ] ) ; then
		echo "Usage:	$0 dbName ( LIST | ADD | DISABLE | ENABLE ) [dataDirectory1 dataDirectory2 ...]"
		echo
		echo "	LIST    - List all Directories currently defined, their status, and time of last scan"
		echo "	ADD     - Define one or more new data directories to be scanned"
		echo "	DISABLE - Disable one or more data directories (already defined) from being scanned, do not delete"
		echo "	ENABLE  - Enable one or more data directories (already defined) to be scanned"
		echo "	(Can only be executed on localhost of MYSQL server)"
		exit
	fi
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

DBName=$1
CMD=$2

# first verify our connection and the requested database
check=`$PQLXBIN/_checkDB localhost $DBName DE`
if [ "$check" == "FALSE"  ] ; then
	exit
fi

# set up the MySQL password and command
PWORD=PqLXAdmiN
execCMD="${mysqlexec} --user=pqlxAdmin --password=$PWORD --compress"

if [ "${CMD}" = "ADD" ] ; then
	echo "ADDing Data Directories to PQLX database $1..."
elif [ "${CMD}" = "DISABLE" ] ; then
	echo "DISABLEing Data Directories in PQLX database $1..."
elif [ "${CMD}" = "ENABLE" ] ; then
	echo "ENABLEing Data Directories in PQLX database $1..."
elif [ "${CMD}" = "LIST" ] ; then
	echo "LISTing all Data Directories in PQLX database $1..."
	echo
	echo "SELECT DIRECTORY, STATUS, FROM_UNIXTIME(time) FROM ${DBName}.DIRS ORDER BY STATUS, time DESC" | ${execCMD} | grep -v DIRECTORY
	exit
fi

num=0
for arg; do
	if [ $num -lt 2 ] ; then
		num=`expr $num + 1`
		continue;
	fi
	
	# remove trailing '/' if there is one
	directory=`echo $arg | sed s:\/$::`
	
	# cnt = total number of current records for this $directory
	sqlCMD="SELECT count(*) as cnt FROM ${DBName}.DIRS WHERE directory = '${directory}'"
	cnt=`echo ${sqlCMD} | ${execCMD} | grep -v cnt`

	if [ "${CMD}" = "ADD" ] ; then
		# cnt2 = total number of DIRS.DIRECTORY that are a sub-directory of $directory 
		sqlCMD="SELECT count(*) as cnt FROM ${DBName}.DIRS WHERE STATUS = 'ACTIVE' AND LOCATE(CONCAT(directory, '/'), '${directory}') > 0"
		cnt2=`echo ${sqlCMD} | ${execCMD} | grep -v cnt`
		# cnt3 = total number $directory that is a sub-directory of DIRS.DIRECTORY
		sqlCMD="SELECT count(*) as cnt FROM ${DBName}.DIRS WHERE STATUS = 'ACTIVE' AND LOCATE('${directory}', directory) > 0"
		cnt3=`echo ${sqlCMD} | ${execCMD} | grep -v cnt`
		
		echo "ADDing Directory $directory"
		if [ "$cnt" -gt "0" ] ; then
			echo "Directory ${directory} Already Exists, Refusing to Duplicate"
			continue
		elif [ "$cnt2" -gt "0" ] ; then
			echo "Directory '${directory}' is a sub-directory of a currently ACTIVE entry, Refusing to Duplicate"
			echo "If this is correct, DISABLE the currently ACTIVE directory and try again."
			continue
		elif [ "$cnt3" -gt "0" ] ; then
			echo "Directory '${directory}' is a parent directory of a currently ACTIVE entry, Refusing to Duplicate"
			echo "If this is correct, DISABLE the currently ACTIVE directory and try again."
			continue
		else
			sqlCMD="INSERT into ${DBName}.DIRS (DIRECTORY) VALUES ('$directory')"
		fi
	fi
	
	if [ "${CMD}" = "DISABLE" ] ; then
		echo "DISABLEing Directory $directory"
		if [ "$cnt" -ne "1" ] ; then
			echo "Directory ${directory} Does Not Exist, cannot DISABLE"
			continue
		else
			sqlCMD="UPDATE ${DBName}.DIRS set STATUS = 'INACTIVE' where DIRECTORY = '$directory'"
		fi
	fi
	
	if [ "${CMD}" = "ENABLE" ] ; then
		echo "ENABLEing Directory $directory"
		if [ "$cnt" -ne "1" ] ; then
			echo "Directory ${directory} Does Not Exist, cannot ENABLE"
			continue
		else
			sqlCMD="UPDATE ${DBName}.DIRS set STATUS = 'ACTIVE' where DIRECTORY = '$directory'"
		fi
	fi
	echo ${sqlCMD} | ${execCMD}
done
