#!/bin/bash

if [ $# -ne 6 ] ; then
	echo "usage: $0 [HOST:]DBName chni_pk NTW STN LOC CHN"
	echo "example:	$0 micros 3028 IU ANMO 00 LHE"
	echo "	fully deletes the specified channel, using its primary key, from the specified PQLX database"
	echo "  including all corresponding data"
	exit
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

HOST=`echo $1 | sed 's/:.*//'`
DBn=`echo $1 | sed 's/.*://'`
chniFK=$2
NTW=$3
STN=$4
LOC=$5
CHN=$6

if [ "$HOST" = "$DBn" ] ; then
	HOST=localhost
fi

# check that the database exists
check=`$PQLXBIN/_checkDB $HOST $DBn DE`
if [ "$check" == "FALSE"  ] ; then
	exit
fi

PWORD=PqLXAdmiN
execCMD="${mysqlexec} --host=$HOST --database=$DBn --user=pqlxAdmin --password=$PWORD --compress"

ntw=`echo "SELECT ntw FROM ${DBn}.nslc WHERE chni_pk = ${chniFK}" | ${execCMD} | grep -v ntw`
stn=`echo "SELECT stn FROM ${DBn}.nslc WHERE chni_pk = ${chniFK}" | ${execCMD} | grep -v stn`
loc=`echo "SELECT loc FROM ${DBn}.nslc WHERE chni_pk = ${chniFK}" | ${execCMD} | grep -v loc`
chn=`echo "SELECT chn FROM ${DBn}.nslc WHERE chni_pk = ${chniFK}" | ${execCMD} | grep -v chn`
if [ "$NTW" != "$ntw" -o "$STN" != "$stn" -o "$LOC" != "$loc" -o "$CHN" != "$chn" ] ; then
	echo "Channel key vs. nlsc table information does not match, please try again..."
	echo "Either the Key you provided is wrong, or the station name details are wrong."
	echo "Quitting without doing anything... Goodbye"
	exit
fi

echo -n "Removing channel $chniFK from PQLX database ${DBn}..."

sql="DELETE GLAP FROM GLAP, TRC WHERE GLAP.trc_fk = TRC.trc_pk and TRC.chni_fk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM TRC WHERE chni_fk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM CHNI where chni_pk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM CHNMETA where chni_fk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM EPOCHSPS where chni_fk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM RESP where chni_fk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM EVENTQ where chni_fk = ${chniFK}"
echo $sql | ${execCMD}

psd=`echo "SELECT chni_fk FROM  ${DBn}.PSDCFG where chni_fk = ${chniFK}" | ${execCMD} | grep -v chni_pk`
if [ "$psd" != "" ] ; then
	PSDidx=${DBn}DATA.psd${chniFK}idx
	PSDpsd=${DBn}DATA.psd${chniFK}
	PSDpdf=${DBn}DATA.pdf${chniFK}
	BLine=${DBn}STATS.BLineS${chniFK}
	Hits=${DBn}STATS.hits${chniFK}
	Stats=${DBn}STATS.stats${chniFK}

	sql="DELETE FROM PDFIDX where chni_fk = ${chniFK}"
	echo $sql | ${execCMD}
	sql="DELETE FROM PSDCFG where chni_fk = ${chniFK}"
	echo $sql | ${execCMD}
	sql="DROP TABLE IF EXISTS ${PSDidx}"
	echo $sql | ${execCMD}
	sql="DROP TABLE IF EXISTS ${PSDpsd}"
	echo $sql | ${execCMD}
	sql="DROP TABLE IF EXISTS ${PSDpdf}"
	echo $sql | ${execCMD}
	sql="DROP TABLE IF EXISTS ${BLine}"
	echo $sql | ${execCMD}
	sql="DROP TABLE IF EXISTS ${Hits}"
	echo $sql | ${execCMD}
	sql="DROP TABLE IF EXISTS ${Stats}"
	echo $sql | ${execCMD}
fi

echo "... Done"
