#!/bin/bash

if [ $# -ne 5 ] ; then
	echo "usage: $0 [HOST:]DBName NTW STN LOC CHN"
	echo "example:	$0 micros IU ANMO -- BHE"
	echo "	Delete the Channel from the Specified PQLX Database"
	echo "  Including ALL Corresponding Data"
	exit
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

HOST=`echo $1 | sed 's/:.*//'`
DBn=`echo $1 | sed 's/.*://'`
NTWi=$2
STNi=$3
LOCi=$4
CHNi=$5

if [ "$HOST" = "$DBn" ] ; then
	HOST=localhost
fi

# check that the database exists
check=`$PQLXBIN/_checkDB $HOST $DBn DE`
if [ "$check" == "FALSE"  ] ; then
	exit
fi

PWORD=PqLXAdmiN
execCMD="${mysqlexec} --host=$HOST --database=$DBn --user=pqlxAdmin --password=$PWORD --compress"

chniFK=`echo "SELECT chni_pk FROM  ${DBn}.nslc WHERE ntw = '$NTWi' and stn = '$STNi' and loc = '$LOCi' and chn = '$CHNi'" | ${execCMD} | grep -v chni_pk`
if [ "$chniFK" = "" ] ; then
	echo "No channel instance found for $NTWi.$STNi.$LOCi.$CHNi, please try again..."
	echo "Quitting without doing anything... Goodbye"
	exit
fi

PSDidx=${DBn}DATA.psd${chniFK}idx
PSDpsd=${DBn}DATA.psd${chniFK}
PSDpdf=${DBn}DATA.pdf${chniFK}
STATShits=${DBn}STATS.hits${chniFK}
STATSstats=${DBn}STATS.stats${chniFK}
TMPtable=${DBn}TMP.dbMaint

echo "Removing channel $NTWi.$STNi.$LOCi.$CHNi from PQLX database ${DBn}..."

sql="DELETE E.* FROM EVENTQ E, TRC T where E.type=1 and E.typeKey = T.trc_pk and T.chni_fk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE E.* FROM EVENTQ E, TRC T where E.type=2 and E.typeKey = T.trc_pk and T.chni_fk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE E.* FROM EVENTQ E, TRC T where E.type=3 and E.typeKey = T.trc_pk and T.chni_fk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE GLAP FROM GLAP, TRC WHERE GLAP.trc_fk = TRC.trc_pk and TRC.chni_fk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE GLAP FROM GLAP, EVENTQ e WHERE GLAP.trc_fk = e.typeKey and e.type = 91 and e.typeKey2 = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM TRC WHERE chni_fk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM EVENTQ where type = 91 and typeKey2 = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM EVENTQ where type=90 and typeKey = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM EVENTQ where type=100 and typeKey = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM EVENTQ where type=101 and typeKey = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM CHNI where chni_pk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM PDFIDX where chni_fk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM RESP where chni_fk = ${chniFK}"
echo $sql | ${execCMD}
sql="DELETE FROM ${TMPtable} where chni_fk = ${chniFK}"
echo $sql | ${execCMD}

# check if PSD data needs to be deleted
psd=`echo "SELECT count(*) AS cnt FROM  ${DBn}.PSDCFG where chni_fk = ${chniFK}" | ${execCMD} | grep -v cnt`
if [ "$psd" -ne "0" ] ; then
	sql="DELETE FROM PSDCFG where chni_fk = ${chniFK}"
	echo $sql | ${execCMD}
	sql="DROP TABLE ${PSDidx}"
	echo $sql | ${execCMD}
	sql="DROP TABLE ${PSDpsd}"
	echo $sql | ${execCMD}
	sql="DROP TABLE ${PSDpdf}"
	echo $sql | ${execCMD}
	sql="DROP TABLE ${STATShits}"
	echo $sql | ${execCMD}
	sql="DROP TABLE ${STATSstats}"
	echo $sql | ${execCMD}
fi
	
echo "... Done"
