#!/bin/bash

# script initializes the mySQL server with our environment
echo
echo "Initializing MYSQL for PQLX System Use..."

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi
T=TRUE
F=FALSE

if [ "$1" != "" ] ; then
  autoRun=$T
  rootPW=$1
else
  autoRun=$F
fi

initCheckI=$T
initCheckNI=$F

echo
# check the version of MySQL, we can't operate against anything < 5.*

if [ "$autoRun" = "$F" ] ; then
  echo -n "Performing MySQL version check against ${mysqlexec} ...  "
  version=`${mysqlexec} --version | awk '{ print $5 }' | sed -e s/\,//`
  echo "(version = $version)"
  version=`${mysqlexec} --version | awk '{ print $5 }' | grep 5.*`

  if [ "$version" = "" ] ; then
	  echo "Version of MySQL is not >= 5.0, cannot continue."
	  echo "Please install version 5.0 or greater and try again."
	  echo "If version 5.0 or greater is installed, set the variable MYSQLBIN in the file env/makeVars"
	  echo "  to the directory where this version is installed."
	  echo "Also be certain that the MySQL database server is executed from this directory"
	  exit
  fi
  echo "MySQL version verified..."
  echo

	echo "MYSQL root user password is necessary for PQLX System Initialization"
	echo "    N.B. And it cannot be blank!"
	echo "    If it is, assign a password with the following mysql command"
	echo "    replacing RootPWord with the real one:"
	echo
	echo "    mysql> SET PASSWORD FOR 'root'@'localhost' = PASSWORD('RootPWord');"
	echo
	echo "    (Ctrl-C to kill this script at this point is okay)"
	echo
	echo "Please provide the MySQL root password: (password is not saved)"
	read rootPW
fi

# first check to see we've not already been called once
initCheck=`$PQLXBIN/_initPQLXCheck.sh ${rootPW}`

if [ "$initCheck" = "$initCheckI" ] ; then
	echo "The PQLX Database environment has already been initialized, cannot redo"
	echo "If you wish to start over, you must execute re-initPQLXdb.sh instead"
	echo "Quitting without doing anything..."
	echo
	exit
fi

# first set up our users
${mysqlexec} --user=root --password=${rootPW} <${PQLXDBDEF}/sql-sup/initPQLXdb.sql

# then create the PQLXMETA data table
${mysqlexec} --user=root --password=${rootPW} <${PQLXDBDEF}/sql-sup/makePQLXMETAdb.sql

check=`echo "show databases" | ${mysqlexec} --user=root --password=${rootPW} 2>/dev/null | grep PQLXMETA | wc -l`
case "$autoRun" in 
  "$T" )
    if [ $check -eq 1 ] ; then
      echo TRUE
    else
      echo FALSE
    fi
  ;;

  "$F" )
    if [ $check -eq 1 ] ; then
	    echo
	    echo "MySQL successfully initialized for PDF usage"
	    echo "Next Steps:"
	    echo "1) Create a PDF database definition file in $PQLXDBDEF, following example from file samplePQLXdb.pqlx"
	    echo "2) Create a PDF database with makePQLXdb.sh script"
    else
	    echo
	    echo "MySQL initialization failed, PQLX database PQLXMETA was not created."
	    echo "Please investigate cause, make proper adjustments and try again."
	    echo "Most likely causes:"
	    echo "	1) root password incorrect"
	    echo "	2) MySQL server not running"
	    echo
    fi
  ;;
esac

exit
