#!/bin/bash

if [ $# -ne 2 ] ; then
	echo "usage: $0 [HOST:]DBName directory"
	echo "	Insert All Sub-Directories of 'directory' to Specified PQLX DB as Directories to be Scanned"
	echo "example:	$0 micros /home/data/microsData"
	echo "	Add All Directories listed in directory /home/data/microsData to the micros DB"
	echo "	All Directories are Initialized to DISABLED"
	exit
fi

hostDB=$1
stations=$2

HOST=`echo ${hostDB} | sed 's/:.*//'`
DBn=`echo ${hostDB} | sed 's/.*://'`

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

if [ "$HOST" = "$DBn" ] ; then
	HOST=localhost
fi

# check that the database exists
check=`$PQLXBIN/_checkDB $HOST $DBn DE`
if [ "$check" == "FALSE"  ] ; then
	exit
fi

outFile=$PQLXLOG/insertDirs-`date +%Y.%j.%H%M`.log
dirs=`ls ${stations}`
for stn in ${dirs} ;  # until it's done
do
	if [ ! -d ${stn} ] ; then
		continue
	fi
	echo "Adding Directory ${stations}/${stn}"
	$PQLXBIN/PQLXDataDirectory.sh ${hostDB} ADD ${stations}/${stn} >>$outFile
	$PQLXBIN/PQLXDataDirectory.sh ${hostDB} DISABLE ${stations}/${stn} >>$outFile
	echo >>$outFile
done

$PQLXBIN/PQLXDataDirectory.sh ${hostDB} LIST
$PQLXBIN/PQLXDataDirectory.sh ${hostDB} LIST >>$outFile
echo "Complete... Detailed Logging in file '$outfile'"
