#!/bin/bash

if [ "$1" = "" -o "$2" = "" ] ; then
	echo "usage:	$0 dbName dataDirectory1 [dataDirectory2 ...]"
	exit
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

# first confirm the requested database doesn't already exist
check=`$PQLXBIN/_checkDB localhost $1 DNE`
if [ "$check" == "FALSE"  ] ; then
	exit
fi

# define the scan directories
num=0
for arg; do
	if [ $num -eq 0 ] ; then
		num=`expr $num + 1`
		continue;
	fi
	if [ ! -e $arg ] ; then
		echo "Directory $arg Does Not Exist...  Use This Directory Anyway? (Y/N)"
		read ans
		if [ "$ans" = "Y" ] ; then
			continue;
		else
			echo "Quitting without doing anything, goodbye..."
			exit
		fi
	fi
done

echo "Making PQLX Database defined in PQLX db definition file $1.pqlx..."

DBDEFFILE=${PQLXDBDEF}/$1.pqlx

if [ ! -e ${DBDEFFILE} ] ; then
	echo "PQLX Database Definition File not found (${DBDEFFILE}), cannot continue..."
	exit
fi

DBName=$1
DBOrg=`grep DBOrg ${DBDEFFILE} | sed s/DBOrg://`
DBDescr=`grep DBDescr ${DBDEFFILE} | sed s/DBDescr://`
DBTechAdmin=`grep DBTechAdmin ${DBDEFFILE} | sed s/DBTechAdmin://`
DBTechEmail=`grep DBTechEmail ${DBDEFFILE} | sed s/DBTechEmail://`
DBDataAdmin=`grep DBDataAdmin ${DBDEFFILE} | sed s/DBDataAdmin://`
DBDataEmail=`grep DBDataEmail ${DBDEFFILE} | sed s/DBDataEmail://`
DBType=`grep DBType ${DBDEFFILE} | sed s/DBType://`
DBAccess=`grep DBAccess ${DBDEFFILE} | sed s/DBAccess://`
DBAdminPWord=`grep DBAdminPWord ${DBDEFFILE} | sed s/DBAdminPWord://`
DBWWWDir=`grep DBWWWDir ${DBDEFFILE} | sed s/DBWWWDir://`
DBMaker=`whoami`
DBSQLFILE=${PQLXDBDEF}/sql-db/${DBName}.sql


if [ "$DBOrg" = "" ] ; then
	echo "DBOrg field is empty, adjust entry in file $1.pqlx and resubmit..."
	exit
fi

if [ "$DBDescr" = "" ] ; then
	echo "DBDescr field is empty, adjust entry in file $1.pqlx and resubmit..."
	exit
fi

if [ "$DBTechAdmin" = "" ] ; then
	echo "DBTechAdmin field is empty, adjust entry in file $1.pqlx and resubmit..."
	exit
fi

if [ "$DBTechEmail" = "" ] ; then
	echo "DBTechEmail field is empty, adjust entry in file $1.pqlx and resubmit..."
	exit
fi

if [ "$DBDataAdmin" = "" ] ; then
	echo "DBDataAdmin field is empty, adjust entry in file $1.pqlx and resubmit..."
	exit
fi

if [ "$DBDataEmail" = "" ] ; then
	echo "DBDataEmail field is empty, adjust entry in file $1.pqlx and resubmit..."
	exit
fi

if [ "$DBType" = "" ] ; then
	echo "DBType field is empty, adjust entry in file $1.pqlx and resubmit..."
	exit
fi

if [ "$DBType" != "PRODUCTION" -a "$DBType" != "SCRATCH" ] ; then
	echo "DBType field must be one of PRODUCTION or SCRATCH, adjust entry in file $1.pqlx and resubmit..."
	exit
fi

if [ "$DBAccess" = "" ] ; then
	echo "DBAccess field is empty, adjust entry in file $1.pqlx and resubmit..."
	exit
fi

if [ "$DBAccess" != "PUBLIC" -a "$DBAccess" != "PRIVATE" ] ; then
	echo "DBAccess field must be one of PUBLIC or PRIVATE, adjust entry in file $1.pqlx and resubmit..."
	exit
fi

if [ "$DBAdminPWord" = "" ] ; then
	echo "DBAdminPWord field is empty, please provide a System Admin Password in file $1.pqlx and resubmit..."
	exit
fi

if [ "$DBWWWDir" = "" ] ; then
	echo "DBWWWDir field is empty, please provide a valid directory or NONE in file $1.pqlx and resubmit..."
	exit
fi

# set up the MySQL password
PWORD=PqLXAdmiN
execCMD="${mysqlexec} --user=pqlxAdmin --password=$PWORD"

echo "INSERT INTO PQLXMETA.PQLXDBS (dbName, organisation, description, techAdminName, techAdminEmail, dataAdminName, dataAdminEmail, type, access, dbMaker, adminPWord, wwwDir) VALUES ('$DBName', '$DBOrg', '$DBDescr', '$DBTechAdmin', '$DBTechEmail', '$DBDataAdmin', '$DBDataEmail', '$DBType', '$DBAccess', '$DBMaker', '$DBAdminPWord', '$DBWWWDir')" | ${execCMD}

# create the DATA-only Database
echo "CREATE DATABASE \`${DBName}DATA\` CHARACTER SET latin1 COLLATE latin1_swedish_ci; " >${DBSQLFILE}

# create the TMP-only Database
echo "CREATE DATABASE \`${DBName}TMP\` CHARACTER SET latin1 COLLATE latin1_swedish_ci; " >>${DBSQLFILE}

# create the INFO-only Database
echo "CREATE DATABASE \`$DBName\` CHARACTER SET latin1 COLLATE latin1_swedish_ci; USE $DBName;" >>${DBSQLFILE}

cat ${PQLXDBDEF}/sql-sup/PQLXdb.sql >>${DBSQLFILE}

${execCMD} <${DBSQLFILE}

# define the scan directories
num=0
for arg; do
	if [ $num -eq 0 ] ; then
		num=`expr $num + 1`
		continue;
	fi
	echo "INSERT into \`${DBName}\`.DIRS (DIRECTORY) VALUES ('$arg')" | ${execCMD}
done
# remove any trailing '/' characters, required for dirsUpd() routine
echo "UPDATE \`${DBName}\`.DIRS SET directory = SUBSTR(directory, 1, length(directory)-1) WHERE RIGHT(directory, 1) = '/'" | ${execCMD}

# verify that the database was really successfully created before issuing the next message
check=`echo "select count(*) from PQLXMETA.PQLXDBS where dbName='$1'" | ${execCMD} | grep -v count`
if [ $check -ne 0 ] ; then
	$PQLXBIN/pqlxSrvr --noScan --dbName=$1 
	echo
	echo "PQLX Database $1 succesfully created..."
	echo "Use pqlx-admin to start the Analysis Program"
fi
