#!/bin/bash

if [ "$1" = "" ] ; then
	echo "usage: $0 [HOST:]DBName"
	echo
	echo "Provides a Summary of all Errors currently existing on the EVENTQ table"
	echo "For the following error types:"
	echo "	1) Channels missing a response file"
	echo "	2) Channels with a sample rate change, including date boundaries"
	echo "	3) Files larger than 30Mb and unprocessed, including file size"
#	echo "	4) Files causing server crash, including event type"
	echo "	4) Channels having an evalresp() error"
	echo
	echo "Output written to \$PQLXLOG/ES-dbName-YYYY.JJJ.HHMM.log"
	echo
	echo "example:"
	echo "	$0 ANSS"
	exit
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

HOST=`echo $1 | sed 's/:.*//'`
DBn=`echo $1 | sed 's/.*://'`

if [ "$HOST" = "$DBn" ] ; then
	HOST=localhost
fi

# check that the database exists
check=`$PQLXBIN/_checkDB $HOST $DBn DE`
if [ "$check" = "FALSE" ] ; then
	exit
fi

PWORD=PqLXAdmiN
execCMD="${mysqlexec} --host=$HOST --database=$DBn --user=pqlxAdmin --password=$PWORD --compress"
outFile=$PQLXLOG/ES-$DBn-`date +%Y.%j.%H%M`.log

# output the info
echo "Channels Missing A Response File:" >$outFile
sql="select distinct(concat(ntw, '.', stn, '.', loc, '.', chn)) as channel from nslc n, EVENTQ e, TRC t where e.error_no = 1001 and t.trc_pk = e.typeKey and n.chni_pk = t.chni_fk order by 1;"
echo $sql | ${execCMD} | grep -v channel >>$outFile 

echo >>$outFile
echo "Channels with a Sample Rate Change:" >>$outFile
echo "(columns: Sample Interval	Start Date	End Date)" >>$outFile
sql="select distinct(chni_pk) as channel from nslc n, EVENTQ e where e.error_no = 1009 and n.chni_pk = e.typeKey order by 1;"
chans=`echo $sql | ${execCMD}  | grep -v channel`
for chan in $chans ; do
	sql="select distinct(concat(ntw, '.', stn, '.', loc, '.', chn)) as channel from nslc where chni_pk=$chan"
	echo $sql | ${execCMD} | grep -v channel >>$outFile
	sql="select sampInt, DATE(start), DATE(end) from TRC where chni_fk=$chan and sampInt <> 0 order by 2;"
	samps=( `echo $sql | ${execCMD} | grep -v sampInt` )
	count=${#samps[@]}
	sampInt=${samps[0]}
	start=${samps[1]}
	stop=${samps[2]}
	echo -n "$sampInt - $start " >>$outFile
	lastSamp=$sampInt
	lastStop=$stop
	index=3
	while [ "$index" -lt "$count" ] ; do
		sampInt=${samps[$index]}
		start=${samps[$index+1]}
		stop=${samps[$index+2]}
		diff=`expr "$sampInt" = "$lastSamp"`
		if [ "$diff" -eq "0" ] ; then
			echo $lastStop >>$outFile
			echo -n "$sampInt - $start " >>$outFile
			lastSamp=$sampInt
		fi
		lastStop=$stop
		let "index = $index + 3"
	done
	echo $stop >>$outFile
	echo >>$outFile
done

# following code removed since trace no longer exist in the TRC table
# after server runs
#echo >>$outFile
#echo "File Larger than 30Mb and Unprocessed:" >>$outFile
#sql="select distinct(concat(directory, '/', filename)) as file from EVENTQ e, TRC t where e.error_no = 1010 and t.trc_pk = e.typeKey  order by 1;"
#files=`echo $sql | ${execCMD} | grep -v file`

#for file in $files ; do
#	ls -sh $file >>$outFile
#done

#echo >>$outFile
#echo "Files causing the server to crash:" >>$outFile
#sql="select c.description, concat(directory, '/', filename) as file from CONSTANT c, nslc n, EVENTQ e, TRC t where e.error_no = 102 and n.chni_pk = t.chni_fk and t.trc_pk = e.typeKey and e.type = c.val and c.type = 'EVENT' order by 1;"
#echo $sql | ${execCMD} | grep -v file >>$outFile

echo >>$outFile
echo "Channels having an evalresp() error:" >>$outFile
sql="select distinct(concat(ntw, '.', stn, '.', loc, '.', chn)) as channel from nslc n, EVENTQ e, TRC t where e.error_no = 1008 and n.chni_pk = t.chni_fk and t.trc_pk = e.typeKey order by 1;"
echo $sql | ${execCMD} | grep -v channel >>$outFile

echo $outFile
