#!/bin/bash

if [ "$1" = "" ] ; then
	echo "usage: $0 [HOST:]DBName"
	echo
	echo "Repair all tables marked as crashed, as listed in .err files for given database"
	echo "N.B. Can only be executed on localhost of DB server"
	echo "Output written to \$PQLXLOG/pqlx-RepairTables-YYYY.JJJ.HH.MM.log"
	exit
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

HOST=localhost
DBn=`echo $1 | sed 's/.*://'`

if [ "$HOST" = "$DBn" ] ; then
	HOST=localhost
fi

# check that the database exists
check=`$PQLXBIN/_checkDB $HOST $DBn DE`
if [ "$check" = "FALSE" ] ; then
	exit
fi

PWORD=PqLXAdmiN
execCMD="${mysqlexec} --host=$HOST --user=pqlxAdmin --password=$PWORD --compress"
outFile=$PQLXLOG/pqlx-RepairTable-`date +%Y.%j.%H.%M`.log

OIFS=$IFS
tables=`grep 'Error: Table' $PQLXLOG/srvr.${DBn}*.err | awk '{print $6}' | sed -e 's@^../@@' | sed -e 's@.\$@@'`
tables=`echo $tables | tr " " "\n" | sort -u`
for table in $tables ; do
	echo "Repairing Table $table..."
	echo -n `date +%H:%M:` >>$outFile
	echo "	Repairing Table $table..." >>$outFile
	IFS='/'
	tableN=( $table )
	IFS=$OIFS
	sql="repair table ${tableN[1]}"
	echo $sql | ${execCMD} --database=${tableN[0]} >>$outFile
	echo "" >>$outFile
done
IFS=$OIFS
