#!/bin/bash

# some constants
up=UP
down=DOWN
T=TRUE
F=FALSE

# define all possible argument values
argLIST=LIST
argDEL=DELETE
argEN=ENABLE
argDIS=DISABLE
argIMP=IMPORT
argEXP=EXPORT
argNONE=NONE

function usage {
	echo "Usage: "
	echo "  Process a Base-Line Request:"
	echo "  $0 [HOST:]DBName $argLIST"
	echo	 "    - List All Baselines Defined in DB"
	echo "  $0 [HOST:]DBName Baseline-Name [ $argDEL | [ $argEN | $argDIS ] ]"
	echo	 "    - DELETE Specified Baseline from DB"
	echo	 "    - ENABLE | DISABLE Specified Baseline for Outlier Computations"
	echo "  $0 [HOST:]DBName $argIMP (BLine.def File)"
	echo	 "    - IMPORT (BLine.def File) - Import Baseline, defined in file BLine.def:"
	echo "          - see \$PQLXBIN/BLine-sample.def for Example Definition File"
	echo "  $0 [HOST:]DBName $argEXP Baseline-Name"
	echo	 "    - EXPORT - Export Baseline Definition for Baseline-Name"
	exit
}

if [ $# -eq "0" ] ; then
	usage
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

HOST=`echo $1 | sed 's/:.*//'`
DBn=`echo $1 | sed 's/.*://'`
if [ "$HOST" = "$DBn" ] ; then
	HOST=localhost
fi
# check that the database exists
check=`$PQLXBIN/_checkDB $HOST $DBn DE`
if [ "$check" == "FALSE"  ] ; then
	exit
fi

USERN=pqlxAdmin
PWORD=PqLXAdmiN
execCMD="${mysqlexec} --host=$HOST --database=$DBn --user=$USERN --password=$PWORD --compress"

# shift arguments by one
shift

# process command line args
cmd=$argNONE
check=$F
if [ $# -gt "0" ] ; then
  for arg in "$@" ; do
    case "$arg" in
    "$argLIST" | "$argDEL" | "$argEN" | "$argDIS" )
    		cmd=$arg
    		if [ "$arg" != "$argLIST" ] ; then
    			check=$T
    		fi
    	;;
    "$argIMP" )
    		cmd=$arg
    		fname=$2
    		break
    	;;
    "$argEXP" )
    		cmd=$arg
		check=$T
    	;;
    * )
    		bline=$arg
    	;;
    esac
  done
fi
if [ "$cmd" = "$argNONE" ] ; then
	usage
fi

# confirm the baseline exists, for all commands except LIST & IMPORT, $check set above
if [ "$check" = "$T" ] ; then
	sqlStr="SELECT bline_pk FROM BLine WHERE chni_fk = 0 and name = '$bline'"
	bline_fk=`echo $sqlStr | ${execCMD} | grep -v pk`
	if [ "$bline_fk" = "" ] ; then
		echo "No Baseline with Name '$bline' Found, Cannot Execute '$cmd' Command, quitting..."
		exit
	fi
fi

dbSTATS=${DBn}STATS
case "$cmd" in
	"$argLIST" )
		# LIST all Baselines
		sqlStr="SELECT name, descr, dir, active FROM BLine WHERE chni_fk = 0 ORDER BY 1"
		echo $sqlStr | ${execCMD}
	;;
	"$argEN" )
		# En-Able specified Baseline
		echo "En-abling Outlier Computations for Baseline '$bline'"
		sqlStr="UPDATE BLine SET active = 'Y' WHERE bline_pk = $bline_fk"
		echo $sqlStr | ${execCMD}
	;;
	"$argDIS" )
		# Dis-Able specified Baseline
		echo "Dis-abling Outlier Computations for Baseline '$bline'"
		sqlStr="UPDATE BLine SET active = 'N' WHERE bline_pk = $bline_fk"
		echo $sqlStr | ${execCMD}
	;;
	"$argDEL" )
		# Delete specified baseline, all instances
		echo "Deleting Baseline '$bline' from Database"

		# delete the interpolated versions, if there are any
		sqlStr="DELETE FROM \`${dbSTATS}\`.\`BLineID\` WHERE blinei_fk IN "
		sqlStr=$sqlStr"(SELECT blinei_pk FROM \`${dbSTATS}\`.\`BLineI\` WHERE bline_fk = $bline_fk)"
		echo $sqlStr | ${execCMD}
		sqlStr="DELETE FROM \`${dbSTATS}\`.\`BLineI\` WHERE bline_fk = $bline_fk"
		echo $sqlStr | ${execCMD}

		# delete all comparison results, for all possible channels
		sqlStr="SELECT chni_pk FROM  \`${DBn}\`.\`CHNI\` c, ${DBn}.PSDCFG p "
		sqlStr=$sqlStr"WHERE c.chni_pk = p.chni_fk AND c.ttlPSD > 0"
		chniFKs=`echo $sqlStr | ${execCMD}| grep -v chni_pk`
		for chniFK in $chniFKs ; do
			tblName="\`BLineS${chniFK}\`"
			sqlStr="DELETE FROM \`${dbSTATS}\`.$tblName WHERE bline_fk = $bline_fk"
			echo $sqlStr | ${execCMD}
		done

		# delete the main versions
		sqlStr="DELETE FROM \`${dbSTATS}\`.\`BLineD\` WHERE bline_fk = $bline_fk"
		echo $sqlStr | ${execCMD}
		sqlStr="DELETE FROM BLine WHERE bline_pk = $bline_fk"
		echo $sqlStr | ${execCMD}
	;;
	"$argEXP" )
		sqlStr="SELECT period, power FROM \`${dbSTATS}\`.\`BLineD\` WHERE bline_fk = $bline_fk "
		sqlStr=$sqlStr"ORDER BY 1"
		echo $sqlStr | ${execCMD} | grep -v period
	;;
	"$argIMP" )
		# import new baseline
		if [ ! -e $fname ] ; then
			echo "File '$fname' not found, cannot import"
			exit
		fi
		nameSTR="Name:"
		name=`grep "$nameSTR" $fname | sed s/$nameSTR//`
		descrSTR="Description:"
		descr=`grep "$descrSTR" $fname | sed s/$descrSTR//`
		dirSTR="Direction:"
		dir=`grep "$dirSTR" $fname | sed s/$dirSTR//`
		fileSTR="File:"
		filePoints=`grep "$fileSTR" $fname | sed s/$fileSTR//`
		clrSTR="R:"
		R=`grep "$clrSTR" $fname | sed s/$clrSTR//`
		clrSTR="G:"
		G=`grep "$clrSTR" $fname | sed s/$clrSTR//`
		clrSTR="B:"
		B=`grep "$clrSTR" $fname | sed s/$clrSTR//`

		echo "Importing new Baseline '$name' into Database"
		# verify input
		if [ "$dir" != "$up" -a \
			 "$dir" != "$down" ] ; then
			 	 echo "Direction value must be one of '$up' or '$down' (without quotes), Cannot Proceed with Input"
			 	 exit
		fi
		
		# insert BLine meta-definition
		sqlStr="INSERT INTO BLine (name, descr, dir, active, chni_fk) "
		sqlStr=$sqlStr"VALUES ('$name', '$descr', '$dir', 'Y', 0)"
		echo $sqlStr | ${execCMD}

		# retrieve primary key
		sqlStr="SELECT bline_pk FROM BLine WHERE chni_fk = 0 and name = '$name'"
		bline_fk=`echo $sqlStr | ${execCMD} | grep -v pk`
		dbSTATS=${DBn}STATS

		# read and import baseline definition
		while read per pow ; do
			sqlStr="INSERT INTO \`${dbSTATS}\`.\`BLineD\` (bline_fk, period, power) " 
			sqlStr=$sqlStr"VALUES ('$bline_fk', '$per', '$pow')"
			echo $sqlStr | ${execCMD}
		done <$filePoints
	;;
esac

