#!/bin/bash

T=TRUE
F=FALSE
B=BEFORE
A=AGO
N=NOW

if [ "$1" = ""  -o "$2" = "" -o "$3" = "" ] ||\
   [ "$2" != "$B" -a "$2" != "$A" ] ; then
	echo
	echo "usage: $0 [HOST:]DBName [ [ BEFORE date ] | [ AGO days ] ] NumCPU's ( NOW )"
	echo "	where:"
	echo "		BEFORE date - Date before which all data will be deleted"
	echo "		AGO days - Number of Days of Data to be Retained"
	echo "		NumCPU's - Number of CPU's to Use for Server Execution"
  echo "		NOW - optional - Execute PSD Deletion Directly, Otherwise Deferred until Server Execution"
  echo "Examples:"
  echo "  $0 micros BEFORE 2003-01-01 3 NOW"
  echo "    - Immediately Delete ALL PSD's existing BEFORE 1-Jan-2003, Using 3 CPU's for Server Execution"
  echo 
  echo "  $0 micros AGO 60 4"
  echo "    - Delete all PSD's Older than 60 Days AGO, Using 4 CPU's for Server Execution"
  echo
	exit
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

HOST=`echo $1 | sed 's/:.*//'`
DBn=`echo $1 | sed 's/.*://'`
if [ "$HOST" = "$DBn" ] ; then
	HOST=localhost
fi
# check that the database exists
check=`$PQLXBIN/_checkDB $HOST $DBn DE`
if [ "$check" = "FALSE" ] ; then
	exit
fi

PWORD=PqLXAdmiN
execCMD="${mysqlexec} --host=$HOST --database=$DBn --user=pqlxAdmin --password=$PWORD --compress"

case "$2" in
  "$A" )
        numDays=$3
        dateAGO=`echo "SELECT DATE_SUB(CURDATE(), INTERVAL $numDays DAY) dateAGO" | ${execCMD} | grep -v dateAGO`
  ;;
  "$B" )
        dateAGO=$3
  ;;
esac
numCPUs=$4
now=$F
if [ $# -eq 5 -a "$5" = "$N" ] ; then
  now=$T
fi

echo "Deleting ALL PSD DATA Older than ${dateAGO}:"

# get the channel instance numbers
chniFKs=`echo "SELECT chni_pk FROM  ${DBn}.CHNI c, ${DBn}.PSDCFG p WHERE c.chni_pk = p.chni_fk AND c.ttlPSD > 0" | ${execCMD}| grep -v chni_pk`

for chniFK in $chniFKs ; do
    PSDidx=\`${DBn}DATA\`.\`psd${chniFK}idx\`
    PSDpsd=\`${DBn}DATA\`.\`psd${chniFK}\`

    echo "    Processing Channel Key $chniFK"
    # if (NOW) then delete the PSDs from the DATA table for given channel
    # and optimize the tables immediately to recover the disk space
    if [ "$now" = "$T" ] ; then
      sql="DELETE $PSDpsd, $PSDidx FROM $PSDpsd, $PSDidx WHERE $PSDpsd.psd_fk = $PSDidx.psd_pk AND $PSDidx.day < '$dateAGO'"
      echo $sql | ${execCMD}
      sql="OPTIMIZE TABLE $PSDpsd"
      echo $sql | ${execCMD} >/dev/null
      sql="OPTIMIZE TABLE $PSDidx"
      echo $sql | ${execCMD} >/dev/null
    fi

    # delete TRC records
    sql="DELETE FROM TRC WHERE chni_fk = ${chniFK} AND end < '$dateAGO' AND format <> 'DUMMY'"
    echo $sql | ${execCMD}

    # insert Sample Rate Epoch Event
    sql="INSERT INTO EVENTQ (type, typeKey, processed, chni_fk) SELECT 93, $chniFK, 'NOPROC', $chniFK"
    echo $sql | ${execCMD}

    # insert Channel Statistics Event
    sql="INSERT INTO EVENTQ (type, typeKey, processed, chni_fk) SELECT 97, $chniFK, 'NOPROC', $chniFK"
    echo $sql | ${execCMD}

    # insert System PDF Event
    sql="INSERT INTO EVENTQ (type, typeKey, processed, chni_fk) SELECT 100, $chniFK, 'NOPROC', $chniFK"
    echo $sql | ${execCMD}

    # insert System Spectrogram Event
#    sql="INSERT INTO EVENTQ (type, typeKey, processed, chni_fk) SELECT 102, $chniFK, 'NOPROC', $chniFK"
#    echo $sql | ${execCMD}

    # insert Baseline Comparison Event
#    sql="INSERT INTO EVENTQ (type, typeKey, processed, chni_fk) SELECT 110, $chniFK, 'NOPROC', $chniFK"
#    echo $sql | ${execCMD}
done

# and start a single instance of the server
echo "Executing the Server to Perform Clean-up - Directory Scanning Disabled"
$PQLXBIN/pqlxSrvr --dbName=$1 --numCPU=$numCPUs --noScan

echo "Processing Complete"
exit
