#!/bin/bash

if [ "$1" != "YES" ] ; then
	echo "Usage: $0 YES"
	echo "	Re-Initializes the PQLX Database Environment"
	echo "	WARNING: This deletes all PQLX databases from the server!"
	echo "	(can only be executed on localhost)"
	exit
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

# script re-initializes the mySQL server with our environment

PWORD=PqLXAdmiN
execCMD="${mysqlexec} --user=pqlxAdmin --password=$PWORD"

# first check to see that our environment already exists
check=`echo "show databases" | ${execCMD} | grep PQLXMETA | wc -l`
if [ $check -ne 1 ] ; then
	echo 
	echo "But the system hasn't yet been initialized, Can't Re-Initialize..."
	echo "Try executing initPQLXdb.sh instead ..."
	echo "Quitting without doing anything... Goodbye"
	exit
fi

databases=`echo "select dbName from PQLXMETA.PQLXDBS" | ${execCMD} | grep -v PQLXMETA | grep -v dbName`

echo "This action will delete *ALL* of the following PQLX databases and will re-initialize the PQLX database environment to a completely empty state..."
for db in $databases ; do
	echo $db
done

echo
echo "Are you absolutely sure you want to do this? (Y | N)"
read ans
if [ "$ans" != "Y" ] ; then
	echo "System NOT re-initialized, Quitting without doing anything... Goodbye"
	exit
fi

# our users have been previously set up, no need to do it again
#${mysqlexec} --user=root <${PQLXPROD}/dbdef/sql-sup/initPQLXdb.sql

# drop all databases defined in PQLXMETA
for db in $databases ; do
	echo "Dropping database $db"
	echo "DROP database ${db}" | ${execCMD}
	echo "DROP database ${db}DATA" | ${execCMD}
	echo "DROP database ${db}STATS" | ${execCMD}
	echo "DROP database ${db}TMP" | ${execCMD}
done
echo "All Databases Deleted"

echo "Recreating Database PQLXMETA"
# and drop the PQLXMETA database itself
echo "DROP database PQLXMETA" | ${execCMD}

# then re-create the PQLXMETA data table
${execCMD} <${PQLXDBDEF}/sql-sup/makePQLXMETAdb.sql
