#!/bin/bash

cmdLIST=LIST
cmdDEL=DELETE

if [ $# -ne 8 -a $# -ne 9 ] ; then
	echo "usage: $0 [HOST:]DBName NTW STN LOC CHN Start_DATE End_DATE ( $cmdLIST | ($cmdDEL SI) )"
	echo "example:	$0 micros IU ANMO -- BHE"
	echo "	$cmdLIST - returns the distinct sample rates for given channel over specified date range"
	echo "	$cmdDEL SI - deletes all TRC files for given date range and given sample interval (SI)"
  echo "    SI = Value as returned by $cmdLIST command"
  echo
	exit
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

HOST=`echo $1 | sed 's/:.*//'`
DBn=`echo $1 | sed 's/.*://'`
if [ "$HOST" = "$DBn" ] ; then
	HOST=localhost
fi

NTWi=$2
STNi=$3
LOCi=$4
CHNi=$5
SDate=$6
EDate=$7
cmd=$8

# check that the database exists
check=`$PQLXBIN/_checkDB $HOST $DBn DE`
if [ "$check" = "FALSE" ] ; then
	exit
fi

PWORD=PqLXClnT
execCMD="${mysqlexec} --host=$HOST --database=$DBn --user=pqlxClnt --password=$PWORD --compress"

chniFK=`echo "SELECT chni_pk FROM  ${DBn}.nslc WHERE ntw = '$NTWi' and stn = '$STNi' and loc = '$LOCi' and chn = '$CHNi'" | ${execCMD} | grep -v chni_pk`
if [ "$chniFK" = "" ] ; then
	echo "No channel instance found for $NTWi.$STNi.$LOCi.$CHNi, please try again..."
	echo "Quitting without doing anything... Goodbye"
	exit
fi

case $cmd in
  "$cmdLIST"  )
      echo
      echo "Known Sample Rates for Channel  ($NTWi $STNi $LOCi $CHNi):"
      echo "('Sample Interval'	'Start'	'End')"

	    sql="SELECT sampInt, DATE(start), DATE(end) FROM TRC WHERE chni_fk=$chniFK AND sampInt <> 0 AND DATE(start) >= '$SDate' AND DATE(end) <= '$EDate' ORDER BY 2"
	    samps=( `echo $sql | ${execCMD} | grep -v sampInt` )
	    count=${#samps[@]}
	    sampInt=${samps[0]}
	    start=${samps[1]}
	    stop=${samps[2]}
	    echo -n "$sampInt - $start "
	    lastSamp=$sampInt
	    lastStop=$stop
	    index=3
	    while [ "$index" -lt "$count" ] ; do
		    sampInt=${samps[$index]}
		    start=${samps[$index+1]}
		    stop=${samps[$index+2]}
		    diff=`expr "$sampInt" = "$lastSamp"`
		    if [ "$diff" -eq "0" ] ; then
			    echo $lastStop
			    echo -n "$sampInt - $start "
			    lastSamp=$sampInt
		    fi
		    lastStop=$stop
		    let "index = $index + 3"
	    done
	    echo $stop
    ;;

  "$cmdDEL" )
      if [ $# -ne 9 ] ; then
	      echo "usage: $0 [HOST:]DBName NTW STN LOC CHN Start_DATE End_DATE ( $cmdLIST | ($cmdDEL SI) )"
        echo
        exit
      fi
      SI=$9
      sql="SELECT COUNT(*) cnt FROM TRC where chni_fk=$chniFK and sampInt = $SI and DATE(start) >= '$SDate' and DATE(end) <= '$EDate'"
      cnt=`echo $sql | ${execCMD} | grep -v cnt`
      sql="DELETE FROM TRC where chni_fk=$chniFK and sampInt = $SI and DATE(start) >= '$SDate' and DATE(end) <= '$EDate'"
      echo $sql | ${execCMD}
      echo "$cnt Trace Records deleted from database"
      echo "You Must re-Run the PQLX Server to Delete ALL Corresponding Subordinate Data"
    ;;

  * )
	      echo "usage: $0 [HOST:]DBName NTW STN LOC CHN Start_DATE End_DATE ( $cmdLIST | ($cmdDEL SI) )"
        echo
        exit
    ;;
esac

echo
exit
