#!/bin/bash

db=$1

function errorOut {
	echo "Adjust entry in file $PQLXDBDEF/$db.pqlx and resubmit..."
	echo "Exiting without doing anything, goodbye..."
	exit
}

error=F

if [ "$1" = "" ] ; then
	echo "usage:	$0 dbName"
	echo "	Update Database information contained in database definition file ${PQLXDBDEF}/dbName.pqlx"
	echo "	(can only be executed on localhost)"
	exit
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

# set up the MySQL password
PWORD=PqLXAdmiN
execCMD="${mysqlexec} --user=pqlxAdmin --password=$PWORD"

echo "Updating PQLX Database $1 defined in PQLX DB definition file $1.pqlx..."

DBDEFFILE=${PQLXDBDEF}/$1.pqlx

if [ ! -e ${DBDEFFILE} ] ; then
	echo "PQLX Database Definition File not found (=${DBDEFFILE}), cannot continue..."
	exit
fi

DBName=$1
DBOrg=`grep DBOrg ${DBDEFFILE} | sed s/DBOrg://`
DBDescr=`grep DBDescr ${DBDEFFILE} | sed s/DBDescr://`
DBTechAdmin=`grep DBTechAdmin ${DBDEFFILE} | sed s/DBTechAdmin://`
DBTechEmail=`grep DBTechEmail ${DBDEFFILE} | sed s/DBTechEmail://`
DBDataAdmin=`grep DBDataAdmin ${DBDEFFILE} | sed s/DBDataAdmin://`
DBDataEmail=`grep DBDataEmail ${DBDEFFILE} | sed s/DBDataEmail://`
DBType=`grep DBType ${DBDEFFILE} | sed s/DBType://`
DBAccess=`grep DBAccess ${DBDEFFILE} | sed s/DBAccess://`
DBAdminPWord=`grep DBAdminPWord ${DBDEFFILE} | sed s/DBAdminPWord://`
DBWWWDir=`grep DBWWWDir ${DBDEFFILE} | sed s/DBWWWDir://`

# get the PDF statistics settings
DBPNGMIN=`grep DBPNGMIN ${DBDEFFILE} | sed s/DBPNGMIN://`
if [ "$DBPNGMIN" = "" ] ; then
	echo "$1.pqlx file must be updated with PDF Statistics plot settings... doing this now"
	cat $DBDEFFILE $PQLXDBDEF/_PDFstats >$DBDEFFILE.tmp
	rm $DBDEFFILE
	mv $DBDEFFILE.tmp $DBDEFFILE
	echo "File has been updated, default values beingused."
	echo "Please verify or modify settings as desired - see section 'PDF Statistics'"
	DBPNGMIN=`grep DBPNGMIN ${DBDEFFILE} | sed s/DBPNGMIN://`
fi
if [ "$DBPNGMIN" = "OFF" ] ; then
	DBPNGMIN=0
elif [ "$DBPNGMIN" = "ON" ] ; then
	DBPNGMIN=1
else
	DBPNGMIN=""
fi
DBPNGMAX=`grep DBPNGMAX ${DBDEFFILE} | sed s/DBPNGMAX://`
if [ "$DBPNGMAX" = "OFF" ] ; then
	DBPNGMAX=0
else
	DBPNGMAX=1
fi
DBPNGMODE=`grep DBPNGMODE ${DBDEFFILE} | sed s/DBPNGMODE://`
if [ "$DBPNGMODE" = "OFF" ] ; then
	DBPNGMODE=0
else
	DBPNGMODE=1
fi
DBPNGMEDIAN=`grep DBPNGMEDIAN ${DBDEFFILE} | sed s/DBPNGMEDIAN://`
if [ "$DBPNGMEDIAN" = "OFF" ] ; then
	DBPNGMEDIAN=0
else
	DBPNGMEDIAN=1
fi
DBPNGMEAN=`grep DBPNGMEAN ${DBDEFFILE} | sed s/DBPNGMEAN://`
if [ "$DBPNGMEAN" = "OFF" ] ; then
	DBPNGMEAN=0
else
	DBPNGMEAN=1
fi
DBPNG5P=`grep DBPNG5P ${DBDEFFILE} | sed s/DBPNG5P://`
if [ "$DBPNG5P" = "OFF" ] ; then
	DBPNG5P=0
else
	DBPNG5P=1
fi
DBPNG10P=`grep DBPNG10P ${DBDEFFILE} | sed s/DBPNG10P://`
if [ "$DBPNG10P" = "OFF" ] ; then
	DBPNG10P=0
else
	DBPNG10P=1
fi
DBPNG25P=`grep DBPNG25P ${DBDEFFILE} | sed s/DBPNG25P://`
if [ "$DBPNG25P" = "OFF" ] ; then
	DBPNG25P=0
else
	DBPNG25P=1
fi
DBPNG75P=`grep DBPNG75P ${DBDEFFILE} | sed s/DBPNG75P://`
if [ "$DBPNG75P" = "OFF" ] ; then
	DBPNG75P=0
else
	DBPNG75P=1
fi
DBPNG90P=`grep DBPNG90P ${DBDEFFILE} | sed s/DBPNG90P://`
if [ "$DBPNG90P" = "OFF" ] ; then
	DBPNG90P=0
else
	DBPNG90P=1
fi
DBPNG95P=`grep DBPNG95P ${DBDEFFILE} | sed s/DBPNG95P://`
if [ "$DBPNG95P" = "OFF" ] ; then
	DBPNG95P=0
else
	DBPNG95P=1
fi

# get the PNG plot settings
PDFW=`grep PDFW ${DBDEFFILE} | sed s/PDFW://`
if [ "$PDFW" = "" ] ; then
	echo "$1.pqlx file must be updated with PDF PNG plot settings... doing this now"
	cat $DBDEFFILE $PQLXDBDEF/_PNGsettings >$DBDEFFILE.tmp
	rm $DBDEFFILE
	mv $DBDEFFILE.tmp $DBDEFFILE
	echo "File has been updated, default values being used."
	echo "Please verify or modify settings as desired - see section 'PDF PNG Settings'"
	PDFW=`grep PDFW ${DBDEFFILE} | sed s/PDFW://`
fi
if [ "$PDFW" -lt "240" ] ; then
	echo "Normal PNG width (PDFW) must be >= 240"
	error=T
fi
PDFH=`grep PDFH ${DBDEFFILE} | sed s/PDFH://`
PDFBRDR=`grep PDFBRDR ${DBDEFFILE} | sed s/PDFBRDR://`
if [ "$PDFBRDR" != "OFF" -a \
	"$PDFBRDR" != "ON" ] ; then
	echo "PNG Border Setting must be 'ON' or 'OFF' (PDFBRDR)"
	error=T
fi
PDFR=`grep PDFR ${DBDEFFILE} | sed s/PDFR://`
PDFG=`grep PDFG ${DBDEFFILE} | sed s/PDFG://`
PDFB=`grep PDFB: ${DBDEFFILE} | sed s/PDFB://`
PDFIW=`grep PDFIW ${DBDEFFILE} | sed s/PDFIW://`
if [ "$PDFIW" -ge "240" ] ; then
	echo "Icon PNG width (PDFIW) must be < 240"
	error=T
fi
PDFIH=`grep PDFIH ${DBDEFFILE} | sed s/PDFIH://`
PDFIBRDR=`grep PDFIBRDR ${DBDEFFILE} | sed s/PDFIBRDR://`
if [ "$PDFIBRDR" != "OFF" -a \
	"$PDFIBRDR" != "ON" ] ; then
	echo "PNG Border Setting must be 'ON' or 'OFF' (PDFIBRDR)"
	error=T
fi
PDFIR=`grep PDFIR ${DBDEFFILE} | sed s/PDFIR://`
PDFIG=`grep PDFIG ${DBDEFFILE} | sed s/PDFIG://`
PDFIB=`grep PDFIB: ${DBDEFFILE} | sed s/PDFIB://`
PDFNM=`grep PDFNM: ${DBDEFFILE} | sed s/PDFNM://`
if [ "$PDFNM" != "OFF" -a \
	"$PDFNM" != "ON" ] ; then
	echo "PNG Plot Noise Model Setting must be 'ON' or 'OFF' (PDFNM)"
	error=T
fi

DBMaker=`whoami`
DBSQLFILE=${PQLXDBDEF}/sql-db/${DBName}.sql

# check that the database exists
check=`$PQLXBIN/_checkDB localhost $DBName DE`
if [ "$check" == "FALSE"  ] ; then
	exit
fi

if [ "$DBOrg" = "" ] ; then
	echo "DBOrg field is empty, adjust entry in file $1.pqlx and resubmit..."
	error=T
fi

if [ "$DBDescr" = "" ] ; then
	echo "DBDescr field is empty, adjust entry in file $1.pqlx and resubmit..."
	error=T
fi

if [ "$DBTechAdmin" = "" ] ; then
	echo "DBTechAdmin field is empty, adjust entry in file $1.pqlx and resubmit..."
	error=T
fi

if [ "$DBTechEmail" = "" ] ; then
	echo "DBTechEmail field is empty, adjust entry in file $1.pqlx and resubmit..."
	error=T
fi

if [ "$DBDataAdmin" = "" ] ; then
	echo "DBDataAdmin field is empty, adjust entry in file $1.pqlx and resubmit..."
	error=T
fi

if [ "$DBDataEmail" = "" ] ; then
	echo "DBDataEmail field is empty, adjust entry in file $1.pqlx and resubmit..."
	error=T
fi

if [ "$DBType" = "" ] ; then
	echo "DBType field is empty, adjust entry in file $1.pqlx and resubmit..."
	error=T
fi

if [ "$DBType" != "PRODUCTION" -a "$DBType" != "SCRATCH" ] ; then
	echo "DBType field must be one of PRODUCTION or SCRATCH, adjust entry in file $1.pqlx and resubmit..."
	error=T
fi

if [ "$DBAccess" = "" ] ; then
	echo "DBAccess field is empty, adjust entry in file $1.pqlx and resubmit..."
	error=T
fi

if [ "$DBAccess" != "PUBLIC" -a "$DBAccess" != "PRIVATE" ] ; then
	echo "DBAccess field must be one of PUBLIC or PRIVATE, adjust entry in file $1.pqlx and resubmit..."
	error=T
fi

if [ "$DBAdminPWord" = "" ] ; then
	echo "DBAdminPWord field is empty, please provide a System Admin Password in file $1.pqlx and resubmit..."
	error=T
fi

if [ "$DBWWWDir" = "" ] ; then
	echo "DBWWWDir field is empty, please provide a valid directory or NONE in file $1.pqlx and resubmit..."
	error=T
fi

if [ "$error" = "T" ] ; then
	errorOut
fi

# update the database info
sqlCmd="UPDATE PQLXMETA.PQLXDBS SET organisation='$DBOrg', description='$DBDescr', techAdminName='$DBTechAdmin', techAdminEmail='$DBTechEmail', dataAdminName='$DBDataAdmin', dataAdminEmail='$DBDataEmail', type='$DBType', access='$DBAccess', adminPWord='$DBAdminPWord', wwwDir='$DBWWWDir' where dbName = '$DBName'"
echo $sqlCmd | ${execCMD}

# update the PNG Plot Statistics for output
if [ "$DBPNGMIN" != "" ] ; then
	sqlCmd="UPDATE ${DBName}.PDFCFG SET min='$DBPNGMIN', max='$DBPNGMAX', mode='$DBPNGMODE', median='$DBPNGMEDIAN', mean='$DBPNGMEAN', p5='$DBPNG5P', p10='$DBPNG10P', p25='$DBPNG25P', p75='$DBPNG75P', p90='$DBPNG90P', p95='$DBPNG95P'"
	echo $sqlCmd | ${execCMD}
fi

# update the PNG plot settings
sqlCmd="UPDATE ${DBName}.PNGWWW SET wwwDir = '$DBWWWDir', "\
"width = $PDFW, height = $PDFH, border = '$PDFBRDR', bordercolorR=$PDFR, bordercolorG=$PDFG, "\
"bordercolorB=$PDFB, Iwidth=$PDFIW, Iheight=$PDFIH, Iborder='$PDFIBRDR', IborderColorR=$PDFIR, "\
"IborderColorG=$PDFIG, IborderColorB=$PDFIB "\
"WHERE type = 101"
echo $sqlCmd | ${execCMD}

echo "Information for PQLX Database '$DBName' succesfully updated..."
