#include <mysql.h>
#include <sys/time.h>
#include <stdlib.h>
#include <unistd.h>
#include "pqlx_defs.h"
#include "dbmysqldefs.h"
#include "db_sql.h"
#define POLLTIME (.5*1000000) 
static gboolean paused;
static gboolean finished;
static int numThreads;
static GQueue *eventsQ;
static GStaticMutex QMutex = G_STATIC_MUTEX_INIT;  
static int (*_initFunc)(MYSQL *, int);
static int (*_procFunc)(MYSQL *, THREADARGS *);
static int (*_doneFunc)(MYSQL *, int);
typedef struct _EVENT
{
int id;
int typeKey;
} EVENT;
typedef struct _ARGS
{
int  threadNum;
} ARGS;
static gboolean _checkForPause(MYSQL *conn)
{ 
if (paused)
return paused;
{
SQLDI sqldi;
int  retCode;
memset(&sqldi, 0, sizeof(SQLDI));
sqldi.descr.tableData = EVENT_GET_PAUSE;
retCode = _db_event(conn, &sqldi);
retCode = (retCode != PQLXSRVRPAUSE) ? PQLXSUCCESS : PQLXSRVRPAUSE;
if (retCode == PQLXSRVRPAUSE)
paused = TRUE;
}
return paused; 
} 
static void _finish()
{
mysql_thread_end();
numThreads--;
if (!numThreads)
{
finished = TRUE;
}
}
static EVENT *getEvent(MYSQL *conn)
{
EVENT *event;
g_static_mutex_lock(&QMutex);
if (_checkForPause(conn))
event = NULL;
else
event = g_queue_pop_head(eventsQ);
g_static_mutex_unlock(&QMutex);
return event;
}
static void _threadProc(ARGS *args)
{
int  threadNum = args->threadNum, ret;
SQLDI sqldi;
MYSQL *conn;
EVENT *event;
THREADARGS threadArgs;
free(args);
mysql_thread_init();
memset(&sqldi, 0, sizeof(SQLDI));
sqldi.descr.tableData = NONE;
sqldi.args.sqr = threadNum;
conn = mngDBConn(&sqldi, GETDBCONN);
if (conn == NULL)
{
fprintf(stderr, "Unable to retrieve a DB connection, cannot process these events\n");
_finish();
return;
}  
if (_initFunc)
{
ret = (*_initFunc)(conn, threadNum);
if (ret != PQLXSUCCESS)
{
fprintf(stderr, "Unable to execute thread Initialize function, cannot process these events.\n");
_finish();
return;
}
}
event = getEvent(conn);
while (event)
{
sqldi.descr.tableData = EVENT_INPROC;
sqldi.event.id = event->id;
_db_event(conn, &sqldi);
threadArgs.evtq_id = event->id;
threadArgs.typeKey = event->typeKey;
threadArgs.threadNum = threadNum;
ret = (*_procFunc)(conn, &threadArgs);
sqldi.descr.tableData = EVENT_DEL;
_db_event(conn, &sqldi);
event = getEvent(conn);
}
if (_doneFunc)
{
ret = (*_doneFunc)(conn, threadNum);
if (ret != PQLXSUCCESS)
{
fprintf(stderr, "WARNING: Unable to execute thread Finalize function, possible problem.\n");
_finish();
return;
}
}
_finish();
}
int procThread(MYSQL *conn, int maxCPU, int eventType, 
void *initCall, 
void *procCall,
void *doneCall)
{
int   i, ret; 
char  *sqlStr;
MYSQL_RES *res_set=NULL;
MYSQL_ROW row;
EVENT  *event;
paused = FALSE;    
finished = FALSE;
if (_checkForPause(conn))
return PQLXSRVRPAUSE;
_initFunc = initCall;
_procFunc = procCall;
_doneFunc = doneCall;
eventsQ = g_queue_new();
sqlStr = g_strdup_printf(SQL_EVENT_LIST, eventType);
ret = db_exec_sql(conn, sqlStr);
if (ret != PQLXSUCCESS)
return ret;
res_set = mysql_store_result(conn);
while((row = mysql_fetch_row(res_set)))
{
event = calloc(1, sizeof(EVENT));
event->id = atoi(row[0]);
event->typeKey = atoi(row[1]);
g_queue_push_head(eventsQ, event);
}
mysql_free_result(res_set); res_set = NULL; bug15752(conn);
if (!g_queue_get_length(eventsQ)) 
{
g_queue_free(eventsQ);
eventsQ = NULL;
return PQLXSUCCESS;
}
sqlStr = g_strdup_printf(SQL_EVENT_UNPROC, eventType);
ret = db_exec_sql(conn, sqlStr);  bug15752(conn);
numThreads = (maxCPU > g_queue_get_length(eventsQ)) ? g_queue_get_length(eventsQ) : maxCPU;
for(i=0;i<numThreads;i++)
{
ARGS *args;
args = calloc(1, sizeof(ARGS));
args->threadNum = i+1;
g_thread_create((GThreadFunc) _threadProc, args, FALSE, NULL);
}
keepAliveReg(0, REGISTERKEEPALIVE);
while (!finished)
usleep(POLLTIME);
keepAliveReg(0, DEREGISTERKEEPALIVE);
g_queue_free(eventsQ);
eventsQ = NULL;
mngDBConnClose();  
return (paused ? PQLXSRVRPAUSE : PQLXSUCCESS);
}
