#ifndef EVRESP_H
#define EVRESP_H
#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <setjmp.h>
#include <stdarg.h>
#ifdef VERSION
#define REVNUM VERSION
#else
#define REVNUM "3.3.3"
#endif
#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif
#define QUERY_DELAY -1
#define STALEN 64
#define NETLEN 64
#define LOCIDLEN 64
#define CHALEN 64
#define OUTPUTLEN 256
#define TMPSTRLEN 64
#define UNITS_STR_LEN 16
#define DATIMLEN 23
#define UNITSLEN 20
#define BLKTSTRLEN 4
#define FLDSTRLEN 3
#define MAXFLDLEN 50
#define MAXLINELEN 256
#define FIR_NORM_TOL 0.02
enum units { UNDEF_UNITS, DIS, VEL, ACC, COUNTS1, VOLTS1, DEFAULT, PRESSURE, TESLA
};
enum filt_types { UNDEF_FILT, LAPLACE_PZ, ANALOG_PZ, IIR_PZ,
FIR_SYM_1, FIR_SYM_2, FIR_ASYM, LIST, GENERIC, DECIMATION,
GAIN, REFERENCE, FIR_COEFFS, IIR_COEFFS 
};
enum stage_types { UNDEF_STAGE, PZ_TYPE, IIR_TYPE, FIR_TYPE, 
GAIN_TYPE, LIST_TYPE, IIR_COEFFS_TYPE, GENERIC_TYPE
};
enum error_codes { NON_EXIST_FLD = -2, ILLEGAL_RESP_FORMAT = -5,
PARSE_ERROR = -4, UNDEF_PREFIX = -3, UNDEF_SEPSTR = -6,
OUT_OF_MEMORY = -1, UNRECOG_FILTYPE = -7,
UNEXPECTED_EOF = -8, ARRAY_BOUNDS_EXCEEDED = -9,
OPEN_FILE_ERROR = 2, RE_COMP_FAILED = 3,
MERGE_ERROR = 4, SWAP_FAILED = 5, USAGE_ERROR = 6,
BAD_OUT_UNITS = 7, IMPROP_DATA_TYPE = -10,
UNSUPPORT_FILTYPE = -11, ILLEGAL_FILT_SPEC = -12,
NO_STAGE_MATCHED = -13, UNRECOG_UNITS = -14
};
#ifdef WIN32
#ifdef complex
#undef complex
#endif
#define complex evr_complex
#define strcasecmp stricmp
#define strncasecmp strnicmp
#endif
struct complex {
double real;
double imag;
};
struct string_array {
int nstrings;
char **strings;
};
struct scn {
char *station;
char *network;
char *locid;
char *channel;
int found;
};
struct response {
char station[STALEN];
char network[NETLEN];
char locid[LOCIDLEN];
char channel[CHALEN];
struct complex *rvec;
int nfreqs; 
double *freqs; 
struct response *next;
};
struct file_list {
char *name;
struct file_list *next_file;
};
struct matched_files {
int nfiles;
struct file_list *first_list;
struct matched_files *ptr_next;
};
struct scn_list {
int nscn;
struct scn **scn_vec;
};
struct pole_zeroType { 
int nzeros; 
int npoles;
double a0;
double a0_freq;
struct complex *zeros;
struct complex *poles;
};
struct coeffType { 
int nnumer; 
int ndenom;
double *numer;
double *denom;
double h0; 
};
struct firType { 
int ncoeffs; 
double *coeffs;
double h0;
};
struct listType{ 
int nresp; 
double *freq;
double *amp;
double *phase;
};
struct genericType { 
int ncorners; 
double *corner_freq;
double *corner_slope;
};
struct decimationType { 
double sample_int; 
int deci_fact;
int deci_offset;
double estim_delay;
double applied_corr;
};
struct gainType { 
double gain; 
double gain_freq;
};
struct referType { 
int num_stages;
int stage_num;
int num_responses;
};
struct blkt {
int type;
union {
struct pole_zeroType pole_zero;
struct coeffType coeff;
struct firType fir;
struct listType list;
struct genericType generic;
struct decimationType decimation;
struct gainType gain;
struct referType reference;
} blkt_info;
struct blkt *next_blkt;
};
struct stage {
int sequence_no;
int input_units;
int output_units;
struct blkt *first_blkt;
struct stage *next_stage;
};
struct channel {
char staname[STALEN];
char network[NETLEN];
char locid[LOCIDLEN];
char chaname[CHALEN];
char beg_t[DATIMLEN];
char end_t[DATIMLEN];
char first_units[MAXLINELEN];
char last_units[MAXLINELEN];
double sensit;
double sensfreq;
double calc_sensit;
double calc_delay;
double estim_delay;
double applied_corr;
double sint;
int nstages;
struct stage *first_stage;
};
struct dateTime {
int year;
int jday;
int hour;
int min;
float sec;
};
extern char myLabel[20];
struct string_array *ev_parse_line(char *);
struct string_array *parse_delim_line(char *, char *);
int get_field(FILE *, char *, int, int, char *, int);
int test_field(FILE *, char *, int *, int *, char *, int);
int get_line(FILE *, char *, int, int, char *); 
int next_line(FILE *, char *, int *, int *, char *); 
int count_fields(char *);
int count_delim_fields(char *, char *);
int parse_field(char *, int, char *);
int parse_delim_field(char *, int, char *, char *);
int check_line(FILE *, int *, int *, char *);
int get_int(char *);
double get_double(char *);
int check_units(char *);
int string_match(const char *, char *, char *);
int is_int(const char *);
int is_real(const char *);
int is_IIR_coeffs (FILE *, int); 
int find_resp(FILE *, struct scn_list *, char *, struct channel *);
int get_resp(FILE *, struct scn *, char *, struct channel *);
int get_channel(FILE *, struct channel *);
int next_resp(FILE *);
struct matched_files *find_files(char *, struct scn_list *, int *);
int get_names(char *, struct matched_files *);
int start_child(char *, FILE **, FILE **, FILE **);
struct complex *alloc_complex(int);
struct response *alloc_response(int);
struct string_array *alloc_string_array(int);
struct scn *alloc_scn(void);
struct scn_list *alloc_scn_list(int);
struct file_list *alloc_file_list(void);
struct matched_files *alloc_matched_files(void);
double *alloc_double(int);
char *alloc_char(int);
char **alloc_char_ptr(int);
struct blkt *alloc_pz(void);
struct blkt *alloc_coeff(void);
struct blkt *alloc_fir(void);
struct blkt *alloc_ref(void);
struct blkt *alloc_gain(void);
struct blkt *alloc_list(void);
struct blkt *alloc_generic(void);
struct blkt *alloc_deci(void);
struct stage *alloc_stage(void);
void free_string_array(struct string_array *);
void free_scn(struct scn *);
void free_scn_list(struct scn_list *);
void free_matched_files(struct matched_files *);
void free_file_list(struct file_list *);
void free_pz(struct blkt *);
void free_coeff(struct blkt *);
void free_fir(struct blkt *);
void free_list(struct blkt *);
void free_generic(struct blkt *);
void free_gain(struct blkt *);
void free_deci(struct blkt *);
void free_ref(struct blkt *);
void free_stages(struct stage *);
void free_channel(struct channel *);
void free_response(struct response *);
void error_exit(int, char *, ...);
void error_return(int, char *, ...);
int parse_channel(FILE *, struct channel *);
int parse_pref(int *, int *, char *);
void parse_pz(FILE *, struct blkt *, struct stage *); 
void parse_coeff(FILE *, struct blkt *, struct stage *); 
void parse_iir_coeff(FILE *, struct blkt *, struct stage *); 
void parse_list(FILE *, struct blkt *, struct stage *); 
void parse_generic(FILE *, struct blkt *, struct stage *); 
int parse_deci(FILE *, struct blkt *); 
int parse_gain(FILE *, struct blkt *); 
void parse_fir(FILE *, struct blkt *, struct stage *); 
void parse_ref(FILE *, struct blkt *, struct stage *); 
int add_null(char *, int, char);
void merge_coeffs(struct blkt *, struct blkt **);
void merge_lists(struct blkt *, struct blkt **); 
void check_channel(struct channel *);
void check_sym(struct blkt *, struct channel *);
void calc_resp(struct channel *chan, double *freq, int nfreqs, struct complex *output,
char *out_units, int start_stage, int stop_stage, int useTotalSensitivityFlag);
void convert_to_units(int, char *, struct complex *, double);
void analog_trans(struct blkt *, double, struct complex *);
void fir_sym_trans(struct blkt *, double, struct complex *);
void fir_asym_trans(struct blkt *, double, struct complex *);
void iir_pz_trans(struct blkt *, double, struct complex *);
void calc_time_shift(double, double, struct complex *);
void zmul(struct complex *, struct complex *);
void norm_resp(struct channel *, int, int);
void calc_list(struct blkt *, int , struct complex *); 
void iir_trans(struct blkt *, double , struct complex *); 
int is_time (const char * );
int timecmp(struct dateTime *dt1, struct dateTime *dt2);
void print_chan(struct channel *, int, int, int, int, int, int);
void print_resp(double *, int, struct response *, char *, int);
void print_resp_itp(double *, int, struct response *, char *, int,
int, double, int);
struct response *evresp(char *, char *, char *, char *, char *, char *, char *,
double *, int, char *, char *, int, int, int, int);
struct response *evresp_itp(char *, char *, char *, char *, char *, char *,
char *, double *, int, char *, char *, int,
int, int, int, int, double, int);
void interpolate_list_blockette(double **freq, double **amp, double **phase,
int *p_number_points, double *req_freq_arr,
int req_num_freqs, double tension);
extern char SEEDUNITS[][UNITS_STR_LEN];
extern char FirstLine[];
extern int FirstField;
extern double Pi;
extern double twoPi;
extern int def_units_flag;
extern struct channel *GblChanPtr;
extern float unitScaleFact;
extern char *curr_file;
extern int curr_seq_no;
extern jmp_buf jump_buffer;
double unwrap_phase(double phase, double prev_phase, double range,
double *added_value);
double wrap_phase(double phase, double range, double *added_value);
int use_delay(int flag);
#endif
