#ifndef _EZXML_H
#define _EZXML_H
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <fcntl.h>
#ifdef __cplusplus
extern "C" {
#endif
#define EZXML_BUFSIZE 1024 
#define EZXML_NAMEM 0x80 
#define EZXML_TXTM 0x40 
#define EZXML_DUP 0x20 
typedef struct ezxml *ezxml_t;
struct ezxml {
char *name; 
char **attr; 
char *txt; 
size_t off; 
ezxml_t next; 
ezxml_t sibling; 
ezxml_t ordered; 
ezxml_t child; 
ezxml_t parent; 
short flags; 
};
ezxml_t ezxml_parse_str(char *s, size_t len);
ezxml_t ezxml_parse_fd(int fd);
ezxml_t ezxml_parse_file(const char *file);
ezxml_t ezxml_parse_fp(FILE *fp);
ezxml_t ezxml_child(ezxml_t xml, const char *name);
#define ezxml_next(xml) ((xml) ? xml->next : NULL)
ezxml_t ezxml_idx(ezxml_t xml, int idx);
#define ezxml_name(xml) ((xml) ? xml->name : NULL)
#define ezxml_txt(xml) ((xml) ? xml->txt : "")
const char *ezxml_attr(ezxml_t xml, const char *attr);
ezxml_t ezxml_get(ezxml_t xml, ...);
char *ezxml_toxml(ezxml_t xml);
const char **ezxml_pi(ezxml_t xml, const char *target);
void ezxml_free(ezxml_t xml);
const char *ezxml_error(ezxml_t xml);
ezxml_t ezxml_new(const char *name);
#define ezxml_new_d(name) ezxml_set_flag(ezxml_new(strdup(name)), EZXML_NAMEM)
ezxml_t ezxml_add_child(ezxml_t xml, const char *name, size_t off);
#define ezxml_add_child_d(xml, name, off) ezxml_set_flag(ezxml_add_child(xml, strdup(name), off), EZXML_NAMEM)
ezxml_t ezxml_set_txt(ezxml_t xml, const char *txt);
#define ezxml_set_txt_d(xml, txt) ezxml_set_flag(ezxml_set_txt(xml, strdup(txt)), EZXML_TXTM)
ezxml_t ezxml_set_attr(ezxml_t xml, const char *name, const char *value);
#define ezxml_set_attr_d(xml, name, value) ezxml_set_attr(ezxml_set_flag(xml, EZXML_DUP), strdup(name), strdup(value))
ezxml_t ezxml_set_flag(ezxml_t xml, short flag);
ezxml_t ezxml_cut(ezxml_t xml);
ezxml_t ezxml_insert(ezxml_t xml, ezxml_t dest, size_t off);
#define ezxml_move(xml, dest, off) ezxml_insert(ezxml_cut(xml), dest, off)
#define ezxml_remove(xml) ezxml_free(ezxml_cut(xml))
#ifdef __cplusplus
}
#endif
#endif 
