#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <cairo.h>
#include "pql_defines.h"
#include "pql_externs.h"
#include "pqlx_externs.h" 
#include "gtk_pql_externs.h"
#include "pqlx_pdf.h"
#include "gtk_pdf_externs.h"
#include "pqlxSrvr.h"
GdkGC  *OGXC[MAXOVERLAY];  
const int numbers[TTLPDFSCREENS*MAXNUMBERS];
char *months[] = {"JAN", "FEB", "MAR", "APR", "MAY", "JUN", 
"JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };
typedef struct _DRAWSTUFF
{
cairo_surface_t *surface, *cBar;
cairo_t   *cr;
} DRAWSTUFF;
static void freePDFData(PDFPNG *pdfPNG)
{
GSList *pdfReqIter, *listI;
PDFREQ *pdfReq;
for(pdfReqIter = pdfPNG->data.pdfReqs;
pdfReqIter;
pdfReqIter = g_slist_next(pdfReqIter))
{
pdfReq = pdfReqIter->data;
if (pdfReq->data.pdf)
free(pdfReq->data.pdf);
if (pdfReq->data.name)
free(pdfReq->data.name);
if (pdfReq->data.pdfName)
free(pdfReq->data.pdfName);
if (pdfReq->data.hits)
{
free(pdfReq->data.hits->hits);
free(pdfReq->data.hits);
}
if (pdfReq->data.stats)
{
free(pdfReq->data.stats->stats);
free(pdfReq->data.stats);
}
for(listI = pdfReq->data.statLines; listI; listI=g_slist_next(listI))
{
PDFSTATSL *line = listI->data;
free(line->line);
free(line);
}
g_slist_free(pdfReq->data.statLines);
free(pdfReq);
}
g_slist_free(pdfPNG->data.pdfReqs);
free(pdfPNG->data.ntw);
free(pdfPNG->data.stn);
free(pdfPNG->data.loc);
free(pdfPNG->data.chn);
}
static void freeSPECData(SPECPNG *specPNG)
{
GSList *specReqIter;
SPECREQ *specReq;
GSList *iter2;
GList *iter1;
SPECTRODATA *spectroData;
for(specReqIter = specPNG->data.specReqs;
specReqIter;
specReqIter = g_slist_next(specReqIter))
{
specReq = specReqIter->data;
if (specReq->data.pdfName)
free(specReq->data.pdfName);
if (specReq->data.name)
free(specReq->data.name);
if (specReq->data.dateBounds[STARTDT])
g_date_free(specReq->data.dateBounds[STARTDT]);
if (specReq->data.dateBounds[ENDDT])
g_date_free(specReq->data.dateBounds[ENDDT]);
if (specReq->plot.dateBounds[STARTDT])
g_date_free(specReq->plot.dateBounds[STARTDT]);
if (specReq->plot.dateBounds[ENDDT])
g_date_free(specReq->plot.dateBounds[ENDDT]);
for(iter1 = specReq->data.spectro;
iter1;
iter1 = g_list_next(iter1))
{
spectroData = iter1->data;
for (iter2 = spectroData->psdData;
iter2;
iter2 = g_slist_next(iter2))
{
free(iter2->data);
}
g_slist_free(spectroData->psdData);
}
g_list_free(specReq->data.spectro);
if (specReq->plot.dataMap)
free(specReq->plot.dataMap);
if (specReq->plot.gaps)
g_hash_table_destroy(specReq->plot.gaps);
}
g_slist_free(specPNG->data.specReqs);
free(specPNG->data.ntw);
free(specPNG->data.stn);
free(specPNG->data.loc);
free(specPNG->data.chn);
}
void freePNGData(int which, void *pngData)
{
switch (which)
{
case SYSPDFPNG:
freePDFData(pngData);
break;
case SYSSPECPNG:
freeSPECData(pngData);
break;
}
}  
typedef struct _PNGDIROUT 
{
char *dirName, *chnName;
} PNGDIROUT;
static PNGDIROUT *makePNGDir(char *wwwDir, char *ntw, char *stn, char *loc, char *chn)
{
PNGDIROUT *pngDirOut;
char *chnName, *stnName, *dirName;
chnName = g_strdup_printf("%s.%s.%s.%s", ntw, stn, loc, chn);
stnName = g_strdup_printf("%s.%s", ntw, stn);
dirName = g_strdup_printf("%s/%s", wwwDir, stnName); 
free(stnName);
if (mkdir(dirName, 0777))
{
if (errno != EEXIST)
{
fprintf(stdout, "Unable to make wwwDir %s, cannot create WWW PNG plots.\n", dirName);
fprintf(stderr, "Unable to make wwwDir %s, cannot create WWW PNG plots.\n", dirName);
return NULL;
}
}
pngDirOut = calloc(1, sizeof(PNGDIROUT));
pngDirOut->chnName = chnName;
pngDirOut->dirName = dirName;
return pngDirOut;
}
static void plotBrdr(cairo_t *cr, PNGARGS *pngArgs, gboolean thumb)
{ 
int  R = pngArgs->plot.R, G = pngArgs->plot.G, B = pngArgs->plot.B;
int  width = pngArgs->plot.width, height = pngArgs->plot.height;
float adj = 0.0;
cairo_save(cr);
switch (thumb)
{
case TRUE:
adj = .5;
cairo_set_line_width(cr, 1.0);
break;
case FALSE:
cairo_set_line_width(cr, 2.0);
break;
}
cairo_set_source_rgb(cr, (double) (R/255.), (double) (G/255.), (double) (B/255.));
cairo_move_to(cr, (double) (0-adj), (double) (1-adj));
cairo_line_to(cr, (double) (width-1+adj), (double) (1-adj));
cairo_line_to(cr, (double) (width-1+adj), (double) (height-1+adj));
cairo_line_to(cr, (double) (1-adj), (double) (height-1+adj));
cairo_line_to(cr, (double) (1-adj), (double) (1-adj));
cairo_stroke(cr);
cairo_restore(cr);
}
int makePNG(PNGARGS *pngArgs)
{
int  plotType = pngArgs->plot.type;
int  width = pngArgs->plot.width, height = pngArgs->plot.height;
char *wwwDir = pngArgs->plot.wwwDir;
gboolean border = pngArgs->plot.border;
gboolean thumb;
int  threadNum = pngArgs->threadNum;
PNGDIROUT *pngDirOut;
char  *dirName, *chnName;
static gboolean invalidCall = FALSE;
static GHashTable *drawLU;
int   w, h, ret=PQLXSUCCESS;
char  *hT, *hTK;
gboolean data;
char  *fName=NULL;
DRAWSTUFF *drawHelp;
cairo_surface_t *surface, *cBar;
cairo_t   *cr;
cairo_surface_t* (*colorBarF)(CBARARGS *);
CBARARGS cBarArgs;
if (invalidCall)
return PQLXSUCCESS;
if (!width)  
return ret;
if (!drawLU)
drawLU = g_hash_table_new_full(g_str_hash, g_str_equal, 
(GDestroyNotify)free, (GDestroyNotify)free);
switch (plotType)
{
case SYSPDFPNG:
thumb = (width < THUMBSIZEMAXWPDF) ? TRUE : FALSE;
hT = g_strdup_printf("%d %d %d", plotType, threadNum, width);
colorBarF = colorBarPDF;
break;
case SYSSPECPNG:
thumb = (width < THUMBSIZEMAXWSPEC) ? TRUE : FALSE;
hT = g_strdup_printf("%d %d %d", plotType, threadNum, width);
colorBarF = colorBarSPEC;
cBarArgs.w = WWWSIDEM-5;
cBarArgs.h = height - (WWWTOPM + WWWBOTM)+2;
break;
}
if (!(drawHelp = g_hash_table_lookup(drawLU, hT)))
{
drawHelp = calloc(1, sizeof(DRAWSTUFF));
drawHelp->surface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, width, height);
drawHelp->cr = cairo_create(drawHelp->surface);
if (!thumb && plotType == SYSPDFPNG)
{ 
w = WWWSIDEM-5;
h = height - (WWWTOPM + WWWBOTM);
cBarArgs.w = w;
cBarArgs.h = h;
drawHelp->cBar = (*colorBarF)(&cBarArgs);
}
else 
drawHelp->cBar = NULL;
hTK = g_strdup(hT);
g_hash_table_insert(drawLU, (gpointer) hTK, drawHelp);
}
free(hT);
surface = drawHelp->surface;
cBar = drawHelp->cBar;
cr = drawHelp->cr;
switch (plotType)
{
case SYSPDFPNG:
{
PDFPNG  *pdfPNG = pngArgs->data.PDF.pdfPNG;
gboolean plotHLNM = pngArgs->data.PDF.plotHLNM;
GSList  *pdfReqIter;
PDFREQ  *pdfReq;
pngDirOut = makePNGDir(wwwDir, pdfPNG->data.ntw, pdfPNG->data.stn,
pdfPNG->data.loc, pdfPNG->data.chn);
if (!pngDirOut)
{
invalidCall = TRUE;
break;
}
dirName = pngDirOut->dirName;
chnName = pngDirOut->chnName;
for(pdfReqIter = pdfPNG->data.pdfReqs;
pdfReqIter;
pdfReqIter = g_slist_next(pdfReqIter))
{
pdfReq = pdfReqIter->data;
data = (pdfReq->data.numRecs == NOPSDDATA || pdfReq->data.numRecs == NOPSDBADSR)
? FALSE
: TRUE;
pdfReq->data.name = g_strdup(chnName);
cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
cairo_paint(cr);
plotPDFCairo(surface, pdfReq, width, height, PDFMAIN, WWWPLOT, plotHLNM);
if (data &&
cBar &&
!thumb)
{
cairo_save(cr);
cairo_set_source_surface(cr, cBar, width-WWWSIDEM+3, WWWTOPM);
cairo_paint(cr);
cairo_restore(cr);
}
if (border)
plotBrdr(cr, pngArgs, thumb);
fName = thumb
? g_strdup_printf("%s/%s.%s.icon.png", dirName, chnName, pdfReq->data.pdfName)
: g_strdup_printf("%s/%s.%s.png", dirName, chnName, pdfReq->data.pdfName);
remove(fName);
cairo_surface_write_to_png(surface, fName);
free(fName);
}
free(dirName);
free(chnName);
free(pngDirOut);
}
break;
case SYSSPECPNG:
{
SPECPNG  *specPNG = pngArgs->data.SPECTRO.specPNG;
GSList  *specReqIter;
SPECREQ  *specReq;
pngDirOut = makePNGDir(wwwDir, specPNG->data.ntw, specPNG->data.stn,
specPNG->data.loc, specPNG->data.chn);
if (!pngDirOut)
{
invalidCall = TRUE;
break;
}
dirName = pngDirOut->dirName;
chnName = pngDirOut->chnName;
for(specReqIter = specPNG->data.specReqs;
specReqIter;
specReqIter = g_slist_next(specReqIter))
{
specReq = specReqIter->data;
data = (specReq->data.numRecs == NOPSDDATA || 
specReq->data.numRecs == NOPSDBADSR)
? FALSE
: TRUE;
specReq->data.name = g_strdup(chnName);
cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
cairo_paint(cr);
plotSPECTROCairo(surface, specReq, width, height, PDFMAIN, WWWPLOT);
if (specReq->plot.dataMap)
{ 
free(specReq->plot.dataMap);
specReq->plot.dataMap = NULL;
}
if (data &&
!thumb)
{
cBarArgs.dbBounds[YMIN] = specReq->data.dbBounds[YMIN];
cBarArgs.dbBounds[YMAX] = specReq->data.dbBounds[YMAX];
cBarArgs.specReq = specReq;
cBar = (*colorBarF)(&cBarArgs);
cairo_save(cr);
cairo_set_source_surface(cr, cBar, width-WWWSIDEM+3, WWWTOPM-1);
cairo_paint(cr);
cairo_restore(cr);
}
if (border)
plotBrdr(cr, pngArgs, thumb);
fName = thumb
? g_strdup_printf("%s/%s.%s.SP.icon.png", dirName, chnName, specReq->data.pdfName)
: g_strdup_printf("%s/%s.%s.SP.png", dirName, chnName, specReq->data.pdfName);
remove(fName);
cairo_surface_write_to_png(surface, fName);
free(fName);
}
free(dirName);
free(chnName);
free(pngDirOut);
}
break;
}
return ret;
}
