#include <unistd.h>
#include <dirent.h> 
#include <time.h>
#include "pqlx_defs.h"
#include "pqlxSrvrUtils.h"
#include "importPDF.h"
typedef struct _PQLXSRVR
{
char *pdfSrvr;  
char *dirsFile;  
char *statsDir;  
FILE *logFP, *errFP; 
} PQLXSRVRSTRUCT;
PQLXSRVRSTRUCT pqlxSrvr;
GSList *ntw, *ntwPDF;
gboolean debugOUTPUT=FALSE;
char timeStr[50]; 
GOptionContext *context;
static GOptionEntry entries[] = 
{
{ "dbName", 0, 0, G_OPTION_ARG_STRING, NULL, "PQLX Database Name - required", "[SERVER:]pqlxDB" },
{ "statsDir", 0, 0, G_OPTION_ARG_STRING, NULL, "PDF-SA Statistics Directory - (this or --dirsFile)", "path-to-directory" },
{ "dirsFile", 0, 0, G_OPTION_ARG_STRING, NULL, "File of Directories to import - (this or --statsDir)", "path-to-file" },
{ NULL }
};
void importPDFdata(SCAN *, GSList *, CHNLIST *);
gboolean printName(GNode *node, gpointer *nothing)
{
NSLCDATA *nodeD=node->data;
fprintf(stdout, "depth: %d\tname: %s\tid:%d\n", 
g_node_depth(node), nodeD->name, nodeD->id);
return FALSE;
}
void makeStnList(GSList **PDFlist, int importType, char *importSrc)
{
switch(importType)
{
case FROMFILE:
{
FILE  *fp;
char inputLn[MAXINPUT];
if ((fp = fopen(importSrc, "r")) == NULL)
{
fprintf(stderr, "Unable to open file %s, quitting.\n", importSrc);
exit(-1);
}
while(fgets(inputLn, MAXINPUT, fp))
{
inputLn[strlen(inputLn)-1]=0;
*PDFlist = g_slist_insert(*PDFlist, (gpointer) g_strdup(inputLn), -1);
}
fclose(fp);
}
break;
case FROMDIR:
{
int  i;
char *ptr;
DIR  *dir = opendir(importSrc);
struct dirent *entry;
if (!dir) 
{
fprintf(stderr, "Cannot read directory '%s', quitting.\n", importSrc);
exit(-1);
}
while ((entry = readdir(dir))) 
{
if (!strcmp(entry->d_name, ".") || !strcmp(entry->d_name, ".."))
continue;
if (entry->d_name[0] == '.')
continue;
ptr = entry->d_name;
for(i=0;*ptr;ptr++) if (!(*ptr^'.')) i++; 
if ((i-NUMDOTS)) continue;     
*PDFlist = g_slist_insert(*PDFlist, 
(gpointer) g_strdup(entry->d_name), -1);
}
(void) closedir(dir);
}
break;
}
}
static void printHelp()
{
int argc=2;
char **argv;
GError *error = NULL;
argv = calloc(2, sizeof(char*));
argv[0] = g_strdup("importPDFdata");
argv[1] = g_strdup("--help");
g_option_context_parse (context, &argc, &argv, &error);
}
static gboolean parsePQLXargs(int argc, char **argv, PQLXSRVRSTRUCT *pqlxSrvr)
{
gboolean ret=TRUE;
GError   *error = NULL;
entries[0].arg_data = &pqlxSrvr->pdfSrvr;
entries[1].arg_data = &pqlxSrvr->statsDir;
entries[2].arg_data = &pqlxSrvr->dirsFile;
context = g_option_context_new ("- Execute importPDFdata");
g_option_context_add_main_entries (context, entries, NULL);
if (!g_option_context_parse (context, &argc, &argv, &error))
{
ret = FALSE;
if (error)
fprintf(stdout, "parse error: %s\n", error->message);
}
if (!pqlxSrvr->pdfSrvr)
ret = FALSE;
if ((!pqlxSrvr->statsDir && 
!pqlxSrvr->dirsFile) ||
(pqlxSrvr->statsDir && 
pqlxSrvr->dirsFile))
ret = FALSE;
if (!ret)
printHelp();
return ret; 
}
int main(int argc, char **argv)
{
int  ret, importT;
SQLDI sqldi; 
SCAN scan;
LOGIN login;
char *importSrc;
GSList *PDFDirs=NULL;
CHNLIST chnList;
TIMESTR(timeStr);
memset(&pqlxSrvr, 0, sizeof(PQLXSRVRSTRUCT));
if (!parsePQLXargs(argc, argv, &pqlxSrvr))
{
exit(-1);
}
memset(&sqldi, 0, sizeof(SQLDI));
memset(&chnList, 0, sizeof(CHNLIST));
memset(&login, 0, sizeof(LOGIN));
if(strchr(pqlxSrvr.pdfSrvr, ':'))
{ 
sscanf(pqlxSrvr.pdfSrvr, "%[^:]%*[:]%s", login.keys.hostname, login.keys.dbname);
}
else
{ 
strcpy(login.keys.hostname, "localhost");
strcpy(login.keys.dbname, pqlxSrvr.pdfSrvr);
}
strcpy(login.keys.username, PQLXSRVR);
strcpy(login.keys.password, PQLXSRVRPW);
openLogFile("importPDFdata", "log", login.keys.dbname, timeStr, &pqlxSrvr.logFP);
openLogFile("importPDFdata", "err", login.keys.dbname, timeStr, &pqlxSrvr.errFP);
sqldi.data = &login;
sqldi.args.dataEvent = CONNECT;
ret = db_request(&sqldi);
if (ret != PQLXSUCCESS)
{
TIMESTR(timeStr);
fprintf(stderr, "%sUnable to connect to Database, quitting.\n", timeStr);
exit(0); 
}
if(pqlxSrvr.statsDir)
{ 
importT = FROMDIR;
importSrc = pqlxSrvr.statsDir;
}
else
{ 
importT = FROMFILE;
importSrc = pqlxSrvr.dirsFile;
}
makeStnList(&PDFDirs, importT, importSrc);
if (!PDFDirs)
{
TIMESTR(timeStr);
fprintf(stderr, "%sNo valid directories found in %s, quitting.\n", timeStr, importSrc);
exit(1);
}
chdir(importSrc);
memset(&scan, 0, sizeof(SCAN));
importPDFdata(&scan, PDFDirs, &chnList);
memset(&sqldi, 0, sizeof(SQLDI));
scan.keys.scanTime = time(NULL);
strcpy(scan.keys.directory, importSrc);
scan.keys.scanType = PDFIMPORT;
sqldi.args.dataEvent = INSERT_DATA;
sqldi.descr.tableData = SCANTBL;
sqldi.data = &scan;
db_request(&sqldi);
memset(&sqldi, 0, sizeof(SQLDI));
sqldi.args.dataEvent = DISCONNECT;
db_request(&sqldi);
return 0;
}
