#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include "pql_defines.h"
#include "pqlx_defs.h"
#include "pqlx_externs.h"
#include "util_externs.h"
#include "pqlxSrvr.h"
#include "pqlxSrvrUtils.h"
#include "importTT.h"
char *procName;
char *modelName;  
gboolean extractPH=FALSE;
GSList *ntw, *ntwPDF; 
GOptionContext *context;
static GOptionEntry entries[] = 
{
{ "modelName", 0, 0, G_OPTION_ARG_STRING, NULL, "Travel-Time Model Name - required", NULL },
{ "extractPhases", 0, 0, G_OPTION_ARG_NONE, NULL, "Extract Phases List - optional", NULL },
{ NULL }
};
void process_tt_input(char *modelName);
void process_tt_phases(char *modelName);
void getOut(gboolean disC)
{
logMsg(LOGMSG_PROG_QUIT, procName);
if (disC)
{
SQLDI sqldi;
memset(&sqldi, 0, sizeof(SQLDI));
sqldi.args.dataEvent = DISCONNECT;
db_request(&sqldi);
}
logging(LOG_SHUTDOWN);
}
static void printHelp()
{
int argc=2;
char **argv;
GError *error = NULL;
argv = calloc(2, sizeof(char*));
argv[0] = g_strdup("pqlxSrvr");
argv[1] = g_strdup("--help");
g_option_context_parse (context, &argc, &argv, &error);
free(argv[0]);
free(argv[1]);
free(argv);
}
#if 0
static gboolean parseByHand(int argc, char **argv, PQLXSRVRSTRUCT *pqlxSrvr)
{
int i, j;
for(i=1;i<argc;i++)
{
for(j=0;entries[j].long_name;j++)
{
if (!strncmp((argv[i]+2), entries[j].long_name, strlen(entries[j].long_name)-1))
{
if (strlen(argv[i]) == strlen(entries[j].long_name)+3)
break;
switch(entries[j].arg)
{
case G_OPTION_ARG_STRING:
*((char**)entries[j].arg_data) = 
g_strdup((char *) (argv[i]+strlen(entries[j].long_name)+3));
break;
case G_OPTION_ARG_INT:
*(int *)entries[j].arg_data = 
atoi((char *) (argv[i]+strlen(entries[j].long_name)+3));
break;
default:
break;
}
break;
}
}
}
return TRUE;
}
#endif
static gboolean parsePQLXargs(int argc, char **argv)
{
gboolean ret=TRUE;
GError   *error = NULL;
entries[0].arg_data = &modelName;
entries[1].arg_data = &extractPH;
context = g_option_context_new ("- Execute importTT");
g_option_context_add_main_entries (context, entries, NULL);
if (!g_option_context_parse (context, &argc, &argv, &error))
{
ret = FALSE;
if (error)
fprintf(stdout, "parse error: %s\n", error->message);
#if 0
else
{ 
ret = parseByHand(argc, argv, pqlxSrvr);
}
#endif
}
if (!modelName)
ret = FALSE;
if (!ret)
printHelp();
return ret; 
}
static void process_tt_request(int type, char *modelName)
{
gchar *xmlInputDir;
char cwd[MAX_PATH_STR+1];
getcwd(cwd, MAX_PATH_STR+1);
xmlInputDir = getenv(ENVXML);
if (!xmlInputDir)
{
fprintf(stderr, "Environment variable '%s' not defined. Cannot import XML data.\n", ENVXML);
return;
}
if (chdir(xmlInputDir))
{
fprintf(stderr, "Unable to change to XMLINPUT directory %s, Cannot import Travel Time data.\n",
xmlInputDir);
return;
}
switch (type)
{
case TT_INPUT:
tt_input(modelName);
break;
case TT_EXTRACT:
tt_extract(modelName);
break;
}
chdir(cwd);
} 
int main(int argc, char **argv)
{
int  ret, type;
SQLDI sqldi;
LOGIN login;
char srvrName[50], dbName[50], execTime[50];
if (!parsePQLXargs(argc, argv))
{
exit(-1);
}
procName = g_strdup("importTT");
strcpy(srvrName, "localhost");
strcpy(dbName, PQLXMETADB);
TIMESTR(execTime); 
memset(&login, 0, sizeof(LOGIN));
memset(&sqldi, 0, sizeof(SQLDI));
if (!loggingInit("importTT", dbName, execTime, TRUE, TRUE))
{
fprintf(stderr, "Unable to initialize Logging, quitting...\n");
exit(-1);
}
strcpy(login.keys.hostname, srvrName);
strcpy(login.keys.dbname, dbName);
strcpy(login.keys.username, PQLXADMIN);
strcpy(login.keys.password, PQLXADMINPW);
login.keys.numCPUs = 1;
logMsg(LOGMSG_PROG_STARTUP, &login);
logMsg(LOGMSG_CONN_ATTEMPT, &login);
sqldi.data = &login;
sqldi.args.dataEvent = CONNECT;
ret = db_request(&sqldi);
if (ret != PQLXSUCCESS)
{
fprintf(stdout, "Connection Failed (error code = %d), cannot continue.\n", ret);
fprintf(stderr, "Unable to connect to '%s' Database, cannot continue.\n", PQLXMETADB);
getOut(FALSE);  
}
if (MINIMUM_PDB_VERSION < login.data.pdbVersion)
{
fprintf(stdout, "Unable to connect:\n");
fprintf(stdout, "\t'%s' Database is at version 0.%d\n", PQLXMETADB, login.data.pdbVersion);
fprintf(stdout, "\tBut this version of the server will only connect to version 0.%d.\n", MINIMUM_PDB_VERSION);
fprintf(stdout, "Please install a more recent version of this program to execute.\n");
getOut(TRUE);  
}
logging(LOG_OUTPUT);
logMsg(LOGMSG_CONN_SUCCESS, &login);
if (extractPH)
{
fprintf(stdout, "Extracting phases for model %s\n""Writing to file $PQLXPROD/XMLINPUT/%s.phases\n", 
modelName,modelName);
}
type = (extractPH) ? TT_EXTRACT : TT_INPUT;
process_tt_request(type, modelName);
getOut(TRUE);
return 0;  
}
