#include <unistd.h>
#include <dirent.h>
#include "pql_defines.h"
#include "pqlx_defs.h"
#include "util_externs.h"
#include "db_iface.h"
#include "ezxml.h"
#include "importTT.h"
#define TT_MODEL_FILE "tt"
#define TT_MODEL_HEAD "head"
#define TTINPUTLINELEN 255
#define TTCOLCNT  4
#if 0
static void printList(TTNAME *phname, gpointer nil)
{
fprintf(stderr, "phase %s\n", phname->name);
}
#endif
static gint findPhase(TTNAME *phaseD, char *phaseN)
{
return(strcmp(phaseD->name, phaseN));
}
static void freePhases(TTNAME *phase, gpointer nil)
{
free(phase);
}
void tt_input(char *modelName)
{
SQLDI sqldi;
TTINFO ttInfo;
TTNAME *name;
TTIME *ttime;
char line[TTINPUTLINELEN+1], phase[25], *in;
GSList *phases=NULL; 
GSList *phaseL=NULL; 
GSList *ttimes=NULL; 
FILE *fpH, *fpT;
float saveDep, depth, distance, time;
gboolean valid;
gchar *file;
int  argument;
file = g_strdup_printf("%s.%s", modelName, TT_MODEL_HEAD);
if (!(fpH = fopen(file, "r")))
{
fprintf(stderr, "Unable to open file %s, skipping\n", file);
free(file);
return;
}
free(file);
file = g_strdup_printf("%s.%s", modelName, TT_MODEL_FILE);
if (!(fpT = fopen(file, "r")))
{
fprintf(stderr, "Unable to open file %s, skipping\n", file);
free(file);
fclose(fpH);
return;
}
fprintf(stdout, "Importing Travel Time Table file %s...\n", file);
free(file);
memset(&sqldi, 0, sizeof(SQLDI));
memset(&ttInfo, 0, sizeof(ttInfo));
sqldi.descr.tableData = TTMODELTBL;
sqldi.args.dataEvent = INSERT_DATA;
sqldi.data = &ttInfo;
argument = MODELINSM;
sqldi.data2 = &argument;
#if 0
if (!fgets(line, TTINPUTLINELEN, fpH))
{
fprintf(stderr, "Error reading TT Table Header file, cannot process, exiting\n");
fclose(fpH);
fclose(fpT);
return;
}
word = strtok(line, sep);
ttInfo.keys.model = calloc(1, sizeof(TTMODEL));
ttInfo.keys.model->name = g_strdup(word);
for((word = strtok(NULL, sep));
word;
(word = strtok(NULL, sep)))
{
if (!strcmp(word, "\n"))
break;
name = calloc(1, sizeof(TTNAME));
name->name = g_strdup(word);
phases = g_slist_append(phases, name);
}
#endif
ttInfo.keys.model = calloc(1, sizeof(TTMODEL));
ttInfo.keys.model->name = g_strdup(modelName);
while ((in=fgets(line, TTINPUTLINELEN, fpH)))
{ 
char str[25];
name = calloc(1, sizeof(TTNAME));
sscanf(line, "%s", str);
name->name = g_strdup(str);
phases = g_slist_append(phases, name);
}
fclose(fpH);
ttInfo.keys.model->phases = phases;
db_request(&sqldi);
argument = MODELINST;
sqldi.data2 = &argument;
valid = FALSE;
while (!valid &&
(in=fgets(line, TTINPUTLINELEN, fpT)))
{ 
if (sscanf(line, "%s %f %f %f", phase, &depth, &distance, &time) != TTCOLCNT)
continue;
phaseL = g_slist_find_custom(ttInfo.keys.model->phases, phase, (GCompareFunc) findPhase);
if (!phaseL)
continue;
ttime = calloc(1, sizeof(TTIME));
ttime->depth = depth;
ttime->distance = distance;
ttime->ttime = time;
ttime->phaseKey = ((TTNAME *)phaseL->data)->key;
ttimes = g_slist_append(ttimes, ttime);
valid = TRUE;
saveDep = ttime->depth;
}
if (!valid)
{
fprintf(stderr, "No phases defined in header file found in Travel Time Table file.\n");
fprintf(stderr, "Nothing to import, Model Information is being deleted.\n");
fprintf(stderr, "Please check the header file contents vs. the Travel Time Table contents and try again.\n");
sqldi.args.dataEvent = DELETE_DATA;
db_request(&sqldi);
fclose(fpT);
return;
}
for (in = fgets(line, TTINPUTLINELEN, fpT);
in;
(in = fgets(line, TTINPUTLINELEN, fpT)))
{
if (sscanf(line, "%s %f %f %f", phase, &depth, &distance, &time) != TTCOLCNT)
continue;
phaseL = g_slist_find_custom(ttInfo.keys.model->phases, phase, (GCompareFunc) findPhase);
if (!phaseL)
continue;
ttime = calloc(1, sizeof(TTIME));
ttime->depth = depth;
ttime->distance = distance;
ttime->ttime = time;
ttime->phaseKey = ((TTNAME *)phaseL->data)->key;
if (saveDep != ttime->depth)
{
ttInfo.data.ttimes = ttimes;
db_request(&sqldi);
g_slist_free(ttimes);
ttimes = NULL;
saveDep = ttime->depth;
}
ttimes = g_slist_prepend(ttimes, ttime);
}
ttInfo.data.ttimes = ttimes;
db_request(&sqldi);
argument = MODELINSB;
sqldi.data2 = &argument;
db_request(&sqldi);
g_slist_free(ttimes);
ttimes = NULL;
free(((TTNAME *) ttInfo.keys.model)->name);
g_slist_foreach(phases, (GFunc) freePhases, NULL);
g_slist_free(phases);
fclose(fpT);
fprintf(stdout, "\tDONE\n");
return;
}
#if 0
void process_tt_input(char *modelName)
{
gchar *xmlInputDir;
char cwd[MAX_PATH_STR+1];
getcwd(cwd, MAX_PATH_STR+1);
xmlInputDir = getenv(ENVXML);
if (!xmlInputDir)
{
fprintf(stderr, "Environment variable '%s' not defined. Cannot import XML data.\n", ENVXML);
return;
}
if (chdir(xmlInputDir))
{
fprintf(stderr, "Unable to change to XMLINPUT directory %s, Cannot import Travel Time data.\n",
xmlInputDir);
return;
}
_tt_table(modelName);
chdir(cwd);
}
#endif
