#include "pql_defines.h"
#include "pqlx_defs.h"
#include "pqlxSrvr.h"
#include "pqlxSrvrUtils.h"
gboolean delRespE=FALSE, spawned=FALSE;
GOptionContext *context;
static GOptionEntry entries[] = 
{
{ "dbName", 0, 0, G_OPTION_ARG_STRING, NULL, "PQLX Database Name - required", "[SERVER:]pqlxDB" },
{ "numCPU", 0, 0, G_OPTION_ARG_INT, NULL, "Number of CPUs to use - optional (default=1)", "#" },
{ "delRespErrors", 0, 0, G_OPTION_ARG_NONE, &delRespE, "Additionally Delete 'No Response File' Errors from EVENTQ table - optional", NULL },
{ "spawned", 0, 0, G_OPTION_ARG_NONE, &spawned, NULL, NULL },
{ NULL }
};
gboolean *loopRunning;
PQLXSRVRSTRUCT pqlxSrvr;
char *procName;
GSList *ntw, *ntwPDF;  
void dbMaintThreads(GQueue *t, int ncpu);
int logSetup(char *procName, char *dbName);
GAsyncQueue *commQ;
static void getOutDBM()
{
SQLDI sqldi;
memset(&sqldi, 0, sizeof(SQLDI));
sqldi.args.dataEvent = UPDATE_DATA;
sqldi.descr.tableData = SRVRCOMPLETE;
db_request(&sqldi);
logMsg(LOGMSG_PROG_QUIT, procName);
memset(&sqldi, 0, sizeof(SQLDI));
sqldi.args.dataEvent = DISCONNECT;
db_request(&sqldi);
logging(LOG_SHUTDOWN);
}
static void printHelp()
{
int argc=2;
char **argv;
GError *error = NULL;
argv = calloc(2, sizeof(char*));
argv[0] = g_strdup(DBMAINTPROG);
argv[1] = g_strdup("--help");
g_option_context_parse (context, &argc, &argv, &error);
}
static gboolean parsePQLXargs(int argc, char **argv, PQLXSRVRSTRUCT *pqlxSrvr)
{
gboolean ret=TRUE;
GError   *error = NULL;
char  *progName = g_strdup_printf(" - Execute %s", DBMAINTPROG);
entries[0].arg_data = &pqlxSrvr->pdfSrvr;
entries[1].arg_data = &pqlxSrvr->numCPUS;
context = g_option_context_new(progName);
g_option_context_add_main_entries(context, entries, NULL);
if (!g_option_context_parse (context, &argc, &argv, &error))
{
ret = FALSE;
if (error)
fprintf(stdout, "parse error: %s\n", error->message);
}
if (!pqlxSrvr->pdfSrvr)
ret = FALSE;
if (!ret)
printHelp();
free(progName);
return ret; 
}
#define MAX_DIR_PATH 2048  
static void mainProcess()
{
SQLDI sqldi;
DBMAINT dbMaint;
gchar *nothing;
GSList *chnIter;
int  ttlChn;
memset(&sqldi, 0, sizeof(SQLDI));
fprintf(stdout, "Optimizing Table 'TRC'\n");
memset(&sqldi, 0, sizeof(SQLDI));
memset(&dbMaint, 0, sizeof(DBMAINT));
sqldi.descr.tableData = SYSDBTBLOPT;
sqldi.args.dataEvent = DBMAINTAIN;
sqldi.data = &dbMaint;
dbMaint.keys.tableName = g_strdup_printf("`TRC`");
db_request(&sqldi);
free(dbMaint.keys.tableName);
fprintf(stdout, "Confirming Integrity and Optimizing Table 'GLAP'\n");
sqldi.descr.tableData = SYSDBGLAP;
db_request(&sqldi);
sqldi.descr.tableData = SYSDBTBLOPT;
dbMaint.keys.tableName = g_strdup_printf("`GLAP`");
db_request(&sqldi);
free(dbMaint.keys.tableName);
memset(&dbMaint, 0, sizeof(DBMAINT));
sqldi.descr.tableData = SYSDBPSDOPTCHANS;
sqldi.args.dataEvent = DBMAINTAIN;
sqldi.data = &dbMaint;
db_request(&sqldi);
ttlChn = g_slist_length(dbMaint.data.channels);
switch (ttlChn)
{
case 0:
fprintf(stdout, "NO PSD Tables to Optimize\n");
break;
default:
fprintf(stdout, "Optimizing PSD Data Tables: (total channels: %d)\n", ttlChn);
break;
}
if (dbMaint.data.channels &&
g_slist_length(dbMaint.data.channels))
{
GQueue *channelsQ = g_queue_new();
for(chnIter = dbMaint.data.channels; 
chnIter; 
chnIter = g_slist_next(chnIter))
{
g_queue_push_head(channelsQ, chnIter->data);
}
g_slist_free(dbMaint.data.channels);
commQ = g_async_queue_new();
dbMaintThreads(channelsQ, pqlxSrvr.numCPUS);
nothing = g_async_queue_pop(commQ);  
}
fprintf(stdout, "Cleaning Database %sTMP\n", pqlxSrvr.dbName);
memset(&dbMaint, 0, sizeof(DBMAINT));
sqldi.descr.tableData = SYSDBTMPDROP;
sqldi.args.dataEvent = DBMAINTAIN;
db_request(&sqldi);
}
int main(int argc, char **argv)
{
SQLDI sqldi;
LOGIN login;
char srvrName[50], dbName[50], execTime[50];
int  ret;
procName = g_strdup(DBMAINTPROG);
memset(&pqlxSrvr, 0, sizeof(PQLXSRVRSTRUCT));
pqlxSrvr.numCPUS = 1;  
if (!parsePQLXargs(argc, argv, &pqlxSrvr))
{
exit(-1);
}
if (!spawned)
{
fprintf(stderr, "This program (%s) cannot be called directly:\n", DBMAINTPROG);
fprintf(stderr, "\tProgram is invoked by pqlxSrvr Program ONLY\n");
fprintf(stderr, "\tDirectly executed, this program does NOTHING except print this message\n");
exit(0);
}
TIMESTR(execTime);
if (strchr(pqlxSrvr.pdfSrvr, ':'))
{ 
sscanf(pqlxSrvr.pdfSrvr, "%[^:]%*[:]%s", srvrName, dbName);
}
else
{ 
strcpy(srvrName, "localhost");
strcpy(dbName, pqlxSrvr.pdfSrvr);
}
pqlxSrvr.srvrName = g_strdup(srvrName);
pqlxSrvr.dbName = g_strdup(dbName);
if (!loggingInit("dbMaint", dbName, execTime, TRUE, TRUE))
{
fprintf(stderr, "Unable to initialize Logging, quitting...\n");
exit(-1);
}
memset(&login, 0, sizeof(LOGIN));
memset(&sqldi, 0, sizeof(SQLDI));
strcpy(login.keys.hostname, pqlxSrvr.srvrName);
strcpy(login.keys.dbname, pqlxSrvr.dbName);
strcpy(login.keys.username, PQLXSRVR);
strcpy(login.keys.password, PQLXSRVRPW);
strcpy(login.keys.pgmName, DBMAINTPROG);
login.keys.numCPUs = pqlxSrvr.numCPUS;
logMsg(LOGMSG_PROG_STARTUP, &login);
logMsg(LOGMSG_CONN_ATTEMPT, &login);
sqldi.data = &login;
sqldi.args.dataEvent = CONNECT;
ret = db_request(&sqldi);
if (ret != PQLXSUCCESS)
{
fprintf(stderr, "Connection Failed (error code = %d), cannot continue.\n", ret);
fprintf(stderr, "Unable to connect to Database '%s' on Server '%s', cannot continue.\n", 
login.keys.dbname, login.keys.hostname);
exit(-1); 
}
logging(LOG_OUTPUT);
logMsg(LOGMSG_CONN_SUCCESS, &login);
mainProcess();
getOutDBM();
return 0;
}
