#include <sys/wait.h>
#include <unistd.h>
#include "pqlxSrvr.h"
PQLXSRVRSTRUCT pqlxSrvr;
static TIMERESTRICT timeRestrict;
char *procName;
GSList *ntw, *ntwPDF; 
int  dsizes[TTLDTYPES];
char *statStr[TTLPDFSTATS] = { "Min", "Max", "Mode", "Median", "Mean", "5%", "10%",
"25%", "75%", "90%", "95%"};
gboolean noScan;
GOptionContext *context;
static GOptionEntry entries[] = 
{
{ "dbName", 0, 0, G_OPTION_ARG_STRING, NULL, "PQLX Database Name - required", "[SERVER:]pqlxDB" },
{ "numCPU", 0, 0, G_OPTION_ARG_INT, NULL, "Number of CPUs to use - optional (default=1)", "#" },
{ "identFile", 0, 0, G_OPTION_ARG_STRING, NULL, "Filename of Traces to be Re-Analyzed - optional", "path-to-file" },
{ "noScan", 0, 0, G_OPTION_ARG_NONE, NULL, "Do Not Scan Directories", NULL},
{ "TRStart", 0, 0, G_OPTION_ARG_STRING, NULL, "Time Restricted Start Time - optional", "HR:MN" },
{ "TREnd", 0, 0, G_OPTION_ARG_STRING, NULL, "Time Restricted End Time - optional", "HR:MN" },
{ "noFork", 0, 0, G_OPTION_ARG_NONE, NULL, "Invoke Event Processor in-line", NULL},
{ NULL }
};
#if 0
gboolean printName(GNode *node, gpointer *nothing)
{
NSLCDATA *nodeD=node->data;
switch (g_node_depth(node))
{
case 4:
fprintf(stdout, "depth: %d\tname: %s\tid:%d\t%s\n",
g_node_depth(node), nodeD->name, nodeD->id, nodeD->fullname);
break;
default:
fprintf(stdout, "depth: %d\tname: %s\tid:%d\n",
g_node_depth(node), nodeD->name, nodeD->id);
break;
}
return FALSE;
}
static void printNodes()
{
int i;
GSList *iter;
for(i=0, iter = ntw;iter!=NULL;i++, iter=g_slist_next(iter))
{
fprintf(stdout, "Network #%d:\n", i+1);
g_node_traverse(iter->data, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
(GNodeTraverseFunc) printName, NULL);
}
getchar();
}
#endif
static void spawnDBMaint()
{
gchar   **args, *binDir;
GSpawnFlags flags = 0;
int   numCPU = pqlxSrvr.numCPUS > 1 ? pqlxSrvr.numCPUS - 1 : 1;
int   numArgs;
char  *pgmName = DBMAINTPROG;
binDir = getenv(ENVBIN);
if (!binDir)
{
fprintf(stderr, "Environment Variable %s not set, Cannot spawn %s() program.\n", ENVBIN, pgmName);
return;
}
numArgs = 5;
args = calloc(numArgs, sizeof(gchar *));
args[0] = g_strdup_printf("%s/%s", binDir, pgmName);
args[1] = g_strdup_printf("--dbName=%s", pqlxSrvr.pdfSrvr);
args[2] = g_strdup_printf("--numCPU=%d", numCPU);
args[3] = g_strdup_printf("--spawned");
args[4] = NULL;
if (g_spawn_async(NULL, args, NULL, flags, NULL, NULL, NULL, NULL))
{
fprintf(stdout, "Spawning program '%s'\n", pgmName);
}
else
{
fprintf(stdout, "Invocation of program '%s' Failed\n", pgmName);
}
FREESTRINGS(args);
}
void getOut()
{
SQLDI sqldi;
memset(&sqldi, 0, sizeof(SQLDI));
sqldi.args.dataEvent = UPDATE_DATA;
sqldi.descr.tableData = SRVRCOMPLETE;
db_request(&sqldi);
logMsg(LOGMSG_PROG_QUIT, procName);
sqldi.args.dataEvent = DISCONNECT;
db_request(&sqldi);
logging(LOG_SHUTDOWN);
}
static void printHelp()
{
int argc=2;
char **argv;
GError *error = NULL;
argv = calloc(2, sizeof(char*));
argv[0] = g_strdup("pqlxSrvr");
argv[1] = g_strdup("--help");
g_option_context_parse(context, &argc, &argv, &error);
free(argv[0]);
free(argv[1]);
free(argv);
}
static gboolean parseByHand(int argc, char **argv, PQLXSRVRSTRUCT *pqlxSrvr)
{
int i, j;
for(i=1;i<argc;i++)
{
for(j=0;entries[j].long_name;j++)
{
if (!strncmp((argv[i]+2), entries[j].long_name, strlen(entries[j].long_name)-1))
{
if (strlen(argv[i]) == strlen(entries[j].long_name)+3)
break;
switch(entries[j].arg)
{
case G_OPTION_ARG_STRING:
*((char**)entries[j].arg_data) = 
g_strdup((char *) (argv[i]+strlen(entries[j].long_name)+3));
break;
case G_OPTION_ARG_INT:
*(int *)entries[j].arg_data = 
atoi((char *) (argv[i]+strlen(entries[j].long_name)+3));
break;
default:
break;
}
break;
}
}
}
return TRUE;
}
static gboolean parsePQLXargs(int argc, char **argv, PQLXSRVRSTRUCT *pqlxSrvr)
{
gboolean ret=TRUE;
GError   *error = NULL;
union {
int scan;
gboolean noScan;
} scan;
union {
int fork;
gboolean noFork;
} fork;
scan.scan = NONE;
fork.fork = NONE;
entries[0].arg_data = &pqlxSrvr->pdfSrvr;
entries[1].arg_data = &pqlxSrvr->numCPUS;
entries[2].arg_data = &pqlxSrvr->identFile;
entries[3].arg_data = &scan.noScan;
entries[4].arg_data = &timeRestrict.start;
entries[5].arg_data = &timeRestrict.end;
entries[6].arg_data = &fork.noFork;
context = g_option_context_new("- Execute pqlxSrvr");
g_option_context_add_main_entries(context, entries, NULL);
if (!g_option_context_parse(context, &argc, &argv, &error))
{
ret = FALSE;
if (error)
fprintf(stdout, "parse error: %s\n", error->message);
else
{ 
ret = parseByHand(argc, argv, pqlxSrvr);
}
}
noScan = (scan.scan == NONE) ? FALSE : TRUE;
pqlxSrvr->fork = (fork.fork == NONE) ? TRUE : FALSE;
if (!pqlxSrvr->pdfSrvr)
ret = FALSE;
if (!pqlxSrvr->fork)
{
pqlxSrvr->numCPUS = 1;
}
else
{
int numCPUs = get_ncpu_pqlx();  
if (pqlxSrvr->numCPUS <= 0)
pqlxSrvr->numCPUS = 1;   
if (pqlxSrvr->numCPUS > numCPUs)
pqlxSrvr->numCPUS = numCPUs; 
}
pqlxSrvr->identify = DIRSCAN;
if (pqlxSrvr->identFile)
pqlxSrvr->identify = FILEINPUT;
if (timeRestrict.start && timeRestrict.end)
timeRestrict.set = TRUE;
if (!ret)
printHelp();
return ret; 
}
static gboolean crashEvents(LOGIN *login, char *execTime, int which)
{ 
SQLDI sqldi;
CRASHD crash;
int  ret;
gboolean crashed = FALSE;
memset(&sqldi, 0, sizeof(SQLDI));
memset(&crash, 0, sizeof(CRASHD));
sqldi.args.dataEvent = GET_DATA;
sqldi.descr.tableData = CRASHEVENTS;
crash.keys.which = which;
sqldi.data = &crash;
ret = db_request(&sqldi);
if (ret == PQLXSUCCESS && 
crash.data.crashList)
{
crashed = TRUE;
switch (which)
{
case TRCCRASH:
{
GSList *iter;
TRCLST  *trcCrash;
LOGFILES *crashFP = openLogFile("srvr", "crash", login->keys.dbname, execTime);
fprintf(stderr, "The PQLX server crashed or abnormally exited attempting to "
"process the following Trace files:\n"); 
fprintf(stderr, "\tEvent\t TRACE\n");
for(iter = crash.data.crashList;
iter;
iter = g_slist_next(iter))
{
trcCrash = iter->data;
fprintf(stderr, "\t%d\t%s\n", trcCrash->event,
trcCrash->trcName);
if (crashFP)
fprintf(crashFP->fp, "%s\n", trcCrash->trcName);
free(trcCrash);
}
g_slist_free(crash.data.crashList);
if (crashFP)
fclose(crashFP->fp);
}
break;
case SYSCRASH:
{
GSList *iter;
SYSLST *sysCrash;
fprintf(stderr, "The PQLX server crashed or abnormally exited attempting "
"to execute System Events for the following Channels:\n");
fprintf(stderr, "\tEvent\tChannel Name\tKey\n");
for(iter = crash.data.crashList;
iter;
iter = g_slist_next(iter))
{
sysCrash = iter->data;
fprintf(stderr, "\t%d\t%s\t%d\n", 
sysCrash->event,
sysCrash->chnName,
sysCrash->chnKey);
free(sysCrash);
}
g_slist_free(crash.data.crashList);
}
break;
}
}
return crashed;
}
static gboolean scanLoop(LOGIN *login)
{
int  status;
int   i, execDays[2], exitStatus;
SQLDI  sqldi;
SCANS  scans;
SCAN  *scan;
GSList  *scanIter;
char  execTime[50];
char  *cwd;
gboolean instanceFound, firstTime=TRUE, connected=TRUE, 
eventsExist, dataAnalyzed = FALSE, scanRet;
gboolean scanForked=FALSE;
int   numForked=0;
scanLockFile(login->keys.dbname, SCAN_FDELETE);
if (noScan)
{
fprintf(stdout, "Option '--noScan' Specified, Directory Scanning is Disabled.\n");
}
memset(&sqldi, 0, sizeof(SQLDI));
if (!(cwd = g_get_current_dir()))
{
fprintf(stderr, "Cannot getcwd(), quitting\n");
sqldi.args.dataEvent = DISCONNECT;
db_request(&sqldi);
exit(0);
}
do 
{ 
if (!connected)
{ 
sqldi.data = login;
sqldi.args.dataEvent = CONNECT;
db_request(&sqldi);
connected = TRUE;
}
switch (noScan)
{
case TRUE:
instanceFound = FALSE;
break;
case FALSE:
memset(&scans, 0, sizeof(SCANS));
sqldi.descr.tableData = SCANTBL;
sqldi.data = &scans;
sqldi.args.dataEvent = GET_DATA;
db_request(&sqldi);
scanLockFile(login->keys.dbname, SCAN_FCREATE);
instanceFound = FALSE;
for (scanIter = scans.data.scans;
scanIter;
scanIter = g_slist_next(scanIter))
{ 
scan = scanIter->data;
TIMESTRGMT(execTime); sscanf(execTime, "%*d %d", &execDays[0]);
scanRet = scanDir(scan, firstTime, &scanForked, scan->keys.directory);
outputScanStats(scan, scanRet);
switch (scanRet)
{
case TRUE:    
login->data.numChn += scan->data.nChn;
scan->keys.scanType = FILESCAN;
sqldi.args.dataEvent = INSERT_DATA;
sqldi.data = scan;
db_request(&sqldi);
if (scan->data.nTrace)
instanceFound = TRUE;  
break;
case FALSE:
continue;
break;
}
}
scanLockFile(login->keys.dbname, SCAN_FDELETE);  
for (scanIter = scans.data.scans;
scanIter;
scanIter = g_slist_next(scanIter))
{
free(scanIter->data);
}
g_slist_free(scans.data.scans);
break;
}
trashDir();
if (pqlxSrvr.fork)
{
int  numToFork = 0;
gboolean forkInit = FALSE;
if (!scanForked)
{
sqldi.args.dataEvent = EVENTQ;
sqldi.descr.tableData = EVENT_GET_UNPROC;
eventsExist = (gboolean) db_request(&sqldi);
forkInit = TRUE;
numToFork = pqlxSrvr.numCPUS;
}
sqldi.args.dataEvent = DISCONNECT;
db_request(&sqldi);
connected = FALSE;
if (instanceFound || eventsExist)
{ 
dataAnalyzed = TRUE;
scanForked
? fprintf(stdout, "Starting %d Additional Event Processor\n", 1)
: fprintf(stdout, "Starting %d Event Processors\n", pqlxSrvr.numCPUS);
numForked = (int) scanForked + (int) forkEventProcs(forkInit, numToFork);
eventsExist = TRUE;
while (eventsExist)
{ 
for(i=numForked;i>0;i--)
wait(&status);   
exitStatus = WEXITSTATUS(status);
switch (exitStatus)
{
case PQLXSRVRPAUSE:
{
waitForResume();
numForked = (int) forkEventProcs(TRUE, pqlxSrvr.numCPUS);
eventsExist = TRUE;
}
break;
default:
eventsExist = FALSE;
break;
}
}
scanForked=FALSE;   
numForked = 0;
}
}
else
{
if(firstTime || instanceFound)
{ 
fprintf(stdout, "Beginning Data Analysis...\n");
pqlxEventProc(login, FALSE);
}
}
fprintf(stdout, "Event Processor Analysis Completed\n");
TIMESTRGMT(execTime); sscanf(execTime, "%*d %d", &execDays[1]);
firstTime = (execDays[0] != execDays[1])  
? TRUE
: FALSE;
} while (instanceFound);
free(cwd);
return dataAnalyzed;
}
static void plotPNGWWW(PNGWWW *pngWWW, int chnCnt)
{
int  status;
int  exitStatus;
SQLDI sqldi;
gboolean eventsExist = FALSE;
int  maxCPUs = chnCnt < pqlxSrvr.numCPUS ? chnCnt : pqlxSrvr.numCPUS;
memset(&sqldi, 0, sizeof(SQLDI));
fprintf(stdout, "Updating %s Plots of System PDFs for %d channels:\t", pngWWW->name, chnCnt);
fprintf(stdout, "(Output Directory '%s')\n", pngWWW->dir);
sqldi.args.dataEvent = EVENTQ;
sqldi.descr.tableData = PNGEVTS;
sqldi.data = pngWWW;
sqldi.data2 = GINT_TO_POINTER(SYSPNGPROC);
db_request(&sqldi);
if (pqlxSrvr.fork)
{ 
eventsExist = TRUE;
while (eventsExist)
{ 
forkEventProcs(TRUE, maxCPUs);
wait(&status);  
exitStatus = WEXITSTATUS(status);
switch (exitStatus)
{
case PQLXSRVRPAUSE:
{
waitForResume();
eventsExist = TRUE;
}
break;
default:
eventsExist = FALSE;
break;
}
}
}
else
pqlxEventProc(&pqlxSrvr.login, FALSE);
}
static gboolean mainProcess()
{
int  ret, chnCnt;
GSList *iter;
SQLDI sqldi;
char execTime[50];
gboolean eventsExist, crashed, dataAnalyzed, pngPlot;
LOGIN *login = &pqlxSrvr.login;
TIMESTR(execTime);
memset(&sqldi, 0, sizeof(SQLDI));
sqldi.data = &pqlxSrvr.pngWWW;
sqldi.descr.tableData = PNGWWWDEF;
sqldi.args.dataEvent = GET_DATA;
ret = db_request(&sqldi);
if (ret != PQLXSUCCESS)
{
fprintf(stderr, "Unable to retrieve PNG WWW information. Something wrong, cannot continue\n");
return FALSE;
}
process_xml_input(login, execTime);
switch(pqlxSrvr.identify)
{
case DIRSCAN:
dataAnalyzed = scanLoop(login);
break;
case FILEINPUT:
reAnalyze(login->keys.dbname);
dataAnalyzed = TRUE;
break;
}
if (pqlxSrvr.fork)
{ 
sqldi.data = login;
sqldi.args.dataEvent = CONNECT;
db_request(&sqldi);
}
if (!dataAnalyzed)
{
sqldi.args.dataEvent = EVENTQ;
sqldi.descr.tableData = EVENT_GET_NOPROC;
eventsExist = (gboolean) db_request(&sqldi);
if (!eventsExist)
{ 
fprintf(stdout, "Nothing Found, Nothing Analyzed, Nothing to do...\n");
return TRUE;
}
}
crashed = crashEvents(login, execTime, TRCCRASH);
sqldi.args.dataEvent = SYSTEM_DATA;
sqldi.descr.tableData = SYSTBLOPT;
db_request(&sqldi); 
pausable(PSDINDECES);
sqldi.args.dataEvent = SYSTEM_DATA;
sqldi.descr.tableData = PSDDELETIONS;
ret = db_request(&sqldi); 
if (ret > 0)
pausable(PSDCHNDELETE);
pausable(SYSSPSEPOCHS);
pausable(SPCHNIUPD);
pausable(SYSDIRSTATS);
pausable(SYSBASELINE);
sqldi.args.dataEvent = EVENTQ;
sqldi.descr.tableData = SYSPDFEVTS;
chnCnt = db_request(&sqldi);
if (chnCnt == 0) 
return TRUE;
pngPlot = FALSE;
for (iter = pqlxSrvr.pngWWW;
iter;
iter = g_slist_next(iter))
{
PNGWWW *pngWWW = iter->data;
if (!(pngWWW->dir &&
strcmp(pngWWW->dir, "NONE")))
continue;
sqldi.args.dataEvent = EVENTQ;
sqldi.descr.tableData = PNGEVTS;
sqldi.data = pngWWW;
sqldi.data2 = GINT_TO_POINTER(SYSPNGMAKE);
db_request(&sqldi);
pngPlot = TRUE;
}
pausable(PDFTBL);
switch (pngPlot)
{
case TRUE:
{ 
for (iter = pqlxSrvr.pngWWW;
iter;
iter = g_slist_next(iter))
{
PNGWWW *pngWWW = iter->data;
if (!(pngWWW->dir &&
strcmp(pngWWW->dir, "NONE")))
{
fprintf(stdout, "PNG Output Directory is NONE, No %s Plots Generated\n", pngWWW->name);
fprintf(stdout, "If you would like to turn this on, "
"Define the PNG Output Directories using 'pqlx-admin'\n");
continue;
}
plotPNGWWW(pngWWW, chnCnt);
}
}
break;
case FALSE:
fprintf(stdout, "ALL PNG Output Directories are NONE, No PNG Plots Generated\n");
fprintf(stdout, "If you would like to turn this on, "
"Define the PNG Output Directories using 'pqlx-admin'\n");
break;
}
if (crashed ||
crashEvents(login, execTime, SYSCRASH))
{ 
fprintf(stdout, "ATTENTION:\tThe PQLX server crashed or abnormally exited "
"while processing data, please see the .err and .crash "
"files for details\n");
}
return TRUE;
}
int main(int argc, char **argv)
{
SQLDI sqldi;
LOGIN *login = &pqlxSrvr.login;
char srvrName[50], dbName[50], *binDir, execTime[50];
int  i, ret;
gboolean mainProc, spawn = FALSE;
procName = g_strdup("pqlxSrvr");
memset(&pqlxSrvr, 0, sizeof(PQLXSRVRSTRUCT));
if (!parsePQLXargs(argc, argv, &pqlxSrvr))
{
exit(-1);
}
binDir = getenv(ENVBIN);
if (!binDir)
{
fprintf(stderr, "Environment Variable %s not set, please define and re-execute\n", ENVBIN);
exit(-1);
}
TIMESTR(execTime);
if (strchr(pqlxSrvr.pdfSrvr, ':'))
{ 
sscanf(pqlxSrvr.pdfSrvr, "%[^:]%*[:]%s", srvrName, dbName);
}
else
{ 
strcpy(srvrName, "localhost");
strcpy(dbName, pqlxSrvr.pdfSrvr);
}
pqlxSrvr.srvrName = g_strdup(srvrName);
pqlxSrvr.dbName = g_strdup(dbName);
pqlxSrvr.execTime = g_strdup(execTime);
if (!g_thread_supported()) 
{
g_thread_init(NULL);
gdk_threads_init();
}
if (!loggingInit("srvr", dbName, pqlxSrvr.execTime, TRUE, TRUE))
{
fprintf(stderr, "Unable to Initialize Logging, Necessarily Quitting...\n");
exit(-1);
}
memset(login, 0, sizeof(LOGIN));
memset(&sqldi, 0, sizeof(SQLDI));
strcpy(login->keys.hostname, pqlxSrvr.srvrName);
strcpy(login->keys.dbname, pqlxSrvr.dbName);
strcpy(login->keys.username, PQLXSRVR);
strcpy(login->keys.password, PQLXSRVRPW);
strcpy(login->keys.pgmName, "pqlxSrvr");
login->keys.numCPUs = pqlxSrvr.numCPUS;
if (pqlxSrvr.fork)
logMsg(LOGMSG_PROG_STARTUP, login);
else
{ 
char *logMsg;
logMsg = g_strdup_printf("pqlxSrvr Process Started (version %s) (on 1 CPU - no forking)", 
PROG_VERSION);
fprintf(stdout, "%s\n", logMsg); free(logMsg);
}
logMsg(LOGMSG_CONN_ATTEMPT, login);
sqldi.data = login;
sqldi.args.dataEvent = CONNECT;
ret = db_request(&sqldi);
if (ret != PQLXSUCCESS)
{
fprintf(stderr, "Connection Failed (error code = %d), cannot continue\n", ret);
fprintf(stderr, "Unable to connect to Database '%s' on Server '%s', cannot continue\n", 
login->keys.dbname, login->keys.hostname);
logging(LOG_SHUTDOWN);
exit(-1); 
}
logMsg(LOGMSG_CONN_SUCCESS, login);
if (MINIMUM_DB_VERSION < login->data.dbVersion)
{
fprintf(stderr, "Unable to connect:\n");
fprintf(stderr, "\tDatabase is at version 0.%d\n", login->data.dbVersion);
fprintf(stderr, "\tBut this version of the server will only connect to version 0.%d or less\n", MINIMUM_DB_VERSION);
fprintf(stderr, "Please install a more recent version of the PQLX server to execute\n");
getOut();
exit(-1);
}
logging(LOG_OUTPUT);
sqldi.args.dataEvent = DBMAINTRUNNING;
ret = db_request(&sqldi);
while (ret == DBMAINT_RUNNING)
{
static gboolean first = TRUE;
if (first)
fprintf(stdout, "Pausing Server execution until DB Maintenance Program completes...\n");
sleep(60);  
ret = db_request(&sqldi);
first = FALSE;
}
for (i=0;i<TTLDTYPES;i++)
{
switch(i)
{
case BIT32:
dsizes[i] = sizeof(gint32);
break;
case BIT16:
dsizes[i] = sizeof(short);
break;
case MY_FLOAT:
dsizes[i] = sizeof(float);
break;
case MY_DOUBLE:
dsizes[i] = sizeof(double);
break;
case MY_CHAR:
dsizes[i] = sizeof(char);
break;
}
} 
if (timeRestrict.set)
g_thread_create((GThreadFunc) restrictTime, &timeRestrict, TRUE, NULL);
mainProc = mainProcess();
if (mainProc)
{ 
sqldi.args.dataEvent = SYSTEM_DATA;
sqldi.descr.tableData = SYSSRVRFINISH;
sqldi.data = login;
spawn = (gboolean) db_request(&sqldi);  
if (spawn)
fprintf(stdout, "Spawning PQLX DB Maintenance Program '%s'\n", DBMAINTPROG);
}
getOut();
if (spawn)
{ 
spawnDBMaint();
}
return 0;
}
