#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <sys/stat.h>
#include "pqlxSrvr.h"
void TIMESTRGMT(char *timeStr)
{
time_t tloc;
struct tm *timeN;
time(&tloc);
timeN = calloc(1, sizeof(struct tm));
gmtime_r(&tloc, timeN);
sprintf(timeStr, "%d %03d %02d:%02d:%02d>>\t", 
timeN->tm_year + 1900, 
timeN->tm_yday+1, 
timeN->tm_hour,
timeN->tm_min, 
timeN->tm_sec);
free(timeN);
}
gboolean trashDir()
{
char *binDir;
binDir = getenv(ENVTRASH);
if (!binDir)
{
fprintf(stderr, "Environment Variable %s not defined. Cannot invoke PQLX Event Processor.\n", 
ENVTRASH);
fprintf(stderr, "All necessary Environment Variables can be found in file '$PQLX/PROD/PQLXprodVars'.\n");
return FALSE;
}
chdir(binDir);
return TRUE;
}
void outputScanStats(SCAN *scan, gboolean new)
{
switch (new)
{
case FALSE:
fprintf(stdout, "Scanned Directory %s: NO NEW Instances Identified\n", scan->keys.directory);
break;
case TRUE:
fprintf(stdout, "Scanned Directory %s: Found NEW Instances of:\n", scan->keys.directory);
if (scan->data.nTrace) fprintf(stdout, "\t%d\t- Trace Files\n", scan->data.nTrace);
if (scan->data.nResp) fprintf(stdout, "\t%d\t- Response Files\n", scan->data.nResp);
if (scan->data.nNtw) fprintf(stdout, "\t%d\t- Network Codes\n", scan->data.nNtw);
if (scan->data.nStn) fprintf(stdout, "\t%d\t- Station Codes\n", scan->data.nStn);
if (scan->data.nLoc) fprintf(stdout, "\t%d\t- Location Codes\n", scan->data.nLoc);
if (scan->data.nChn) fprintf(stdout, "\t%d\t- Channel Codes\n", scan->data.nChn);
break;
}
}
gboolean forkEventProcs(gboolean init, int numCPUs)
{
pid_t  pid;
static int ttlCPUs=0;
int  forkCPUs;
switch(init)
{ 
case TRUE:    
if (numCPUs < 1)
forkCPUs = 1;
else
forkCPUs = numCPUs;
ttlCPUs = forkCPUs;
break;
case FALSE:    
forkCPUs = pqlxSrvr.numCPUS - ttlCPUs;
break;
}
if (forkCPUs < 1)
return FALSE;
switch(pid=fork())
{
case -1:
fprintf(stderr, "Can't fork the PQLX Server Event Processor, Quitting...\n");
getOut();
break;
case 0:    
forkEvents(&pqlxSrvr.login, forkCPUs);
exit(0);
break;
default:;   
}
return TRUE;
}
#define NTWBIT 0x1
#define STNBIT 0x2
#define LOCBIT 0x4
#define CHNBIT 0x8
#define LINELEN 120
static int check_resp(traceInfo *dataHead)
{
char input[LINELEN+1];
int  i, ret = FAILURE;
char *line;
FILE *fp;
short tags=0;
if((fp=fopen(dataHead->data.fileName, "r")) == NULL)
{
fprintf(stderr, "Unable to open file %s.\n", dataHead->data.fileName);
return NOFILE;
}
memset(input, 0, sizeof(input));
if ((line=fgets(input, LINELEN, fp))==NULL)
{
fclose(fp);
return ret;
}
for(tags=0;
line && tags != 15;    
line=fgets(input,LINELEN,fp))
{
if (strstr(input, RESPNTW))
{
for(i=0;input[i]!=':' && i != LINELEN;i++);
if (i==LINELEN)
continue;
i++;
sscanf(&input[i], "%s", dataHead->data.head.network);
tags |= NTWBIT;
continue;
}
if (strstr(input, RESPSTN))
{
for(i=0;input[i]!=':' && i != LINELEN;i++);
if (i==LINELEN)
continue;
i++;
sscanf(&input[i], "%s", dataHead->data.head.station);
tags |= STNBIT;
continue;
}
if (strstr(input, RESPLOC))
{
for(i=0;input[i]!=':' && i != LINELEN;i++);
if (i==LINELEN)
continue;
i++;
sscanf(&input[i], "%s", dataHead->data.head.location);
if (!strcmp(dataHead->data.head.location, RESPNLOC) ||
!strlen(dataHead->data.head.location))
{ 
strcpy(dataHead->data.head.location, PQLXNLOC);
}
tags |= LOCBIT;
continue;
}
if (strstr(input, RESPCHN))
{
for(i=0;input[i]!=':' && i != LINELEN;i++);
if (i==LINELEN)
continue;
i++;
sscanf(&input[i], "%s", dataHead->data.head.channel);
tags |= CHNBIT;
continue;
}
}
if (tags == 15)
{
ret = PQLXSUCCESS;
}
fclose(fp);
return ret;
}
static int fileInfo(traceInfo *dataHead)
{ 
int  ret = NOFILE;
dataHead->data.dataInfo.head.numSamples = NONE; 
if (checkTrace(dataHead) == PQLXSUCCESS)
{
if (dataHead->data.fileType <= NANO_DATA) 
ret = dataHead->data.fileType;   
}
else
{
if (check_resp(dataHead) == PQLXSUCCESS)
{
ret = RESPFILE;
}
}
return ret;
}
static gboolean validFile(char *dir, char *fileName)
{
static GPtrArray *strings;
char    *str;
int     i;
if (!strings)
{
strings = g_ptr_array_new();
g_ptr_array_add(strings, g_strdup(".tar"));
g_ptr_array_add(strings, g_strdup("core"));
g_ptr_array_add(strings, g_strdup("idx.dat"));
g_ptr_array_add(strings, g_strdup("index.dat"));
g_ptr_array_add(strings, g_strdup("netindex.idx"));
}
for (i = 0;i<strings->len;i++)
{
str = g_ptr_array_index(strings, i);
if (strstr(fileName, str))
{
fprintf(stderr, "Filename '%s/%s' contains the string '%s', and, therefore, will not be processed.\n",
dir, fileName, str);
fprintf(stderr, "\tIf this file is a valid trace file, please remove '%s' from the name and re-run the server.\n",
str);
return FALSE;
}
}
return TRUE;
}
int findChn(TRC1 *trc1, SCAN *scanPtr)
{
gboolean failure=FALSE;
static traceInfo dataHead;
char ids[NUMIDS][10];
gboolean createID;
if (!validFile(trc1->keys.directory, trc1->keys.filename)) 
return 0;
memset(&dataHead, 0, sizeof(traceInfo));
dataHead.data.fileName = g_strdup_printf("%s/%s", trc1->keys.directory, trc1->keys.filename);
trc1->keys.fileType = fileInfo(&dataHead);
switch(trc1->keys.fileType)
{
case MSEED_DATA:
case SEGY_DATA:
case AH_DATA:
case SAC_DATA:
case DR100_DATA:
case NANO_DATA:
if (!strlen(dataHead.data.head.location))
{ 
strcpy(dataHead.data.head.location, PQLXNLOC);
}
trc1->data.sTime.epochS = (unsigned long long) dataHead.data.head.epochs;
trc1->data.sTime.date.yr = dataHead.data.head.year;
trc1->data.sTime.date.day = dataHead.data.head.day;
trc1->data.sTime.date.hr = dataHead.data.head.hour;
trc1->data.sTime.date.mn = dataHead.data.head.minute;
trc1->data.sTime.date.sec = dataHead.data.head.second;
trc1->data.sTime.m_secs = dataHead.data.head.m_secs;
passcal_month_day(&trc1->data.sTime.date, dataHead.data.head.day);
if (!strlen(dataHead.data.head.channel))
{ 
fprintf(stderr, "%s/%s:\tTrace File has no Channel Information specified, unable to process.\n",
trc1->keys.directory, trc1->keys.filename);
failure = TRUE;
}
if (!strlen(dataHead.data.head.network))
{
strcpy(dataHead.data.head.network, UNKNOWN);
}
if (!strlen(dataHead.data.head.station))
{
if (strlen(dataHead.data.head.sensorID))
{ 
strcpy(dataHead.data.head.station, dataHead.data.head.sensorID);
}
else
{ 
fprintf(stderr, "%s/%s:\tTrace File has no Station or SensorID Information specified, unable to process.\n",
trc1->keys.directory, trc1->keys.filename);
failure = TRUE;
}
}
strcpy(ids[NTW], dataHead.data.head.network);
strcpy(ids[STN], dataHead.data.head.station);
strcpy(ids[LOC], dataHead.data.head.location);
strcpy(ids[CHN], dataHead.data.head.channel);
freeMyData(TRCINFO, &dataHead);
createID = TRUE;
break;
case RESPFILE:  
strcpy(ids[NTW], dataHead.data.head.network);
strcpy(ids[STN], dataHead.data.head.station);
strcpy(ids[LOC], dataHead.data.head.location);
strcpy(ids[CHN], dataHead.data.head.channel);
free(dataHead.data.fileName);
createID = FALSE;
break;
default:   
free(dataHead.data.fileName);
failure = TRUE;
createID = FALSE;
break;
}
return( failure
? 0
: chnID( scanPtr,
ids[NTW],
ids[STN],
ids[LOC],
ids[CHN],
createID));
}
static char *scanLockF;
void scanLockFile(char *dbName, int cmd)
{
if (!scanLockF)
{
scanLockF = g_strdup_printf("/tmp/%s.scan.lock", dbName);
}
switch(cmd)
{
case SCAN_FCREATE:
{
FILE *fp;
if ((fp = fopen(scanLockF, "w")) == NULL)
{
fprintf(stderr, "Unable to create Scan Lock File %s\n", scanLockF);
break;
}
fclose(fp);
}
break;   
case SCAN_FDELETE:
remove(scanLockF);
break;
}
return;
}
gboolean scanIsLocked(char *dbName)
{
if (!scanLockF)
return FALSE;
return (g_file_test(scanLockF, G_FILE_TEST_EXISTS));
}
void waitForResume()
{ 
SQLDI   sqldi;
static int retCode;
memset(&sqldi, 0, sizeof(SQLDI));
sqldi.args.sqr = THR_SQR_WFR;
sqldi.args.dataEvent = EVENTQ;
sqldi.descr.tableData = EVENT_PAUSE_INPROC;
retCode = db_request(&sqldi);
fprintf(stdout, "%s\n", PQLXPAUSEMSG);
memset(&sqldi, 0, sizeof(SQLDI));
sqldi.args.sqr = THR_SQR_WFR;
sqldi.args.dataEvent = EVENTQ;
sqldi.descr.tableData = EVENT_GET_RESUME;
retCode = db_request(&sqldi);
while (retCode != PQLXSRVRRESUME)
{
usleep(POLLINTERVAL);
retCode = db_request(&sqldi);
}
fprintf(stdout, "%s\n", PQLXRESUMEMSG);
}
void pausable(int eventType)
{ 
SQLDI  sqldi;
int  ret;
memset(&sqldi, 0, sizeof(SQLDI));
sqldi.args.dataEvent = SYSTEM_DATA;
sqldi.descr.tableData = eventType;
ret = db_request(&sqldi);
while (ret == PQLXSRVRPAUSE)
{
waitForResume();
ret = db_request(&sqldi);
}
}
#if 0
debug purposes only
gboolean printName(GNode *node, gpointer *nothing)
{
NSLCDATA *nodeD=node->data;
fprintf(logFP, "depth: %d\tname: %s\tid:%d\n", 
g_node_depth(node), nodeD->name, nodeD->id);
return FALSE;
}
{
GSList *iter;
for(i=0, iter = ntw;iter!=NULL;i++, iter=g_slist_next(iter))
{
fprintf(logFP, "Network #%d:\n", i+1);
g_node_traverse(iter->data, G_IN_ORDER, G_TRAVERSE_ALL, -1, 
(GNodeTraverseFunc) printName, NULL);
}
getchar();
}
#endif
enum {
SRVRSTATE_PAUSE,
SRVRSTATE_PLAY
};
static int curTime()
{
time_t timeSEC;
struct tm *now;
timeSEC = time(NULL);
now = localtime(&timeSEC);
return(now->tm_sec + now->tm_min*60 + now->tm_hour*60*60);
}
void *restrictTime(TIMERESTRICT *tr)
{
int   SH, SM, EH, EM;
int   startSEC, endSEC, curSEC;
gboolean  midnightWrap;
int   nextState;
unsigned int sleepSEC;
fprintf(stdout, "Time Restricted Execution: (START = %s, END = %s)\n", tr->start, tr->end);
sscanf(tr->start, "%d:%d", &SH, &SM);
sscanf(tr->end, "%d:%d", &EH, &EM);
startSEC = SH*60*60 + SM*60;
endSEC = EH*60*60 + EM*60;
if (startSEC > endSEC)
midnightWrap = TRUE;
else
midnightWrap = FALSE;
while (1)
{
curSEC = curTime();
switch(midnightWrap)
{
case TRUE:
if (curSEC > endSEC && curSEC <= startSEC)
{ 
nextState = SRVRSTATE_PAUSE;
}
else
{ 
nextState = SRVRSTATE_PLAY;
}
break;
case FALSE:
if (curSEC >= startSEC && curSEC < endSEC)
{ 
nextState = SRVRSTATE_PLAY;
}
else
{ 
nextState = SRVRSTATE_PAUSE;
}
break;
}
switch(nextState)
{
case SRVRSTATE_PAUSE:
{ 
SQLDI sqldi;
fprintf(stdout, "Pausing Execution - Time Restricted\n");
memset(&sqldi, 0, sizeof(SQLDI));
sqldi.args.dataEvent = EVENTQ;
sqldi.descr.tableData = EVENT_PAUSE;
sqldi.args.sqr = THR_SQR_TR;
sqldi.args.callType = ASYNC; 
db_request(&sqldi);
curSEC = curTime();
sleepSEC = (startSEC > curSEC) 
? (startSEC - curSEC)
: (ONEDAYOFSECS - (curSEC - startSEC));
}
break;
case SRVRSTATE_PLAY:
{ 
SQLDI sqldi;
fprintf(stdout, "Resuming Execution - Time Restricted\n");
memset(&sqldi, 0, sizeof(SQLDI));
sqldi.args.dataEvent = EVENTQ;
sqldi.descr.tableData = EVENT_RESUME;
sqldi.args.sqr = THR_SQR_TR;
sqldi.args.callType = ASYNC;
db_request(&sqldi);
curSEC = curTime();
sleepSEC = (endSEC > curSEC) 
? (endSEC - curSEC)
: (ONEDAYOFSECS - (curSEC - endSEC));
}
break;
}
sleep(sleepSEC);
}
}
