#!/bin/bash

if	[ $# -ne 3 ] || \
	[ "$2" != "TOTAL" -a "$2" != "LIST" ] || \
	[ "$3" != "ALL" -a "$3" != "ACTIVE" ] ; then
	echo "usage: $0 [HOST:]DBName ( TOTAL | LIST ) ( ALL | ACTIVE )"
	echo "where: "
	echo "	TOTAL = Output TOTAL Number of Files"
	echo "	LIST = Output File Names"
	echo "	ALL = For ALL Defined Directories"
	echo "	ACTIVE = For Only ACTIVE Directories"
	echo "example:	$0 micros TOTAL ALL"
	echo "	Print TOTAL number of files for ALL Directories that will be identified by next pqlxSrvr execution"
	exit
fi

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

HOST=`echo $1 | sed 's/:.*//'`
DBn=`echo $1 | sed 's/.*://'`

if [ "$HOST" = "$DBn" ] ; then
	HOST=localhost
fi

# check that the database exists
check=`$PQLXBIN/_checkDB $HOST $DBn DE`
if [ "$check" == "FALSE"  ] ; then
	exit
fi

PWORD=PqLXAdmiN
execCMD="${mysqlexec} --host=$HOST --database=$DBn --user=pqlxAdmin --password=$PWORD --compress"

if [ "$3" = "ACTIVE" ] ; then
	sqlCMD="SELECT directory FROM DIRS WHERE STATUS = 'ACTIVE'"
else
	sqlCMD="SELECT directory FROM DIRS"
fi

dirs=`echo $sqlCMD | ${execCMD} | grep -v directory`

if [ "$2" = "TOTAL" ] ; then
	echo "Total Files next Identified by pqlxSrvr:"
else
	echo "Files next Identified by pqlxSrvr:"
fi
	
for dir in ${dirs}; do
	sqlCMD="SELECT FROM_UNIXTIME(time) AS STAMP FROM DIRS WHERE directory = '${dir}'"
	timeStamp=`echo ${sqlCMD} | ${execCMD} | grep -v STAMP`
	sqlCMD="SELECT DATE_FORMAT(FROM_UNIXTIME(time), '%Y%m%d%H%i.%s') AS STAMP FROM DIRS WHERE directory = '${dir}'"
	time=`echo ${sqlCMD} | ${execCMD} | grep -v STAMP`
	touch -t ${time} /tmp/timeFile
	if [ "$2" = "TOTAL" ] ; then
		echo -n "'${dir}' (since ${timeStamp}):	"
		find -L ${dir} -type f -newer /tmp/timeFile | wc -l
	else
		echo "'${dir}' (since ${timeStamp}):"
		find -L ${dir} -type f -newer /tmp/timeFile
	fi
	/bin/rm /tmp/timeFile
done


