#!/bin/bash

if [ $# -ne 2 ] ; then
	echo "usage: $0 [HOST:]DBName numCPU"
	echo "	Execute pqlxSrvr - Looping each execution over a single UN-Scanned Data Directory"
	echo "example:	$0 micros 4"
	echo "	Loop the pqlxSrvr for DB micros over UN-Scanned Data Directories using 4 CPU's"
	exit
fi

hostDB=$1
numCPU=$2

HOST=`echo ${hostDB} | sed 's/:.*//'`
DBn=`echo ${hostDB} | sed 's/.*://'`

if [ "$MYSQLBIN" = "" ] ; then
	mysqlexec=mysql
else
	mysqlexec=$MYSQLBIN/mysql
fi

if [ "$HOST" = "$DBn" ] ; then
	HOST=localhost
fi

# check that the database exists
check=`$PQLXBIN/_checkDB $HOST $DBn DE`
if [ "$check" == "FALSE"  ] ; then
	exit
fi

PWORD=PqLXAdmiN
execCMD="${mysqlexec} --host=$HOST --database=$DBn --user=pqlxAdmin --password=$PWORD --compress"

outFile=$PQLXLOG/srvr.${DBn}.DirLoop-`date +%Y.%j.%H%M`.log
echo "Beginning Execution... Detailed Logging found in file '$outFile'"
while [[ 1 ]] ; do
	sqlCMD="SELECT directory as dirName FROM DIRS WHERE time = 0 ORDER BY directory LIMIT 1"
	dir=`echo ${sqlCMD} | ${execCMD} | grep -v dirName`
	if [ "${dir}" = "" ] ; then
		break;		# no more directories where time = 0, we're done...
	fi
	echo "Executing pqlxSrvr to Analyze Data Directory ${dir}"
	echo "Executing pqlxSrvr to Analyze Data Directory ${dir}" >>$outFile
	$PQLXBIN/PQLXDataDirectory.sh ${hostDB} ENABLE ${dir} >>$outFile
	$PQLXBIN/pqlxSrvr --dbName=${hostDB} --numCPU=${numCPU}
	$PQLXBIN/PQLXDataDirectory.sh ${hostDB} DISABLE ${dir} >>$outFile
done

echo "No More UN-Scanned Directories to Process, Done..."
echo "No More UN-Scanned Directories to Process, Done..."  >>$outFile
