#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <cairo.h>
#include "pqlx_defs.h"
#include "pql_defines.h"
#include "pql_externs.h"
#include "gtk_pql_externs.h"
#include "pqlx_pdf.h"
#include "gtk_pdf_externs.h"
#include "pqlx_externs.h"
#include "magnify.h"
#include "noiseModels.h"
typedef struct _PDFPLOTARGS
{
int w, h, scr, type;
float *perB;
graph *gr;
cairo_surface_t *surface;
cairo_t *cr;
PDFREQ *pdfReq;
gboolean data;
} PDFPLOTARGS;
void viewport(graph *gr, int xmin, int xmax, int ymin, int ymax);
void window(graph *gr, double xmin, double xmax, double ymin, double ymax, int lx, int ly);
static gboolean pdf(guchar *rgbbuf, 
int widthDISP, 
int heightDISP, 
PDFREQ *pdfReq, 
graph *grP, 
float *perB, 
int stride)
{
gboolean ret;
int   prob, i;
int   pixX, pixY, pixelPos;
guchar  *probMapS;
int   widthIN, heightIN;
int   yBounds[TTLYBOUNDS];
float  maxProbability = PDFPLOT_MAX_PROB;
MAGARGS  magArgs;
yBounds[YMIN] = pdfReq->keys.yBounds[YMIN];
yBounds[YMAX] = pdfReq->keys.yBounds[YMAX];
int perCnt=0;
double oldPer=0;
for(i=0;i<pdfReq->data.numRecs;i++)
{
if (pdfReq->data.pdf[i].period != oldPer)
{
perCnt++;
oldPer=pdfReq->data.pdf[i].period;
}
}
widthIN = perCnt;
heightIN = (yBounds[YMAX]) - (yBounds[YMIN]);
if (!(probMapS = (guchar *) calloc(widthIN*heightIN, sizeof(guchar))))
{
fprintf(stderr, "Unable to allocate %d bytes, quitting\n", 
(int) (widthIN*heightIN*sizeof(guchar)));
exit(-1);
}
viewport(grP, 0, widthIN-1, 0, heightIN-1);
perB[PERSTART] = pdfReq->data.pdf[0].period;
perB[PEREND] = pdfReq->data.pdf[pdfReq->data.numRecs-1].period;
window(grP,  (double) (log(perB[PERSTART])/log(10)), 
(double) (log(perB[PEREND])/log(10)),
(double) yBounds[YMIN], (double) yBounds[YMAX], 
FALSE, 0);
for(i=0;i<pdfReq->data.numRecs;i++)
{
pixX = USER2PIXX(grP, (double) log(pdfReq->data.pdf[i].period)/log(10));
pixY = USER2PIXY(grP, (double) (pdfReq->data.pdf[i].power));
if (pixY > grP->pix_ymax || pixY < grP->pix_ymin ||
pixX > grP->pix_xmax || pixX < grP->pix_xmin ||
pdfReq->data.pdf[i].prob == 0.0)
continue;
prob = (pdfReq->data.pdf[i].prob*100.+.5)*PROBFACT;
pixelPos = pixY*widthIN + pixX;
probMapS[pixelPos] = (char) ((prob>MAX_PIXEL_VALUE) ? MAX_PIXEL_VALUE : prob);
}
memset(&magArgs, 0, sizeof(MAGARGS));
magArgs.plotType = PDFPLOT;
magArgs.probMapS = probMapS;
magArgs.rgbbuf = &rgbbuf;
magArgs.widthIN = widthIN;
magArgs.heightIN = heightIN;
magArgs.widthDISP = widthDISP;
magArgs.heightDISP = heightDISP;
magArgs.stride = stride;
ret = magnifyC(&magArgs);
free(probMapS);
return ret;
}
static void HLNM(PDFPLOTARGS *pdfPlotArgs)
{ 
graph  grL;
GdkPoint *line;
int   pixX, pixY, lineCnt, i, sideM, topM, botM;
int   w=pdfPlotArgs->w, h=pdfPlotArgs->h, type=pdfPlotArgs->type;
float  *perB=pdfPlotArgs->perB, adj=0.0;
cairo_t  *cr = pdfPlotArgs->cr;
double  grey = 125./256.;
gboolean valid;
PDFREQ  *pdfReq = pdfPlotArgs->pdfReq;
int yBounds[TTLYBOUNDS];
yBounds[YMIN] = pdfReq->keys.yBounds[YMIN];
yBounds[YMAX] = pdfReq->keys.yBounds[YMAX];
cairo_save(cr);
cairo_set_line_width(cr, 2.0);
cairo_set_source_rgb(cr, grey, grey, grey);
switch(type)
{
case STNICON:
sideM = 1;
topM = 1;
botM = 1;
break;
case PDFPLOT:
sideM = PDFSIDEM;
topM = PDFTOPM;
botM = PDFBOTM;
break;
case WWWPLOT:
if (w >= THUMBSIZEMAXWPDF)
{
sideM = WWWSIDEM;
topM = WWWTOPM;
botM = WWWBOTM;
}
else
{
sideM = THUMBSIDEM;
topM = THUMBTOPM;
botM = THUMBBOTM;
cairo_set_line_width(cr, 1.0);
adj = .5;
}
break;
}
viewport(&grL, sideM, w-sideM-1, topM, h-botM-1);
window(&grL, (double) perB[PERSTART], (double) perB[PEREND], 
(double) yBounds[YMIN], (double) yBounds[YMAX], TRUE, 0);
line = calloc(sizeof(GdkPoint), 1001);
valid = FALSE;
for(lineCnt=i=0;!valid && i<1001;i++)
{ 
pixX = USER2PIXX(&grL, (double) highNM[i][0]);
pixY = USER2PIXY(&grL, (double) highNM[i][1]);
if (pixY > grL.pix_ymax || pixY < grL.pix_ymin ||
pixX > grL.pix_xmax || pixX < grL.pix_xmin)
continue;
valid = TRUE;
line[lineCnt].x = pixX;
line[lineCnt++].y = pixY;
}
cairo_move_to(cr, pixX+adj, pixY+adj);
for(i--;i<1001;i++)
{
pixX = USER2PIXX(&grL, (double) highNM[i][0]);
pixY = USER2PIXY(&grL, (double) highNM[i][1]);
if (pixY >= grL.pix_ymax || pixY <= grL.pix_ymin ||
pixX >= grL.pix_xmax || pixX <= grL.pix_xmin)
continue;
if (lineCnt && 
(pixX == line[lineCnt-1].x && pixY == line[lineCnt-1].y))
continue;
cairo_line_to(cr, pixX+adj, pixY+adj);
line[lineCnt].x = pixX;
line[lineCnt++].y = pixY;
}
valid = FALSE;
for(lineCnt=i=0;!valid && i<1001;i++)
{ 
pixX = USER2PIXX(&grL, (double) lowNM[i][0]);
pixY = USER2PIXY(&grL, (double) lowNM[i][1]);
if (pixY > grL.pix_ymax || pixY < grL.pix_ymin ||
pixX > grL.pix_xmax || pixX < grL.pix_xmin)
continue;
valid = TRUE;
line[lineCnt].x = pixX;
line[lineCnt++].y = pixY;
}
cairo_move_to(cr, pixX+adj, pixY+adj);
for(i--;i<1001;i++)
{
pixX = USER2PIXX(&grL, (double) lowNM[i][0]);
pixY = USER2PIXY(&grL, (double) lowNM[i][1]);
if (pixY >= grL.pix_ymax || pixY <= grL.pix_ymin ||
pixX >= grL.pix_xmax || pixX <= grL.pix_xmin)
continue;
if (lineCnt && 
(pixX == line[lineCnt-1].x && pixY == line[lineCnt-1].y))
continue;
cairo_line_to(cr, pixX+adj, pixY+adj);
line[lineCnt].x = pixX;
line[lineCnt++].y = pixY;
}
cairo_stroke(cr);
cairo_restore(cr);
free(line);
}
static void plot_stats(PDFPLOTARGS *pdfPlotArgs)
{ 
graph  grL;
GdkPoint *line=NULL;
int   pixX, pixY, lineCnt, i, sideM, topM, botM, pixH, width, height, pixYStart;
int   w=pdfPlotArgs->w, h=pdfPlotArgs->h, type=pdfPlotArgs->type, numPts;
int   j, lineSize, stat;
float  *perB=pdfPlotArgs->perB;
GSList  *lineI;
PDFREQ  *pdfReq=pdfPlotArgs->pdfReq;
PDFSTATSL *pdfStatsL;
cairo_t  *cr = pdfPlotArgs->cr;
GdkColor offWhite = {0, 255*256-1, 242*256-1, 222*256-1},
green = {0, 0, 255*256-1, 0},
myWhite = {0, 255*256-1, 255*256-1, 255*256-1},
myBlack = {0, 0, 0, 0};
GdkColor *lineCol[TTLLINESIZES][TTLPDFSTATS] = {
{&green, &green, &myBlack, &myBlack, &myBlack, &myWhite, 
&myWhite, &myWhite, &myWhite, &myWhite, &myWhite},
{&green, &green, &myBlack, &myBlack, &myBlack, &offWhite, 
&offWhite, &offWhite, &offWhite, &offWhite, &offWhite}};
const double dash[2] = {4, 3}, ddash[4] = {6, 3, 3, 3};
int yBounds[TTLYBOUNDS];
yBounds[YMIN] = pdfReq->keys.yBounds[YMIN];
yBounds[YMAX] = pdfReq->keys.yBounds[YMAX];
if (!pdfReq->data.statLines ||  
w < THUMBSIZEMAXWPDF)    
return;
switch(type)
{
case WWWPLOT:
sideM = WWWSIDEM;
topM = WWWTOPM;
botM = WWWBOTM;
break;
}
lineSize = (h < 350) ? SMALLLINE : LARGELINE;
viewport(&grL, sideM, w-sideM-1, topM, h-botM-1);
window(&grL, (double) perB[PERSTART], (double) perB[PEREND], 
(double) yBounds[YMIN], (double) yBounds[YMAX], TRUE, 0);
pixYStart = USER2PIXY(&grL, (double) -195);
width = fontSIZEPCairo(cr, statStr[PDFSTAT_MEDIAN], PDFSTATSF, WIDTH);
height = fontSIZEPCairo(cr, statStr[PDFSTAT_MEDIAN], PDFSTATSF, HEIGHT);
pixH = abs(pixYStart - USER2PIXY(&grL, (double) (-192)));
if (pixH < height/2)
{
pixH = height/2;
}
cairo_save(cr);
cairo_set_line_width(cr, 2.0);
for(lineI=pdfReq->data.statLines, j=0;
lineI;
lineI=g_slist_next(lineI), j++)
{
pdfStatsL = lineI->data;
stat = pdfStatsL->statistic;
cairo_set_source_rgb(cr, lineCol[lineSize][stat]->red/65535., 
lineCol[lineSize][stat]->green/65535., 
lineCol[lineSize][stat]->blue/65535.);
switch(stat)
{
case PDFSTAT_MEDIAN:
case PDFSTAT_P10:
case PDFSTAT_P90:
cairo_set_dash(cr, dash, 2, 0);
break;
case PDFSTAT_MEAN:
case PDFSTAT_P25:
case PDFSTAT_P75:
cairo_set_dash(cr, ddash, 4, 0);
break;
default:
cairo_set_dash(cr, NULL, 0, 0);
break;
}   
if (!line || 
numPts != pdfStatsL->numPts)
{ 
if (line)
free(line);
numPts = pdfStatsL->numPts;
line = calloc(sizeof(GdkPoint), numPts);
}
pixX = USER2PIXX(&grL, (double) pdfStatsL->line[0].period);
pixY = USER2PIXY(&grL, (double) pdfStatsL->line[0].power);
cairo_move_to(cr, pixX, pixY);
for(lineCnt=i=1; i<pdfStatsL->numPts; i++)
{
if (pdfStatsL->line[i].period < perB[PERSTART] ||
pdfStatsL->line[i].period > perB[PEREND])
continue;
pixX = USER2PIXX(&grL, (double) pdfStatsL->line[i].period);
pixY = USER2PIXY(&grL, (double) pdfStatsL->line[i].power);
if (pixY > grL.pix_ymax || pixY < grL.pix_ymin ||
pixX > grL.pix_xmax || pixX < grL.pix_xmin)
continue;
if (lineCnt && 
(pixX == line[lineCnt-1].x && pixY == line[lineCnt-1].y))
continue;
line[lineCnt].x = pixX;
line[lineCnt++].y = pixY;
cairo_line_to(cr, pixX, pixY);
}
cairo_stroke(cr);
pixX = USER2PIXX(&grL, (double) perB[PERSTART]); 
pixY = pixYStart - j*pixH*2;
cairo_save(cr);
cairo_set_source_rgb(cr, (200*256-1)/65535., (200*256-1)/65535., (200*256-1)/65535.);
cairo_rectangle (cr, pixX+(LEG_LINE_START-2), pixY - pixH-1, 
(LEG_LINE_START-2+LEG_LINE_LEN+width+2), pixH*2+2);
cairo_fill(cr);
cairo_restore(cr);
line[0].x = pixX + LEG_LINE_START;
line[0].y = pixY;
cairo_move_to(cr, line[0].x, line[0].y);
line[1].x = pixX + (LEG_LINE_START + LEG_LINE_LEN);
line[1].y = pixY;
cairo_line_to(cr, line[1].x, line[1].y);
cairo_stroke(cr);
fontDrawPCairo(cr, statStr[stat], pixX+38, pixY-height/2, PDFSTATSF, FALSE);
}
free(line);
cairo_restore(cr);
}
static void plot_pdf_extras(PDFPLOTARGS *pdfPlotArgs)
{
char  *dispStr;
gboolean thumb = FALSE;
graph  grL;
int   pixX, pixY, i, horz, vert, w1, w2, h1, sideM, topM, botM;
int   w=pdfPlotArgs->w, h=pdfPlotArgs->h, scr=pdfPlotArgs->scr, 
type=pdfPlotArgs->type;
PDFREQ  *pdfReq=pdfPlotArgs->pdfReq;
float  *perB=pdfPlotArgs->perB;
int   font;
int   yrS, yrE, moS, moE, dyS, dyE, jdayS, jdayE;
cairo_t  *cr = pdfPlotArgs->cr;
cairo_surface_t *surface = pdfPlotArgs->surface;
int   yBounds[TTLYBOUNDS], x1;
yBounds[YMIN] = pdfReq->keys.yBounds[YMIN];
yBounds[YMAX] = pdfReq->keys.yBounds[YMAX];
cairo_save(cr);
cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
cairo_set_line_width (cr, 1.0);
switch(type)
{
case PDFPLOT:
sideM = PDFSIDEM;
topM = PDFTOPM;
botM = PDFBOTM;
font = LABELF;
break;
case WWWPLOT:
if (w >= THUMBSIZEMAXWPDF)
{
sideM = WWWSIDEM;
topM = WWWTOPM;
botM = WWWBOTM;
}
else
{
sideM = THUMBSIDEM;
topM = THUMBTOPM;
botM = THUMBBOTM;
thumb = TRUE;
}
if (w >= 500)
font = WWWLABELF1;
else if (w >= 400)
font = WWWLABELF2;
else if (w >= 300)
font = WWWLABELF3;
else if (w >= THUMBSIZEMAXWPDF)
font = WWWLABELF4;
else
font = WWWLABELF6;
break;
}
if (!pdfReq->data.name)
pdfReq->data.name = g_strdup(ChnName(pdfReq->keys.chnKey));
switch(pdfPlotArgs->data)
{
case TRUE:
dispStr = g_strdup_printf("%s : %d PSDs", pdfReq->data.name, pdfReq->data.numPSDs);
break;
case FALSE:
dispStr = g_strdup_printf("%s : 0 PSDs", pdfReq->data.name);
break;
}
vert = topM/2 - fontSIZEPCairo(cr, dispStr, font, HEIGHT);
w1 = fontSIZEPCairo(cr, dispStr, font, WIDTH)/2;
switch(scr)
{
case PDFMAIN:
case PDFREQUEST:
horz = w/2 - w1;
break;
case PDFDETAIL:
horz = sideM;
break;
}
fontDrawPCairo(cr, dispStr, horz, vert, font, FALSE);
free(dispStr);
if(pdfPlotArgs->data)
{ 
passcal_month_day(&pdfReq->data.sTime.date, pdfReq->data.sTime.date.jday);
passcal_month_day(&pdfReq->data.eTime.date, pdfReq->data.eTime.date.jday);
jdayS = pdfReq->data.sTime.date.jday;
jdayE = pdfReq->data.eTime.date.jday;
yrS = pdfReq->data.sTime.date.yr;
yrE = pdfReq->data.eTime.date.yr;
moS = pdfReq->data.sTime.date.mo;
moE = pdfReq->data.eTime.date.mo;
dyS = pdfReq->data.sTime.date.day;
dyE = pdfReq->data.eTime.date.day;
switch(pdfReq->data.dateFmt)
{
case PDF_DF_1:
dispStr = g_strdup_printf("%04d:%03d / %04d:%03d", yrS, jdayS, yrE, jdayE);
break;
case PDF_DF_2:
dispStr = g_strdup_printf("%04d-%02d-%02d / %04d-%02d-%02d", yrS, moS, dyS, yrE, moE, dyE);
break;
case PDF_DF_3:
dispStr = g_strdup_printf("%02d-%02d-%04d / %02d-%02d-%04d", dyS, moS, yrS, dyE, moE, yrE);
break;
case PDF_DF_4:
dispStr = g_strdup_printf("%02d-%s-%04d / %02d-%s-%04d", dyS, months[moS-1], yrS, dyE, months[moE-1], yrE);
break;
}
vert = topM/2;
w2 = fontSIZEPCairo(cr, dispStr, font, WIDTH)/2;
switch(scr)
{
case PDFMAIN:
case PDFREQUEST:
horz = w/2 - w2;
break;
case PDFDETAIL:
horz = sideM + w1 - w2;
break;
}
fontDrawPCairo(cr, dispStr, horz, vert, font, FALSE);
free(dispStr);
}
if (!pdfPlotArgs->data)
{ 
int dispFont = (thumb ? WWWLABELF5 : MSGF);
switch(pdfReq->data.numRecs)
{
case NOPSDDATA:
dispStr = g_strdup_printf("NO PDF Data Available");
break;
case NOPSDBADSR:
dispStr = g_strdup_printf("NO PDF Possible:\nMultiple Sample Rates");
break;
}
fontDrawPCairo(cr, dispStr,
w/2 - fontSIZEPCairo(cr, dispStr, dispFont, WIDTH)/2, 
h/2 - fontSIZEPCairo(cr, dispStr, dispFont, HEIGHT)/2, 
dispFont, FALSE);
free(dispStr);
cairo_move_to(cr, (double) sideM+.5, topM-.5);
cairo_line_to(cr, (double) w-sideM-1+.5, topM-.5);
cairo_move_to(cr, (double) sideM+.5, h-botM-.5);
cairo_line_to(cr, (double) w-sideM-1+.5, h-botM-.5);
goto GETOUT;
}
if (thumb)
{ 
cairo_move_to(cr, (double) sideM+.5, topM-.5);
cairo_line_to(cr, (double) w-sideM-1+.5, topM-.5);
}
else
{ 
dispStr = g_strdup("Period (Sec)");
h1 = fontSIZEPCairo(cr, dispStr, AXISF, HEIGHT);
w1 = fontSIZEPCairo(cr, dispStr, AXISF, WIDTH);
fontDrawPCairo(cr, dispStr, (w/2-w1/2), h-1-h1, AXISF, FALSE);
free(dispStr);
}
viewport(&grL, sideM, w-sideM-1, h-botM, h);
window(&grL, (double) perB[PERSTART], (double) perB[PEREND], 
(double) 0, (double) botM, TRUE, 0);
xaxisPCairo(surface, &grL, (double) perB[PERSTART], (double) botM, 
(double) perB[PEREND], 1, -1, -1, (double) 0.0, FALSE, (double) 0.0, 
(thumb ? font : AXISF));
if (thumb)
goto GETOUT;  
viewport(&grL, 0, sideM, topM, h-botM);
window(&grL, (double) 0, (double) sideM, (double) yBounds[YMIN], (double) yBounds[YMAX], 0, 0);
pixX = USER2PIXX(&grL, sideM);
dispStr = g_strdup("-200");
h1 = fontSIZEPCairo(cr, dispStr, AXISF, HEIGHT);
free(dispStr);
for(i=0;i<(yBounds[YMAX]-yBounds[YMIN])/10+1;i++)
{
pixY = USER2PIXY(&grL, yBounds[YMAX]-i*10);
cairo_move_to(cr, pixX, pixY+.5);
cairo_line_to(cr, pixX+4, pixY+.5);
if ((thumb == FALSE &&
!(i%3)) ||
(thumb == TRUE &&
(yBounds[YMAX]-i*10 == -200 ||
yBounds[YMAX]-i*10 == -50)))
{
switch(pdfReq->keys.type)
{
case BBAND_CONT:
dispStr = g_strdup_printf("%d", yBounds[YMAX]-i*10);
font = AXISF;
break;
case STMO_TRIG:
dispStr = g_strdup_printf("<span>10<sup>%d</sup></span>", (yBounds[YMAX]-i*10+20)/10);
font = PDFSTATSF;
break;
}
w1 = fontSIZEPCairo(cr, dispStr, font, WIDTH);
fontDrawPCairo(cr, dispStr, sideM-4-w1, pixY-h1/2, font, FALSE);
free(dispStr);
}
}
cairo_move_to(cr, (double) sideM+.5, topM);
cairo_line_to(cr, (double) sideM+.5, h-botM+1);
cairo_move_to(cr, (double) w-sideM-1+.5, topM);
cairo_line_to(cr, (double) w-sideM-1+.5, h-botM+1);
switch(pdfReq->keys.type)
{
case BBAND_CONT:
dispStr = g_strdup_printf("dB");
x1 = sideM/2-h1;
font = AXISF;
break;
case STMO_TRIG:
dispStr = g_strdup_printf("<span>(cm/sec<sup>2</sup>)<sup>2</sup></span>");
x1 = sideM/2-h1*1.5;
font = PDFSTATSF;
break;
}
h1 = fontSIZEPCairo(cr, dispStr, font, HEIGHT);
w1 = fontSIZEPCairo(cr, dispStr, font, WIDTH);
pixY = USER2PIXY(&grL, (yBounds[YMIN]-yBounds[YMAX])/2+yBounds[YMAX]);
fontDrawPCairo(cr, dispStr, x1, pixY+w1/2, font, TRUE);
free(dispStr);
GETOUT:
cairo_stroke(cr);
cairo_restore(cr);
}
void plotPDFCairo(cairo_surface_t *surface, PDFREQ *pdfReq, 
int width, int height, int scr, int type, gboolean plotHLNM)
{
guchar *rgbbuf;
graph gr;
int  wL, hL, sideM, topM, botM, yBounds[TTLYBOUNDS];
gboolean ret;
float perB[2];
PDFPLOTARGS pdfPlotArgs;
gboolean plotExtras;
cairo_surface_t *pdfOut;
cairo_t *cr;
int  stride;
yBounds[YMIN] = pdfReq->keys.yBounds[YMIN];
yBounds[YMAX] = pdfReq->keys.yBounds[YMAX];
if (pdfReq->data.numRecs != -1 &&
yBounds[YMIN] == 0 &&
yBounds[YMAX] == 0)
{
fprintf(stderr, "Internal Error: PDF Y-Axis size/dimension == 0, not allowed. (channel: %s)\n",
pdfReq->data.name);
return;
}
switch(type)
{
case STNICON:
sideM = 1;
topM = 1;
botM = 1;
plotExtras = FALSE;
break;
case PDFPLOT:
sideM = PDFSIDEM;
topM = PDFTOPM;
botM = PDFBOTM;
plotExtras = TRUE;
break;
case WWWPLOT:
if (width >= THUMBSIZEMAXWPDF)
{
sideM = WWWSIDEM;
topM = WWWTOPM;
botM = WWWBOTM;
}
else
{
sideM = THUMBSIDEM;
topM = THUMBTOPM;
botM = THUMBBOTM;
}
plotExtras = TRUE;
break;
}
wL = width - 2*sideM;
hL = height - (topM + botM);
if (wL <= 0 ||  
hL <= 0)
return;
cr = cairo_create(surface);
pdfPlotArgs.surface = surface;
pdfPlotArgs.cr = cr;
pdfPlotArgs.pdfReq = pdfReq;
pdfPlotArgs.gr = &gr;
pdfPlotArgs.perB = perB;
pdfPlotArgs.w = width;
pdfPlotArgs.h = height;
pdfPlotArgs.scr = scr;
pdfPlotArgs.type = type;
if (pdfReq->data.numRecs == NOPSDDATA ||
pdfReq->data.numRecs == NOPSDBADSR)
pdfPlotArgs.data = FALSE;
else
pdfPlotArgs.data = TRUE;
stride = CAIRO_STRIDE_FOR_WIDTH_BPPM(wL, 32);
rgbbuf = malloc(stride * hL);
memset(rgbbuf, 240, stride * hL);  
if (pdfPlotArgs.data)
{
ret = pdf(rgbbuf, wL, hL, pdfReq, &gr, perB, stride);
if (ret)
{
pdfOut = cairo_image_surface_create_for_data(rgbbuf, CAIRO_FORMAT_RGB24, wL, hL, stride);
cairo_set_source_surface(cr, pdfOut, sideM, topM);
cairo_rectangle(cr, sideM, topM, wL, hL);
cairo_fill(cr);
cairo_surface_destroy(pdfOut);
if (plotHLNM)
HLNM(&pdfPlotArgs);
plot_stats(&pdfPlotArgs);
}
else
plotExtras = FALSE;
}
else
{
pdfOut = cairo_image_surface_create_for_data(rgbbuf, CAIRO_FORMAT_RGB24, wL, hL, stride);
cairo_set_source_surface(cr, pdfOut, sideM, topM);
cairo_rectangle(cr, sideM, topM, wL, hL);
cairo_fill(cr);
cairo_surface_destroy(pdfOut);
}
if(plotExtras)
{
plot_pdf_extras(&pdfPlotArgs);
}
free(rgbbuf);
cairo_destroy(cr);
return;
}
cairo_surface_t *colorBarPDF(CBARARGS *cBarArgs)
{
int  w = cBarArgs->w, h = cBarArgs->h;
int  i, j, y, y1, stride, strideW;
graph grL;
myColorStruct *colorProb;
guchar *pos, *rgbbuf;
gchar *str;
cairo_surface_t *cBar, *cBar2;
cairo_t *cr;
guint32 *pixel;
if (h <= 0)  
return NULL;
stride = CAIRO_STRIDE_FOR_WIDTH_BPPM(COLORBW, 32);
strideW = stride/4;
rgbbuf = malloc(stride * h);
viewport(&grL, 0, COLORBW, 0, h);
window(&grL, (double) 0, (double) COLORBW, (double) 0., (double) 30.0, 0, 0);
pos = rgbbuf;
for(i=0;i<h;i++)
{
colorProb = colorP(PDFPLOT, (float) PIX2USERY(&grL, i));
for(j=0;j<COLORBW;j++)
{
pos = rgbbuf + (i*strideW + j)*4;  
pixel = (guint32 *)pos;
*pixel = (0 << 24) | (colorProb->R << 16) | (colorProb->G << 8) | (colorProb->B);
}
}
cBar = cairo_image_surface_create_for_data(rgbbuf, CAIRO_FORMAT_RGB24, COLORBW, h, stride);
cBar2 = cairo_image_surface_create(CAIRO_FORMAT_RGB24, w, h);
cr = cairo_create(cBar2);
cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
cairo_paint(cr);
cairo_set_source_surface(cr, cBar, 0, 0);
cairo_paint(cr);
cairo_surface_destroy(cBar);
free(rgbbuf);
for(i=0;i<31;i+=6)
{
y = USER2PIXY(&grL, (double) i);
switch(i)
{
case 0:
str = g_strdup_printf("%d%%", i);
y1 = y - fontSIZEPCairo(cr, str, AXISF, HEIGHT);
y -=1;
break;
case 30:
str = g_strdup_printf("%d+", i);
y1 = y -2;
break;
default:
str = g_strdup_printf("%d%%", i);
y1 = y - fontSIZEPCairo(cr, str, AXISF, HEIGHT)/2;
break;
}
fontDrawPCairo(cr, str, COLORBW+1, y1, AXISF, FALSE);
free(str);
}
cairo_destroy(cr);
return cBar2;
}
