#include <unistd.h>
#include <sys/stat.h> 
#include <dirent.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include "pqlx_defs.h"
#define FLUSHMSG "FLUSH YOURSELF..."
enum { 
LOG_CACHE,
LOG_WRITE,
TTLLOGWRITESTATES
};
enum {
READFD, 
WRITEFD, 
TTLPIPEFDS
};
#define TTLLOGSTREAMS (TTLSRVRLOGTYPES-1)
static GIOChannel *channel[TTLLOGSTREAMS];   
static void  *loggingFuncs[TTLLOGCMDS];   
static GSList *cacheFuncs[TTLLOGFUNCS];   
static GSList *writeFuncs[TTLLOGFUNCS];   
static GSList *outputFuncs[TTLLOGWRITESTATES]; 
static LOGFILES *files[TTLLOGSTREAMS];    
static FILE  *streams[TTLLOGSTREAMS];   
static GSList *logLists[TTLLOGSTREAMS];   
static GMainLoop *loop;
static GAsyncQueue *commsQ[TTLLOGSTREAMS];   
static int  writeState;       
static gboolean inited=FALSE;
static char  *dbName;
static char  *procName;
void TIMESTR(char *timeStr);
static char *errStrings[TTLHANDLEDERRTYPES] = {
MYSQL_TBL_CRASH_MSG,
MYSQL_TBL_REPAIR_MSG,
EVALRESP_EXEC_CONT_MSG
};
static int parseErrorMsg(char *msg)
{
int i, err = NONE;
for (i = 0; err == NONE && i < TTLHANDLEDERRTYPES; i++)
{
if (g_strrstr(msg, errStrings[i]))
{
err = i;
}
}
return err; 
}
static void *logLoop(gpointer l)
{ 
loop = g_main_loop_new(NULL, FALSE);
usleep(100);
gdk_threads_enter();
g_main_loop_run(loop);
g_main_loop_unref(loop);
gdk_threads_leave();
loop = NULL;
return NULL;
}
LOGFILES *openLogFile(char *prefix, char *suffix, char *dbName, gchar *timeStr)
{
gchar *logFN, *logDir;
gchar year[5], jday[4], hr[3], mn[3];
FILE *fp;
LOGFILES *logFile;
logDir = getenv(ENVLOG);
if (!logDir)
{
fprintf(stderr, "Environment Variable %s not set, please define and re-execute.\n", ENVLOG);
return(NULL);
}
sscanf(timeStr, "%s %s %2s:%2s", year, jday, hr, mn);
if (dbName)
logFN = g_strdup_printf("%s/%s.%s.%s.%s.%s%s.%s", logDir, prefix, dbName, year, jday, hr, mn, suffix);
else
logFN = g_strdup_printf("%s/%s.%s.%s.%s%s.%s", logDir, prefix, year, jday, hr, mn, suffix);
if ((fp = fopen(logFN, "w")) == NULL)
{
fprintf(stderr, "Unable to open log file %s for output, cannot continue.\n", logFN);
free(logFN);
return(NULL);
}
logFile = calloc(1, sizeof(LOGFILES));
logFile->fileName = logFN; 
logFile->fp = fp;
return(logFile);
}
static void _logToDB(int type, char *msg)
{ 
static SQLDI *sqldi;
static LOGMSG logMsg;
if (!sqldi)
{
memset(&logMsg, 0, sizeof(LOGMSG));
logMsg.keys.dbName = dbName;
sqldi = calloc(1, sizeof(SQLDI));
sqldi->args.threaded = TRUE;
sqldi->args.sqr = LOGCONN;
sqldi->args.dataEvent = INSERT_DATA;
sqldi->descr.tableData = LOGMSGTBL;
sqldi->data = &logMsg;
}
logMsg.data.type = type;
logMsg.data.msg = msg;
db_request(sqldi);
}
static void _logToDisk(int type, char *msg)
{ 
static char timeStr[50];
TIMESTR(timeStr); 
fprintf(files[type]->fp, "%s%s\n", timeStr, msg);
fflush(files[type]->fp);
}
static void _logToCache(int type, char *msg)
{ 
logLists[type] = g_slist_append(logLists[type], g_strdup(msg));
}
static void _logOutput()
{ 
int  i;
void (*flushFunc)() = loggingFuncs[LOG_FLUSH];
(*flushFunc)();
writeState = LOG_WRITE;
for (i = 0; i < TTLLOGSTREAMS; i++)
{
GSList *iter;
char *msg;
for (iter = logLists[i];
iter;
iter = g_slist_next(iter))
{
msg = iter->data;
fprintf(streams[i], "%s\n", msg);
free(msg);
}
g_slist_free(logLists[i]);
logLists[i] = NULL;
}
}
static void _logShutdown()
{ 
int i;
struct stat file_stat;
void (*flushFunc)() = loggingFuncs[LOG_FLUSH];
switch (writeState)
{
case LOG_WRITE:
(*flushFunc)();
break;
case LOG_CACHE:  
writeFuncs[LOG_WRITEFUNC] = g_slist_remove(writeFuncs[LOG_WRITEFUNC], _logToDB);
_logOutput();
(*flushFunc)();
break;
}
if (loop)
g_main_loop_quit(loop);
fclose(files[MSGTYPE_LOG]->fp);
fclose(files[MSGTYPE_ERR]->fp);
for (i = 0; i < TTLLOGSTREAMS; i++)
{ 
if (stat(files[i]->fileName, &file_stat) == -1) 
{ 
continue;
}
if (file_stat.st_size == 0)
{ 
remove(files[i]->fileName);
}
}
}
static gboolean msgAsCMD(int type, char *msg)
{ 
if (!strcmp(msg, FLUSHMSG))
{
g_async_queue_push(commsQ[type], g_strdup("FLUSHED"));
return TRUE; 
}
return FALSE;  
}
static void msgProcess(int type, char *msg)
{
gboolean output = TRUE;
char *writeMsg;
GSList *iter;
void (*writeFunc)(int, char*);
if (!strlen(msg))    
return;
if (msgAsCMD(type, msg))  
return;
if (g_strrstr(msg, "\""))
{ // must convert all double-quote (") to single quote (') for output to DB
strReplace(msg, writeMsg, "\"", "'");
}
else
{
writeMsg = g_strdup(msg);
}
switch (type)
{
case MSGTYPE_ERR: // process an error message
{ // follow up any errors where possible
int err = parseErrorMsg(writeMsg);
switch (err)
{
case EVALRESP_EXEC_CONT: // message we don't care about, do not output
output = FALSE;
break;
case MYSQL_TBL_CRASH:
case MYSQL_TBL_REPAIR:
{ // handle crashed tables error
SQLDI *sqldi;
char *tblName, *ptr1, *ptr2, *ptr3;
int  len;
// extract the table name
ptr1 = g_strstr_len(writeMsg, -1, "'");
ptr1 += 3;  // remove the "./" from the table name
ptr2 = g_strstr_len(ptr1, -1, "'");
len = (long long) ptr2 - (long long) ptr1;
len -= ((type == MYSQL_TBL_REPAIR) ? 4 : 0);
ptr3 = g_strndup(ptr1, len);
strReplace(ptr3, tblName, "/", ".");
free(ptr3);
fprintf(stderr, "Automatically Repairing Crashed MYSQL Table (%s)\n",
tblName);
sqldi = calloc(1, sizeof(SQLDI));
sqldi->args.callType = ASYNC;
sqldi->args.dataEvent = SYSTEM_DATA;
sqldi->descr.tableData = SYSTBLREPAIR;
sqldi->data = tblName;
db_request(sqldi);  
}
break;
}
}
break;
}
for (iter = outputFuncs[writeState];
output && iter;
iter = g_slist_next(iter))
{
writeFunc = iter->data;
(*writeFunc)(type, writeMsg);
}
g_free(writeMsg);
}
static gboolean outputHandle(GIOChannel *source, GIOCondition condition, gpointer t)
{
int  type = GPOINTER_TO_INT(t);
if (condition == G_IO_IN)
{
static gchar buf[1000];
static char *msgPart[TTLLOGSTREAMS]={NULL, NULL};
int  i, lastMsg;
GError *error=NULL;
gsize  bytesRead;
gchar **msgs=NULL, *msg;
memset(buf, 0, sizeof(buf));
g_io_channel_read_chars(source, buf, sizeof(buf)-1, &bytesRead, &error);
msgs = g_strsplit(buf, "\n", -1);
if (msgPart[type])
{ 
char *newMsg;
newMsg = g_strconcat(msgPart[type], msgs[0], NULL);
free(msgPart[type]); free(msgs[0]);
msgs[0] = newMsg;
msgPart[type] = NULL;
}
for(lastMsg = 0;msgs[lastMsg]; lastMsg++); lastMsg--;
if (strlen(msgs[lastMsg]))
{ 
msgPart[type] = msgs[lastMsg];
msgs[lastMsg] = g_strdup("");
}
for(i = 0; msgs[i]; i++)
{
msg = msgs[i];
msgProcess(type, msg);
}
g_strfreev(msgs);
}
return TRUE;
}
static void _logFlush()
{
char *nil;
int  i;
gboolean mainL = (loop == NULL) ? FALSE : TRUE;
switch (mainL)
{
case FALSE:  
for (i = 0; i < TTLLOGSTREAMS; i++)
{
g_io_channel_flush(channel[i], NULL);
}
while (gtk_events_pending())  
gtk_main_iteration();
break;
case TRUE: 
for (i = 0; i < TTLLOGSTREAMS; i++)
{
fprintf(streams[i], "\n%s\n", FLUSHMSG);
g_io_channel_flush(channel[i], NULL);
nil = g_async_queue_pop(commsQ[i]);  
free(nil);
}
break;
}
}
static gboolean _logInit(char *execTime)
{
int  outPipes[TTLLOGSTREAMS][TTLPIPEFDS];
char *logSuffix[TTLLOGSTREAMS] = { "log", "err" };
int  *p, i;
for (i = 0; i < TTLLOGSTREAMS; i++)
{ 
if (!(files[i] = openLogFile(procName, logSuffix[i], dbName, execTime)))
{
fprintf(stderr, "Unable to open output File , quitting...\n");
return(FALSE);
}
p = outPipes[i];
if (pipe(p) < 0)
{
fprintf(stderr, "Unable to create pipe to redirect stream, quitting...\n");
return(FALSE);
}
if (dup2(p[WRITEFD], fileno(streams[i])) < 0)
{
fprintf(stderr, "Unable to dup2() a stream to a pipe, quitting...\n");
return(FALSE);
}  
close(p[WRITEFD]);
setlinebuf(streams[i]);
channel[i] = g_io_channel_unix_new(p[READFD]);
g_io_channel_set_encoding(channel[i], NULL, NULL); 
g_io_channel_set_buffered(channel[i], FALSE); 
g_io_add_watch(channel[i], G_IO_IN,(GIOFunc) outputHandle, GINT_TO_POINTER(i));
}
return(TRUE);
}
gboolean logging(int cmd)
{ 
void (*logFunc)();
if (!inited)
return FALSE;
if (!(cmd >= 0 &&
cmd < TTLLOGCMDS))
return FALSE;
logFunc = loggingFuncs[cmd];
(*logFunc)();
return TRUE;
}
gboolean loggingInit(char *procN, char *dbN, char *execTime, gboolean mainLoop, gboolean initCaching)
{
int  i;
if (!g_thread_supported()) 
{
g_thread_init(NULL);
gdk_threads_init();
}
streams[MSGTYPE_LOG] = stdout;
streams[MSGTYPE_ERR] = stderr;
for (i=0;i<TTLLOGSTREAMS;i++)
commsQ[i] = g_async_queue_new();
loggingFuncs[LOG_OUTPUT] = _logOutput;
loggingFuncs[LOG_FLUSH] = _logFlush;
loggingFuncs[LOG_SHUTDOWN] = _logShutdown;
procName = g_strdup(procN);
dbName = dbN ? g_strdup(dbN) : NULL;
writeState = LOG_CACHE;
cacheFuncs[LOG_WRITEFUNC] = g_slist_append(cacheFuncs[LOG_WRITEFUNC], _logToCache);
writeFuncs[LOG_WRITEFUNC] = g_slist_append(writeFuncs[LOG_WRITEFUNC], _logToDisk);
if (dbName &&      
strcmp(dbName, PQLXMETADB))  
writeFuncs[LOG_WRITEFUNC] = g_slist_append(writeFuncs[LOG_WRITEFUNC], _logToDB);
outputFuncs[LOG_CACHE] = cacheFuncs[LOG_WRITEFUNC];
outputFuncs[LOG_WRITE] = writeFuncs[LOG_WRITEFUNC];
if (!_logInit(execTime))
return FALSE;
if (mainLoop)
{
g_thread_create(logLoop, NULL, TRUE, NULL);
}
else
{ 
loop = NULL;
}
if (!initCaching)
_logOutput();
inited = TRUE;
return TRUE;
}
