#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <float.h>
#include <evresp.h>
#include "whiten.h"
#include "getpzfr.h"
#define PI 3.1415926535897932384626433
int next2 (int value);
static double taper (double freq, double fqh, double fql);
static int findtaper (double *taperfreq, double *xreal, double *ximag,
int nfreqs, double delfreq);
static int fft (double real[], double imag[], int nfft, int direction);
static int
convolve (float data[], int npts, double delta,
double *xreal, double *ximag, int nfreqs, int nfft,
double taperfreq[], int *prewhiten, int deconvflag,
int verbose)
{
int i, j;
int wlwarning = 0;
double whitecoef[21];
double delfreq, denr, fac, freq;
double *sreal, *simag;
delfreq = 1.0 / (delta * nfft);
if ( verbose > 2 )
{
fprintf (stderr, "Convolution parameters:\n");
fprintf (stderr, " npts: %d\n", npts);
fprintf (stderr, " nfft: %d\n", nfft);
fprintf (stderr, " nfreqs: %d\n", nfreqs);
fprintf (stderr, " delfreq: %g\n", delfreq);
if ( taperfreq )
fprintf (stderr, " Tapering: %g/%g => %g/%g\n",
taperfreq[0], taperfreq[1], taperfreq[2], taperfreq[3]);
else
fprintf (stderr, " Tapering: none\n");
}
if ( (sreal = (double *) malloc (2*nfft*sizeof(double))) == NULL )
{
fprintf (stderr, "convolve(): Error allocating memory\n");
return -1;
}
if ( (simag = (double *) malloc (2*nfft*sizeof(double))) == NULL )
{
fprintf (stderr, "convolve(): Error allocating memory\n");
return -1;
}
if ( *prewhiten > 0 )
{
if ( verbose > 1 )
fprintf (stderr, "Prewhitening the data\n");
if ( whiten (data, npts, prewhiten, whitecoef) )
{
fprintf (stderr, "convolve(): Error prewhitening the data\n");
return -1;
}
}
if ( taperfreq && (taperfreq[0] == -1.0 || taperfreq[1] == -1.0 ||
taperfreq[2] == -1.0 || taperfreq[3] == -1.0) )
{
if ( findtaper (taperfreq, xreal, ximag, nfreqs, delfreq) )
{
fprintf (stderr, "Error determing taper parameters\n");
return -1;
}
if ( verbose )
fprintf (stderr, "Final tapering: %g/%g => %g/%g\n",
taperfreq[0], taperfreq[1], taperfreq[2], taperfreq[3]);
}
if ( deconvflag )
{
for ( i = 0; i < nfreqs; i++ )
{
denr = (xreal[i]*xreal[i] + ximag[i]*ximag[i]);
if ( denr <= FLT_MIN )
{
xreal[i] = 0.0;
ximag[i] = 0.0;
wlwarning++;
}
else
{
denr = 1.0 / denr;
xreal[i] = xreal[i] * denr;
ximag[i] = -ximag[i] * denr;
}
}
if ( verbose && wlwarning > 0 )
fprintf (stderr, "WARNING: Water level reached for %d coeffients during response inversion\n",
wlwarning);
}
for ( i = 0; i < nfreqs; i++ )
{
freq = i * delfreq;
if ( taperfreq )
fac = delfreq * (taper (freq, taperfreq[1], taperfreq[0]) *
taper (freq, taperfreq[2], taperfreq[3]));
else
fac = delfreq;
xreal[i] *= fac;
ximag[i] *= fac;
}
for ( i = 0; i < npts; i++ )
{
sreal[i] = data[i] * delta;
simag[i] = 0.0;
}
for ( i = npts; i < nfft; i++ )
{
sreal[i] = 0.0;
simag[i] = 0.0;
}
if ( verbose > 1 )
fprintf (stderr, "Calculating FFT\n");
fft ( sreal, simag, nfft, 1 );
for ( i = 0; i < nfreqs; ++i )
{
double tempR = xreal[i]*sreal[i] - ximag[i]*simag[i];
double tempI = xreal[i]*simag[i] + ximag[i]*sreal[i];
sreal[i] = tempR;
simag[i] = tempI;
if ( i > 0 && i < (nfreqs-1) )
{
j = nfft - i;
sreal[j] = tempR;
simag[j] = -tempI;
}
}
if ( verbose > 1 )
fprintf (stderr, "Calculating inverse FFT\n");
fft ( sreal, simag, nfft, -1 );
for ( i = 0; i < npts; ++i )
data[i] = sreal[i];
if ( *prewhiten > 0 )
{
if ( verbose > 1 )
fprintf (stderr, "Dewhitening the data\n");
if ( dewhiten (data, npts, *prewhiten, whitecoef) )
{
fprintf (stderr, "convolve(): Error dewhitening data\n");
return -1;
}
}
if ( sreal )
free (sreal);
if ( simag )
free (simag);
return 0;
} 
int
convolve_sac (float data[], int npts, double delta, double taperfreq[], 
int *prewhiten, int deconvflag, char *sacpzfilename,
int verbose)
{
int retval;
int nfft;
int nfreqs;
double delfreq;
double *xreal, *ximag;
nfft = next2 (npts);
nfreqs = nfft/2 + 1;
delfreq = 1.0 / (delta * nfft);
if ( (xreal = (double *) malloc (2*nfreqs*sizeof(double))) == NULL )
{
fprintf (stderr, "convolve_sac(): Error allocating memory\n");
return -1;
}
if ( (ximag = (double *) malloc (2*nfreqs*sizeof(double))) == NULL )
{
fprintf (stderr, "convolve_sac(): Error allocating memory\n");
return -1;
}
if ( verbose > 1 )
fprintf (stderr, "Determinig frequency response from SAC Poles & Zeros\n");
if ( getpzfr (nfreqs, delfreq, xreal, ximag, sacpzfilename) )
{
fprintf (stderr, "convolve_sac(): Error determining frequency response for: %s\n",
sacpzfilename);
return -1;
}
retval = convolve (data, npts, delta, xreal, ximag, nfreqs, nfft,
taperfreq, prewhiten, deconvflag, verbose);
if ( xreal )
free (xreal);
if ( ximag )
free (ximag);
return retval;
} 
int
convolve_resp (float data[], int npts, double delta,
char *net, char *sta, char *loc, char *chan,
int startstage, int stopstage,
char *units, time_t resptime, int usedelay,
double taperfreq[], int *prewhiten,
int deconvflag, char *respfilename,
int totalsensflag, int verbose)
{
int i;
int retval;
int nfft;
int nfreqs;
double delfreq; 
double *xreal, *ximag;
struct tm *rtime;
char datetime[20];
struct response *resp = NULL;
double *freqs = NULL;
nfft = next2 (npts);
nfreqs = nfft/2 + 1;
delfreq = 1.0 / (delta * nfft);
if ( (xreal = (double *) malloc (2*nfreqs*sizeof(double))) == NULL )
{
fprintf (stderr, "convolve_resp(): Error allocating memory\n");
return -1;
}
if ( (ximag = (double *) malloc (2*nfreqs*sizeof(double))) == NULL )
{
fprintf (stderr, "convolve_resp(): Error allocating memory\n");
return -1;
}
rtime = gmtime(&resptime);
snprintf (datetime, sizeof(datetime), "%04d,%03d,%02d:%02d:%02d",
rtime->tm_year+1900, rtime->tm_yday,
rtime->tm_hour, rtime->tm_min, rtime->tm_sec);
if ( (freqs = (double *) malloc(nfreqs * sizeof(double))) == NULL )
{
fprintf (stderr, "convolve_resp(): Cannot allocate memory\n");
return -1;
}
for (i=0 ; i < nfreqs ; i++)
{
freqs[i] = i * delfreq;
}
if ( usedelay )
use_delay (TRUE);
else
use_delay (FALSE);
if ( verbose > 1 )
fprintf (stderr, "Determinig frequency response from SEED RESP\n");
resp = evresp_itp (sta, chan, net, loc, datetime, units, respfilename, freqs, nfreqs,
"cs", (verbose)?"-v":NULL, startstage, stopstage, 0, 0, 0, 0.0, totalsensflag);
if ( ! resp )
{
fprintf (stderr, "convolve_resp(): Error with evresp()\n");
return -1;
}
else if ( verbose )
{
fprintf (stderr, "Found response for %s.%s.%s.%s at %s (stages %d to %d)\n",
resp->network, resp->station, resp->locid, resp->channel,
datetime, startstage, stopstage);
}
for (i=0 ; i < nfreqs ; i++)
{
xreal[i] = resp->rvec[i].real;
ximag[i] = resp->rvec[i].imag;
}
retval = convolve (data, npts, delta, xreal, ximag, nfreqs, nfft,
taperfreq, prewhiten, deconvflag, verbose);
if ( resp )
free_response (resp);
if ( freqs )
free (freqs);
if ( xreal )
free (xreal);
if ( ximag )
free (ximag);
return retval;
} 
int
next2 (int value)
{
int npower = 2;
int nextpow2 = 0;
while ( nextpow2 < value )
{
nextpow2 = (unsigned int) 1 << npower++;
if ( npower > 100 )
return 0;
}
return (nextpow2);
} 
static double
taper (double freq, double fqh, double fql)
{
const double twopi = 6.283185307179586;
double dblepi;
double taper_v = 0;
dblepi = 0.5*twopi;
if ( fql > fqh )
{
if ( freq < fqh )
taper_v = 1.0;
if ( freq >= fqh && freq <= fql )
taper_v = 0.5 * (1.0 + cos (dblepi*(freq - fqh)/(fql - fqh)));
if ( freq > fql )
taper_v = 0.0;
}
else if ( fqh > fql )
{
if ( freq < fql )
taper_v = 0.0;
if ( freq >= fql && freq <= fqh )
taper_v = 0.5 * (1.0 - cos (dblepi*(freq - fql)/(fqh - fql)));
if ( freq > fqh )
taper_v = 1.0;
}
else
{
fprintf (stderr, "taper(): Invalid window specified\n");
}
return (taper_v);
} 
static int
findtaper (double *taperfreq, double *xreal, double *ximag,
int nfreqs, double delfreq)
{
int i;
double amp;
double maxamp = 0.0;
double minamp = 0.0;
double lowestfreq = 0.0;
double highestfreq = 0.0;
if ( taperfreq[0] == -1.0 )
{
maxamp = 0.0;
minamp = 0.0;
lowestfreq = 0.0;
for ( i = (nfreqs-1); i > 0; i-- )
{
amp = sqrt (xreal[i] * xreal[i] + ximag[i] * ximag[i]);
if ( maxamp == 0.0 || amp > maxamp )
{
maxamp = amp;
minamp = amp * 0.01;
}
if ( amp >= minamp )
{
lowestfreq = i * delfreq;
}
}
taperfreq[0] = lowestfreq;
}
if ( taperfreq[1] == -1.0 )
{
taperfreq[1] = lowestfreq * 1.1;
}
if ( taperfreq[3] == -1.0 )
{
maxamp = 0.0;
minamp = 0.0;
highestfreq = 0.0;
for ( i = 0; i < nfreqs-1; i++ )
{
amp = sqrt (xreal[i] * xreal[i] + ximag[i] * ximag[i]);
if ( maxamp == 0.0 || amp > maxamp )
{
maxamp = amp;
minamp = amp * 0.01;
}
if ( amp >= minamp )
{
highestfreq = i * delfreq;
}
}
taperfreq[3] = highestfreq;
}
if ( taperfreq[2] == -1.0 )
{
taperfreq[2] = taperfreq[3] * 0.95;
}
return 0;
} 
static int
fft (double real[], double imag[], int nfft, int direction)
{
static double *sintab1 = 0;
static double *sintab2 = 0;
static int sinallocated = 0;
int i, step, j, m, mmax;
int n2, k;
double sinphi, phi, coef_i, coef_r, sci, scr;
double tmpr, tmpi, pi2;
double treal, timag;
if ( ! real && ! imag )
{
if ( sintab1 )
free (sintab1);
if ( sintab2 )
free (sintab2);
sintab1 = 0;
sintab2 = 0;
sinallocated = 0;
}
if ( direction != 1 && direction != -1 )
{
fprintf (stderr, "fft(): direction argument must be 1 (forward) or -1 (inverse)\n");
return -1;
}
m = 2;
while ( m < nfft )
{
m = m << 1;
}
if ( m != nfft )
{
fprintf (stderr, "fft(): nfft (%d) is not a perfect power of two\n", nfft);
return -1;
}
if ( sinallocated != nfft )
{
if ( (sintab1 = (double *) realloc (sintab1, nfft * sizeof(double))) == NULL )
{
fprintf (stderr, "Error allocating memory\n");
return -1;
}
if ( (sintab2 = (double *) realloc (sintab2, nfft * sizeof(double))) == NULL )
{
fprintf (stderr, "Error allocating memory\n");
return -1;
}
pi2 = 0.5 * PI;
mmax = 1;
k = 0;
while ( mmax < nfft )
{
step = mmax << 1;
phi = pi2 / (double) mmax;
sintab1[k] = sin (phi);
sintab2[k] = sin (phi * 2.0);
k++;
m = 0;
while ( m < mmax )
{
i = m;
while ( i < nfft )
{
j = i + mmax;
i += step;
}
m++;
}
mmax = step;
}
sinallocated = nfft;
}
n2 = nfft >> 1;
j = 0;
i = 0;
while ( i < nfft )
{
if ( i < j )
{
treal = real[j]; timag = imag[j];
real[j] = real[i]; imag[j] = imag[i];
real[i] = treal; imag[i] = timag;
}
m = n2;
while ( (j > m-1) && (m >= 2) )
{ j -= m; m = m >> 1; }
j += m;
i++;
}
mmax = 1;
k = 0;
while ( mmax < nfft )
{
step = mmax << 1;
sinphi = -1 * (double)direction * sintab1[k];
sci = -1 * (double)direction * sintab2[k];
scr = -2.0 * sinphi * sinphi;
k++;
m = 0;
coef_r = 1.0;
coef_i = 0.0;
while ( m < mmax )
{
i = m;
while ( i < nfft )
{
j = i + mmax;
tmpr = (coef_r * real[j]) - (coef_i * imag[j]);
tmpi = (coef_r * imag[j]) + (coef_i * real[j]);
real[j] = real[i] - tmpr;
imag[j] = imag[i] - tmpi;
real[i] += tmpr;
imag[i] += tmpi;
i += step;
}
tmpr = coef_r;
m++;
coef_r = coef_r + (coef_r * scr) - (coef_i * sci);
coef_i = coef_i + (coef_i * scr) + (tmpr * sci);
}
mmax = step;
}
return 0;
} 
