#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include "complex.h"
#define MAXPOLES 30
#define MAXZEROS 30
void calcfr (int nfreq, double delfreq, double constant,
int nzero, complexf zeros[],
int npole, complexf poles[],
double xreal[], double ximag[]);
int
getpzfr (int nfreq, double delfreq, double xreal[], double ximag[],
char *pzfilename)
{
int idx;
FILE *fp;
char line[1024];
char *lp;
char reading = 0; 
int linecount = 0;
complexf zeros[MAXZEROS];
complexf poles[MAXPOLES];
double constant;
int rdconstant = 0;
int nzeros = 0;
int zeroidx = 0;
int rdzeros = 0;
int npoles = 0;
int poleidx = 0;
int rdpoles = 0;
double reald;
double imagd;
for ( idx = 0; idx < nfreq; idx++ )
{
xreal[idx] = 1.0e0;
ximag[idx] = 0.0e0;
}
constant = 1.0;
for ( idx = 0; idx < MAXZEROS; idx++ )
{
zeros[idx] = flttocmplx (0.0, 0.0);
}
for ( idx = 0; idx < MAXPOLES; idx++ )
{
poles[idx] = flttocmplx (0.0, 0.0);
}
if ( (fp = fopen(pzfilename, "rb")) == NULL )
{
fprintf (stderr, "Error opening %s: %s\n",
pzfilename, strerror(errno));
return -1;
}
while ( fgets (line, sizeof(line), fp) )
{
lp = line;
linecount++;
while ( isspace(*lp) ) lp++;
if ( *lp == '*' || *lp == '#' ) 
continue;
if ( strlen (lp) == 0 ) 
continue;
else if ( ! strncasecmp ("ZEROS", lp, 5) )
{
if ( rdzeros )
{
fprintf (stderr, "%s: Already read a ZEROS line, skipping\n",
pzfilename);
reading = 0;
continue;
}
if ( (sscanf (lp+5, " %d\n", &nzeros)) != 1 )
{
fprintf (stderr, "Error parsing ZEROS (line %d): %s\n",
linecount, lp);
return -1;
}
if ( nzeros > MAXZEROS )
{
fprintf (stderr, "Number of ZEROS (%d) cannot be more than %d\n",
nzeros, MAXZEROS);
return -1;
}
rdzeros = 1;
reading = 1;
continue;
}
else if ( ! strncasecmp ("POLES", lp, 5) )
{
if ( rdpoles )
{
fprintf (stderr, "%s: Already read a POLES line, skipping\n",
pzfilename);
reading = 0;
continue;
}
if ( (sscanf (lp+5, " %d\n", &npoles)) != 1 )
{
fprintf (stderr, "Error parsing POLES (line %d): %s\n",
linecount, lp);
return -1;
}
if ( nzeros > MAXPOLES )
{
fprintf (stderr, "Number of POLES (%d) cannot be more than %d\n",
npoles, MAXPOLES);
return -1;
}
rdpoles = 1;
reading = 2;
continue;
}
else if ( ! strncasecmp ("CONSTANT", lp, 8) )
{
if ( rdconstant )
{
fprintf (stderr, "%s: Already read a CONSTANT line, skipping\n",
pzfilename);
reading = 0;
continue;
}
if ( (sscanf (lp+8, " %lf\n", &constant)) != 1 )
{
fprintf (stderr, "Error parsing POLES (line %d): %s\n",
linecount, lp);
return -1;
}
rdconstant = 1;
reading = 0;
continue;
}
else if ( reading == 1 ) 
{
if ( zeroidx >= MAXZEROS )
{
fprintf (stderr, "Too many zeros in %s, max: %d\n",
pzfilename, MAXZEROS);
return -1;
}
if ( (sscanf (lp, "%lf %lf", &reald, &imagd)) != 2 )
{
fprintf (stderr, "Error parsing ZERO (line %d): %s\n",
linecount, lp);
return -1;
}
zeros[zeroidx++] = flttocmplx (reald, imagd);
continue;
}
else if ( reading == 2 ) 
{
if ( poleidx >= MAXPOLES )
{
fprintf (stderr, "Too many poles in %s, max: %d\n",
pzfilename, MAXPOLES);
return -1;
}
if ( (sscanf (lp, "%lf %lf", &reald, &imagd)) != 2 )
{
fprintf (stderr, "Error parsing POLE (line %d): %s\n",
linecount, lp);
return -1;
}
poles[poleidx++] = flttocmplx (reald, imagd);
continue;
}
} 
if ( fp )
fclose (fp);
if ( poleidx != npoles )
{
fprintf (stderr, "Read %d POLES but was expecting %d\n", poleidx, npoles);
return -1;
}
fprintf (stderr, "Calculating frequency response for poles and zeros\n");
calcfr (nfreq, delfreq, constant, nzeros, zeros, npoles, poles, xreal, ximag);
return 0;
} 
void
calcfr (int nfreq, double delfreq, double constant,
int nzero, complexf zeros[],
int npole, complexf poles[],
double xreal[], double ximag[])
{
double delomg, fac, omega;
double ti, ti0, tid, tin;
double tr, tr0, trd, trn;
int idx, jdx;
delomg = 6.283185307179586 * delfreq;
for ( jdx = 0; jdx < nfreq; jdx++ )
{
omega = delomg * jdx;
trn = 1.0;
tin = 0.0;
if ( nzero > 0 )
{
for ( idx = 0; idx < nzero; idx++ )
{
tr = -1.0 * cmplxreal (zeros[idx]);
ti = omega - cmplximag (zeros[idx]);
tr0 = trn*tr - tin*ti;
ti0 = trn*ti + tin*tr;
trn = tr0;
tin = ti0;
}
}
trd = 1.0;
tid = 0.0;
if ( npole > 0 )
{
for ( idx = 0; idx < npole; idx++ )
{
tr = -1.0 * cmplxreal (poles[idx]);
ti = omega - cmplximag (poles[idx]);
tr0 = trd*tr - tid*ti;
ti0 = trd*ti + tid*tr;
trd = tr0;
tid = ti0;
}
}
fac = constant / (trd*trd + tid*tid);
xreal[jdx] = fac * (trn*trd + tin*tid);
ximag[jdx] = fac * (trd*tin - trn*tid);
}
return;
} 
