#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "whiten.h"
#define NCMAX 12
#define MAX(A,B) ((A)>(B)?(A):(B)) 
#define MIN(A,B) ((A)>(B)?(B):(A)) 
int
whiten (float data[], int npts, int *order, double coef[])
{
int idx, jdx, j2, kdx, kb, torder;
double at, q;
double ref[NCMAX+1];
double sa[NCMAX + 1] ;
double pefcoef[NCMAX];
if ( *order < 1 || *order > NCMAX )
{
fprintf (stderr, "Error, prewit(): Predictor order out of bounds (1-12): %d", *order);
return -1;
}
LPCautocorr (data, npts, *order, coef, ref);
torder = *order;
if ( torder != *order )
{
for ( idx = 0; idx < torder; idx++ )
{
sa[idx] = ref[idx];
}
for ( jdx = 1; jdx < torder; jdx++ )
{
j2 = (jdx+1)/2;
q = ref[jdx];
for( kdx = 0; kdx < j2; kdx++ )
{
kb = jdx - kdx;
at = sa[kdx] + q*sa[kb];
sa[kb] = sa[kb] + q*sa[kdx];
sa[kdx] = at;
}
}
for ( idx = 0; idx < torder; idx++ )
{
coef[idx + 1] = sa[idx];
}
for ( idx = torder + 1; idx < (*order + 1); idx++ )
{
coef[idx] = 0.0;
}
coef[0] = 1.0;
*order = torder;
}
for ( idx = 0; idx < *order; idx++ )
{
pefcoef[idx] = -coef[idx];
}
if ( applypef (data, npts, pefcoef, *order, data) )
return -1;
return 0;
} 
int
dewhiten (float data[], int npts, int order, double coef[])
{
long int i;
double precoef[NCMAX];
if ( order < 1 || order > 12 )
{
fprintf (stderr, "Error, dewhiten(): Predictor order out of bounds (1-12): %d", order);
return -1;
}
for ( i = 0; i < order; i++ )
{
precoef[i] = -coef[i];
}
if ( applypf (data, npts, precoef, order, data) )
return -1;
return 0;
} 
double
LPCautocorr (float data[], int npts, int order,
double lpc[], double ref[])
{
double corr[NCMAX];
double sum;
double r;
double error;
double ctemp;
int idx, jdx;
int aclag = order+1;
while (aclag--)
{
for (idx = aclag, sum = 0; idx < npts; idx++)
sum += data[idx] * data[idx-aclag];
corr[aclag] = sum;
}
if ( corr[0] == 0.0 ) {
for (idx = 0; idx < order; idx++)
ref[idx] = 0.0;
return 0.0;
}
error = corr[0];
for (idx = 0; idx < order; idx++)
{
r = -corr[idx + 1];
for (jdx = 0; jdx < idx; jdx++)
r -= lpc[jdx] * corr[idx - jdx];
ref[idx] = r /= error;
lpc[idx] = r;
for (jdx = 0; jdx < idx/2; jdx++)
{
ctemp = lpc[jdx];
lpc[jdx] += r * lpc[idx-1-jdx];
lpc[idx-1-jdx] += r * ctemp;
}
if (idx % 2)
lpc[jdx] += lpc[jdx] * r;
error *= 1.0 - r * r;
}
return error;
} 
int
applypef (float data[], int npts, double coef[], int nc, float result[])
{
int bufptr, datptr;
int idx, lsamp, ncmp, nextra;
double e;
float inbuf[2000];
float outbuf[2000];
datptr = 0;
nextra = 2000 - nc;
if ( nextra < 1 )
{
fprintf (stderr, "Error, applypef(): Filter too large: %d\n", nc);
return -1;
}
memset (inbuf, 0, 2000);
memset (outbuf, 0, 2000);
while ( datptr < npts )
{
memmove (inbuf, &inbuf[nextra], nc * sizeof(float));
bufptr = nc;
lsamp = MIN (2000, npts - datptr + nc);
ncmp = lsamp - nc;
memcpy (&inbuf[nc], &data[datptr], ncmp * sizeof(float));
while ( bufptr <= lsamp )
{
e = inbuf[bufptr];
for ( idx = 0; idx < nc; idx++ )
{
e -= coef[idx] * inbuf[bufptr - idx - 1];
}
outbuf[bufptr] = e;
bufptr++;
}
memcpy (&result[datptr], &outbuf[nc], ncmp * sizeof(float));
datptr += ncmp;
}
return 0;
} 
int
applypf (float data[], int npts, double coef[], int nc, float result[])
{
int bufptr, dataptr;
int idx, lsamp, ncmp;
float history[NCMAX];
double eo;
float inbuf[2000];
float outbuf[2000];
if ( nc > NCMAX )
{
fprintf (stderr, "Error, applypf(): Filter too large: %d\n", nc);
return -1;
}
for ( idx = 0 ; idx < 2000 ; idx++ )
inbuf[idx] = outbuf[idx] = 0.0;
for ( idx = 0 ; idx < NCMAX ; idx++ )
history[idx] = 0.0;
dataptr = 0;
while ( dataptr < npts )
{
bufptr = 0;
lsamp = MIN (2000, (npts - dataptr));
ncmp = lsamp;
memcpy (inbuf, &data[dataptr], ncmp * sizeof(float));
while ( bufptr < lsamp )
{
eo = inbuf[bufptr];
for( idx = (nc-1); idx >= 0; idx-- )
{
eo += coef[idx]*history[idx];
if ( idx > 0 )
history[idx] = history[idx-1];
}
outbuf[bufptr] = eo;
history[0] = eo;
bufptr++;
}
memcpy (&result[dataptr], outbuf, ncmp * sizeof(float));
dataptr += ncmp;
}
return 0;
} 
