#!/bin/bash

if [ $# -ne 1 ] ; then
    echo "Must provide MySQL root password to execute"
    echo "usage: "
    echo "\t$0 mysqlRootPassword"
    exit
fi

# set some constants
# major, minor, and micro minimums we require
minMaj=5
minMin=1
minMic=46

# return values
SUCCESS="TRUE"
FAILURE="FAIL-MIN-VERSION"
BADPW="FAIL-BAD-PASSWORD"
NOINSTALL="FAIL-NO-INSTALL"

# multipication factors per version part, 
# used to construct mathematical version number for comparison purposes
(( majF = 10000 ))		# major version factor
(( minF = 100 ))		# minor version factor
(( micF = 1 ))			# micro version factor

# define the mathematical minimum version
(( reqMaj = $minMaj * $majF ))
(( reqMin = $minMin * $minF ))
(( reqMic = $minMic * $micF ))
(( minVer = $reqMaj + $reqMin + $reqMic ))		# mathematical minimum version

# confirm MySQL installed
install=`which mysql`
if [ $install = "" ] ; then
	echo $NOINSTALL
	exit
fi

# get the installed MySQL version
PWRD=$1
sql="select version()"
ver=`echo $sql | mysql -uroot -p$PWRD 2>&1 | grep -v version`
denied=`echo $ver | grep -i denied | wc -l`

# confirm password
if [ $denied -gt 0 ] ; then
	echo $BADPW
	exit
fi

# deconstruct output to mathematical version
ver=`echo $ver | sed s/-.*//`
verS=${ver//./ }
maj=`echo $verS | awk '{ print $1 }'`
min=`echo $verS | awk '{ print $2 }'`
mic=`echo $verS | awk '{ print $3 }'`
(( instMaj = $maj * $majF ))
(( instMin = $min * $minF ))
(( instMic = $mic * $micF ))
(( instVer = $instMaj + $instMin + $instMic ))

# compare installed version vs. minimum necessary
if [ $instVer -lt $minVer ] ; then
	echo $FAILURE
	exit
fi
echo $SUCCESS
