#ifndef __PQL_DATA_STRUCTS_H
#define __PQL_DATA_STRUCTS_H

#include <sys/types.h>
#include <stdio.h>
#include <glib.h>
#include "ptime.h"

typedef struct _HEADINFO
{
	int		data_form, dataSize;
	gboolean	endianSwap;
	double	sampInt;			// msecs/sample
	int		year, day, hour, minute, second, m_secs;
	double	epochs;	// start time in epoch secs, format: epoch Seconds.msec
	double	epochStop;
	double	length;				// in seconds
	int		gainConst;
	float		scale_fac;
	char		*descr1, *descr2, *longFN, *shortFN;
	char		network[10], station[10], location[10], channel[10], sensorID[10];
	int64_t	MSstartTime;	// start time in mini-seed format, only used for multi-plexed data reading
} HEADINFO;

typedef struct {
	char		*data;					// data buffer
	struct {
		int	numSamples;
		int	data_form;
		double	sampInt;
		double	secsPsamp;				// seconds/sample
	} head;
} DATAINFO;

typedef	struct _TRACEINFO 
{
	struct {
		int original;					// original position at startup
		int current;					// current position in the list
	}	position;						// position # in our list
	char		*fileName;				// Filename of trace
  	int			fileType; 				// DATA TYPE
	HEADINFO	head;					// PQL header info
	DATAINFO	dataInfo;				// buffer of data and stats
} TRACEINFO;

typedef struct _traceInfo
{
	TRACEINFO	data;
} traceInfo;

GSList *identifyTrace(char *);
int readTrace(traceInfo *trace);
void revmem(u_char *buf, int size);
void passcal_etoh(register struct ptime *t, double epoch);
int get_julday(int month, int day, int year);
gint32 passcal_htoe(struct ptime *t, gboolean convertJul);
void getFiles(char *dirFile);
void freeData2(traceInfo *trc);
int testByteOrder();

// terribly useful macros
// generic data extractor
// in order to use this, the names of the data pointers 
// must be set to point to the data array, and be exactly named, as:
// gint32	*ptr;	to integer (32-bit) data
// short	*sptr;	to short data
// float	*lptr;	to float data
// double	*dptr;	to double data
#define DATAP(b, c, d) {switch (b) { \
							case BIT32: d=ptr[c];break;		\
							case BIT16: d=sptr[c];break;		\
							case MY_FLOAT: d=lptr[c];break;	\
							case MY_DOUBLE: d=dptr[c];break;	\
							}	\
						}

#endif
