#include <sys/stat.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <unistd.h>
#include <glib.h>
#include "ezxml.h"
#include "ref2sac.h"
#include "sac.h"

GHashTable	*chnDefTbl;
GSList	*fileList;
char	*inDir, *outDir;

static int writeSAC(traceInfo *trc)
{
	struct sac SH;
	float	*data;
	CHNDEF	*chnDef;
	char	*fileName, **chan, *chnStr, *dir1, *dir2, *file1;
	int		dataFormat = trc->data.dataInfo.head.data_form;
	gint32	*ptr = (gint32 *) trc->data.dataInfo.data;
	short	*sptr = (short *) trc->data.dataInfo.data;
	float	*lptr = (float *) trc->data.dataInfo.data;
	double	*dptr = (double *) trc->data.dataInfo.data;
	FILE	*fp;
	int		i;
	mode_t process_mask;

	if (chnDefTbl)
	{
		chnStr = g_strdup_printf("%s:%s", trc->data.head.sensorID, trc->data.head.channel);
		chnDef = g_hash_table_lookup(chnDefTbl, chnStr);
		g_free(chnStr);
	}
	else
	{
		chnDef = NULL;
	}
	
	// initialize the header
	memcpy(&SH, &sac_null, sizeof(struct sac));
	
	// fill in the values that we can
	SH.leven = 1;
	SH.idep = IVEL;
	SH.nvhdr = 6;
	SH.npts = trc->data.dataInfo.head.numSamples;
	SH.delta = (float) (trc->data.dataInfo.head.sampInt/1000000.);
	SH.nzyear = trc->data.head.year;
	SH.nzjday = trc->data.head.day;
	SH.nzhour = trc->data.head.hour;
	SH.nzmin = trc->data.head.minute;
	SH.nzsec = trc->data.head.second;
	SH.nzmsec = trc->data.head.m_secs;
	SH.scale = 1;
	SH.b = 0;
	SH.e = trc->data.head.length + SH.delta;
	SH.iftype = ITIME;
	
	switch((int) chnDef)
	{
		case 0:		// no hash table entry found
			strncpy(SH.kstnm, trc->data.head.sensorID, 8);	// stn = sensor ID
			strncpy(SH.kcmpnm, trc->data.head.channel, 8);	// channel = stream:channel
			dir1 = g_strdup_printf("%s/%s", outDir, trc->data.head.sensorID);
			process_mask = umask(0);
			mkdir(dir1, S_IRWXU | S_IRWXG | S_IRWXO);
			umask(process_mask);
			// make the filename for later
			chan = g_strsplit(trc->data.head.channel, ":", -1);
			file1 = g_strdup_printf("%s_%s_%04d_%03d_%02d%02d%02d.sac",
						chan[0], chan[1],
						SH.nzyear, SH.nzjday, SH.nzhour, SH.nzmin, SH.nzsec);
			g_strfreev(chan);
		break;
		default:	// use hash table values
			strncpy(SH.knetwk, chnDef->ntw, 8);
			strncpy(SH.kstnm, chnDef->stn, 8);
			strncpy(SH.khole, chnDef->loc, 8);
			strncpy(SH.kcmpnm, chnDef->chn, 8);
			dir1 = g_strdup_printf("%s/%s.%s", outDir, chnDef->ntw, chnDef->stn);
			process_mask = umask(0);
			mkdir(dir1, S_IRWXU | S_IRWXG | S_IRWXO);
			umask(process_mask);
			// make the filename for later
			file1 = g_strdup_printf("%s_%s_%04d_%03d_%02d%02d%02d.sac",
						chnDef->loc, chnDef->chn,
						SH.nzyear, SH.nzjday, SH.nzhour, SH.nzmin, SH.nzsec);
		break;
	}
	dir2 = g_strdup_printf("%s/%04d.%03d", dir1, SH.nzyear, SH.nzjday);
	process_mask = umask(0);
	mkdir(dir2, S_IRWXU | S_IRWXG | S_IRWXO);
	umask(process_mask);
	g_free(dir1);

	fileName = g_strdup_printf("%s/%s", dir2, file1);
	g_free(dir2);	g_free(file1);
	if ((fp = fopen(fileName, "w")) == NULL)
	{
		free(fileName);
		return FAILURE;
	}
	g_free(fileName);
	
	// convert the data to float
	data = malloc(SH.npts * sizeof(float));
	for (i=0;i<SH.npts;i++)
		DATAP(dataFormat, i, data[i]);
	
	// write out SAC file
	if (fwrite(&SH, sizeof(struct sac), 1, fp) != 1)
	{
		fclose(fp);
		free(data);
		return FAILURE;
	}
	
	if (fwrite(data, sizeof(float), SH.npts, fp) != SH.npts)
	{
		fclose(fp);
		free(data);
		return FAILURE;
	}
	
	fclose(fp);
	free(data);
	return SUCCESS;
}

#define IDTAG		"ID"
#define SENSORTAG	"SENSOR"
#define STREAMTAG	"STREAM"
#define CHANNELTAG	"CHANNEL"

static gboolean readChnMap(char *chnMap)
{
	CHNDEF	*chnDef;
	ezxml_t	sensor, stream, channel;
	ezxml_t	chnMapXML;
	char	*sensorID, *streamID, *channelID;
	char	*ntw, *stn, *loc, *chn;
	char	*key;

	if (!chnMap)
		return TRUE;
	
	chnMapXML = ezxml_parse_file(chnMap);
	if (!chnMapXML)
	{
		fprintf(stderr, "Cannot open file %s for reading, quitting without doing anything, goodbye...\n",
						chnMap);
		return FALSE;
	}
	
	chnDefTbl = g_hash_table_new(g_str_hash, g_str_equal);
	
	for (sensor = ezxml_child(chnMapXML, SENSORTAG);
		 sensor;
		 sensor = ezxml_next(sensor))
	{
		sensorID = g_strdup(ezxml_attr(sensor, IDTAG));
		ntw = g_strdup((char *) ezxml_child(sensor, "NTW")->txt);
		stn = g_strdup((char *) ezxml_child(sensor, "STN")->txt);
		for(stream = ezxml_child(sensor, STREAMTAG);
			stream;
			stream = ezxml_next(stream))
		{
			streamID = g_strdup((char *) ezxml_attr(stream, IDTAG));
			loc = g_strdup((char *) ezxml_child(stream, "LOC")->txt);
			for(channel = ezxml_child(stream, CHANNELTAG);
				channel;
				channel = ezxml_next(channel))
			{
				channelID = g_strdup((char *) ezxml_attr(channel, IDTAG));
				chn = g_strdup((char *) ezxml_attr(channel, "NAME"));
				// make our hash table entry
				key = g_strdup_printf("%s:%s:%s", sensorID, streamID, channelID);
				chnDef = calloc(1, sizeof(CHNDEF));
				chnDef->ntw = ntw;
				chnDef->stn = stn;
				chnDef->loc = loc;
				chnDef->chn = chn;
				g_hash_table_insert(chnDefTbl, key, chnDef);
			}
		}
	}
	ezxml_free(chnMapXML);
	return TRUE;
}

static void printHelp(char *progName)
{
	fprintf(stderr, "Usage:\n\t%s [-h] --inDir input_directory "\
						"--outDir output_directory " \
						"[--chnDef channel_def_file ]\n",
					progName);
}

int main(int argc, char **argv)
{
	char 	*file, curDir[FILENAME_MAX], execDir[FILENAME_MAX], *chnMap=NULL, *count;
	GSList	*traces, *fileIter, *tracesIter;
	traceInfo	*trc;
	int		read, write, i, cnt, backSpace;
	int		option_index = 0, opt;
	static struct option long_options[] = {
		{"inDir", 1, 0, 0},
		{"outDir", 1, 0, 0},
		{"chnDef", 1, 0, 0},
		{0, 0, 0, 0}
	};
	
	while(1) 
	{
		option_index = 0;
		opt = getopt_long(argc, argv, "h", long_options, &option_index);
		if (opt == -1)
			break;
		
		switch(opt)
		{
			case 0:
				switch(option_index)
				{
					case 0:
						inDir = g_strdup(optarg);
					break;
					case 1:
						outDir = g_strdup(optarg);
					break;
					case 2:
						chnMap = g_strdup(optarg);
					break;
				}
			break;
		  
			case 'h':
				printHelp(argv[0]);
				exit(0);
			break;
		}
	}
	
	if (!inDir || !outDir)
	{
		printHelp(argv[0]);
		exit(-1);
	}

	// convert directories to full pathnames
	getcwd(execDir, sizeof(execDir));

	if (chdir(inDir) != 0)
	{
		fprintf(stderr, "Directory %s invalid, Quitting without doing anything, goodbye...\n",
					inDir);
		exit(-1);
	}
	getcwd(curDir, sizeof(curDir));
	free(inDir);	inDir = g_strdup(curDir);
	
	// reset to executing directory
	chdir(execDir);

	if (chdir(outDir) != 0)
	{
		fprintf(stderr, "Directory %s invalid, Quitting without doing anything, goodbye...\n",
					outDir);
		exit(-1);
	}
	getcwd(curDir, sizeof(curDir));
	free(outDir);	outDir = g_strdup(curDir);
	// reset to executing directory
	chdir(execDir);
	
	// read channel mapping file if provided
	if (!readChnMap(chnMap))
		exit(-1);
	
	// indentify all the files on disk to be read for conversion
	fprintf(stdout, "%s: version %s\n", argv[0], REF2SAC_VER);
	fprintf(stdout, "Identifying Reftek Data Files for Processing...\t");
	getFiles(inDir);
	fprintf(stdout, "done\n");
	
	fprintf(stdout, "Creating SAC Trace Files...\n");
	for (fileIter = fileList, cnt = 1;
		 fileIter;
		 fileIter = g_slist_next(fileIter))
	{
		file = fileIter->data;
		// convert the file's contents to individual traces
		traces = identifyTrace(file);

		// loop over all traces, converting each to SAC
		for (tracesIter = traces;
			 tracesIter;
			 tracesIter = g_slist_next(tracesIter), cnt++)
		{
			trc = tracesIter->data;
			read = readTrace(trc);
			if (read != SUCCESS)
				continue;
			write = writeSAC(trc);
			if (write != SUCCESS)
				fprintf(stderr, "SAC file Write FAILED!\n");
			freeData2(trc);
			if (cnt==1)
			{
				fprintf(stdout, "  Files Created:...\t");
				fflush(stdout);
				backSpace = 0;
			}
			if (!(cnt%50))
			{
				for(i = backSpace;
					i;
					i--)
					fprintf(stdout, "\b");
				fprintf(stdout, "%d", cnt);
				fflush(stdout);
				count = g_strdup_printf("%d", cnt);
				backSpace = strlen(count);
				g_free(count);
			}
			free(trc);
		}
		g_slist_free(traces);
		traces = NULL;
	}
	for(i = backSpace;
		i;
		i--)
		fprintf(stdout, "\b");
	fprintf(stdout, "\n\tTotal Files:\t%d", cnt);
	fprintf(stdout, "\n...done\n");
	
	exit(0);
}
